@@ -31,7 +31,7 @@
 
 #include <errno.h>
 #include <string.h>
-#ifndef G_OS_WIN32
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <stdio.h>
@@ -68,7 +68,7 @@ int main (int argc, char **argv)
   PangoFontFamily** families = NULL;
   PangoFontFace** faces = NULL;
   int nb, i;
-  gchar* family_name = NULL;
+  char *family_name = NULL;
   PangoLanguage *lang = pango_language_from_string (g_win32_getlocale ());
   HDC hdc = NULL;
   int line = 0;
