@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include "pango-break.h"
+#include "pango-item-private.h"
 
 #define DEV_RRA 0x0931 /* 0930 + 093c */
 #define DEV_QA 0x0958 /* 0915 + 093c */
@@ -110,7 +111,7 @@ break_indic (const char          *text,
 	     PangoLogAttr        *attrs,
 	     int                  attrs_len G_GNUC_UNUSED)
 {
-  const gchar *p, *next = NULL, *next_next;
+  const char *p, *next = NULL, *next_next;
   gunichar prev_wc, this_wc, next_wc, next_next_wc;
   gboolean is_conjunct = FALSE;
   int i;
@@ -144,7 +145,7 @@ break_indic (const char          *text,
 
       switch (analysis->script)
       {
-        case PANGO_SCRIPT_SINHALA:
+        case G_UNICODE_SCRIPT_SINHALA:
 	  /*
 	   * TODO: The cursor position should be based on the state table.
 	   *       This is the wrong place to be doing this.
