@@ -23,6 +23,8 @@
 #include <glib.h>
 #include "pango-glyph.h"
 #include "pango-font.h"
+#include "pango-item-private.h"
+#include "pango-glyph-item-private.h"
 #include "pango-impl-utils.h"
 
 #include <hb-ot.h>
@@ -56,7 +58,7 @@ pango_glyph_string_new (void)
  * Resize a glyph string to the given length.
  */
 void
-pango_glyph_string_set_size (PangoGlyphString *string, gint new_len)
+pango_glyph_string_set_size (PangoGlyphString *string, int new_len)
 {
   g_return_if_fail (new_len >= 0);
 
@@ -69,7 +71,7 @@ pango_glyph_string_set_size (PangoGlyphString *string, gint new_len)
       else
 	{
 	  const guint max_space =
-	    MIN (G_MAXINT, G_MAXSIZE / MAX (sizeof(PangoGlyphInfo), sizeof(gint)));
+	    MIN (G_MAXINT, G_MAXSIZE / MAX (sizeof(PangoGlyphInfo), sizeof(int)));
 
 	  guint more_space = (guint)string->space * 2;
 
@@ -89,7 +91,7 @@ pango_glyph_string_set_size (PangoGlyphString *string, gint new_len)
     }
 
   string->glyphs = g_realloc (string->glyphs, string->space * sizeof (PangoGlyphInfo));
-  string->log_clusters = g_realloc (string->log_clusters, string->space * sizeof (gint));
+  string->log_clusters = g_realloc (string->log_clusters, string->space * sizeof (int));
   string->num_glyphs = new_len;
 }
 
@@ -120,7 +122,7 @@ pango_glyph_string_copy (PangoGlyphString *string)
   new_string->glyphs = g_memdup2 (string->glyphs,
                                   string->space * sizeof (PangoGlyphInfo));
   new_string->log_clusters = g_memdup2 (string->log_clusters,
-                                        string->space * sizeof (gint));
+                                        string->space * sizeof (int));
 
   return new_string;
 }
@@ -296,8 +298,6 @@ pango_glyph_string_extents (PangoGlyphString *glyphs,
  * @geometry.width for each glyph in the @glyphs.
  *
  * Return value: the logical width of the glyph string.
- *
- * Since: 1.14
  */
 int
 pango_glyph_string_get_width (PangoGlyphString *glyphs)
@@ -317,32 +317,29 @@ pango_glyph_string_get_width (PangoGlyphString *glyphs)
  * @text: the text corresponding to the glyphs
  * @length: the length of @text, in bytes
  * @embedding_level: the embedding level of the string
- * @logical_widths: (array) (out caller-allocates): an array whose length is
- *   the number of characters in text (equal to `g_utf8_strlen (text, length)`
- *   unless text has `NUL` bytes) to be filled in with the resulting character
- *   widths.
+ * @logical_widths: (array): an array whose length is the number of
+ *   characters in text (equal to `g_utf8_strlen (text, length)` unless
+ *   text has `NUL` bytes) to be filled in with the resulting character widths.
  *
  * Given a `PangoGlyphString` and corresponding text, determine the width
  * corresponding to each character.
  *
  * When multiple characters compose a single cluster, the width of the
  * entire cluster is divided equally among the characters.
- *
- * See also [method@Pango.GlyphItem.get_logical_widths].
  */
 void
 pango_glyph_string_get_logical_widths (PangoGlyphString *glyphs,
-				       const char       *text,
-				       int               length,
-				       int               embedding_level,
-				       int              *logical_widths)
+                                       const char       *text,
+                                       int               length,
+                                       int               embedding_level,
+                                       int              *logical_widths)
 {
   /* Build a PangoGlyphItem and call the other API */
-  PangoItem item = {0, length, pango_utf8_strlen (text, length),
-		    {NULL, NULL, NULL,
-		     embedding_level, PANGO_GRAVITY_AUTO, 0,
-		     PANGO_SCRIPT_UNKNOWN, NULL,
-		     NULL}};
+  PangoItem item = {0, length, pango_utf8_strlen (text, length), 0,
+                    {NULL, NULL,
+                     embedding_level, PANGO_GRAVITY_AUTO, 0,
+                     G_UNICODE_SCRIPT_UNKNOWN, NULL,
+                     NULL}};
   PangoGlyphItem glyph_item = {&item, glyphs};
 
   pango_glyph_item_get_logical_widths (&glyph_item, text, logical_widths);
@@ -366,7 +363,7 @@ pango_glyph_string_get_logical_widths (PangoGlyphString *glyphs,
  * @index_: the byte index within @text
  * @trailing: whether we should compute the result for the beginning (%FALSE)
  *   or end (%TRUE) of the character.
- * @x_pos: (out) (optional): location to store result
+ * @x_pos: (out): location to store result
  *
  * Converts from character position to x position.
  *
@@ -381,13 +378,13 @@ pango_glyph_string_get_logical_widths (PangoGlyphString *glyphs,
  * </picture>
  */
 void
-pango_glyph_string_index_to_x (PangoGlyphString *glyphs,
-                               const char       *text,
-                               int               length,
-                               PangoAnalysis    *analysis,
-                               int               index,
-                               gboolean          trailing,
-                               int              *x_pos)
+pango_glyph_string_index_to_x (PangoGlyphString    *glyphs,
+                               const char          *text,
+                               int                  length,
+                               const PangoAnalysis *analysis,
+                               int                  index,
+                               gboolean             trailing,
+                               int                 *x_pos)
 {
   pango_glyph_string_index_to_x_full (glyphs,
                                       text, length,
@@ -407,7 +404,7 @@ pango_glyph_string_index_to_x (PangoGlyphString *glyphs,
  * @index_: the byte index within @text
  * @trailing: whether we should compute the result for the beginning (%FALSE)
  *   or end (%TRUE) of the character.
- * @x_pos: (out) (optional): location to store result
+ * @x_pos: (out): location to store result
  *
  * Converts from character position to x position.
  *
@@ -415,18 +412,16 @@ pango_glyph_string_index_to_x (PangoGlyphString *glyphs,
  * accepts a `PangoLogAttr` array. The grapheme boundary information
  * in it can be used to disambiguate positioning inside some complex
  * clusters.
- *
- * Since: 1.50
  */
 void
-pango_glyph_string_index_to_x_full (PangoGlyphString *glyphs,
-                                    const char       *text,
-                                    int               length,
-                                    PangoAnalysis    *analysis,
-                                    PangoLogAttr     *attrs,
-                                    int               index,
-                                    gboolean          trailing,
-                                    int              *x_pos)
+pango_glyph_string_index_to_x_full (PangoGlyphString    *glyphs,
+                                    const char          *text,
+                                    int                  length,
+                                    const PangoAnalysis *analysis,
+                                    PangoLogAttr        *attrs,
+                                    int                  index,
+                                    gboolean             trailing,
+                                    int                 *x_pos)
 {
   int i;
   int start_xpos = 0;
@@ -549,7 +544,7 @@ pango_glyph_string_index_to_x_full (PangoGlyphString *glyphs,
     }
 
   if (trailing)
-    cluster_offset = MIN (cluster_offset + 1, cluster_chars);
+    cluster_offset += 1;
 
   if (G_UNLIKELY (!cluster_chars)) /* pedantic */
      {
@@ -630,11 +625,9 @@ fallback:
  * @length: the number of bytes (not characters) in text.
  * @analysis: the analysis information return from [func@itemize]
  * @x_pos: the x offset (in Pango units)
- * @index_: (out) (optional): location to store calculated byte index
- *   within @text
- * @trailing: (out) (optional): location to store a boolean indicating
- *   whether the user clicked on the leading or trailing edge of the
- *   character
+ * @index_: (out): location to store calculated byte index within @text
+ * @trailing: (out): location to store a boolean indicating whether the
+ *   user clicked on the leading or trailing edge of the character
  *
  * Convert from x offset to character position.
  *
@@ -645,13 +638,13 @@ fallback:
  * attributes for the text to compute the valid cursor position.
  */
 void
-pango_glyph_string_x_to_index (PangoGlyphString *glyphs,
-                               const char       *text,
-                               int               length,
-                               PangoAnalysis    *analysis,
-                               int               x_pos,
-                               int              *index,
-                               gboolean         *trailing)
+pango_glyph_string_x_to_index (PangoGlyphString    *glyphs,
+                               const char          *text,
+                               int                  length,
+                               const PangoAnalysis *analysis,
+                               int                  x_pos,
+                               int                 *index,
+                               gboolean            *trailing)
 {
   int i;
   int start_xpos = 0;
