@@ -31,8 +31,10 @@
 #include "pango-fontmap-private.h"
 #include "pango-script-private.h"
 #include "pango-emoji-private.h"
-#include "pango-attributes-private.h"
+#include "pango-attr-iterator-private.h"
+#include "pango-attr-private.h"
 #include "pango-item-private.h"
+#include "pango-bidi-private.h"
 
 #include <hb-ot.h>
 
@@ -133,11 +135,11 @@ typedef struct _PangoWidthIter PangoWidthIter;
 
 struct _PangoWidthIter
 {
-        const gchar *text_start;
-        const gchar *text_end;
-        const gchar *start;
-        const gchar *end;
-        gboolean     upright;
+        const char *text_start;
+        const char *text_end;
+        const char *start;
+        const char *end;
+        gboolean upright;
 };
 
 static gboolean
@@ -315,7 +317,7 @@ struct _ItemizeState
 
   PangoScriptIter script_iter;
   const char *script_end;
-  PangoScript script;
+  GUnicodeScript script;
 
   PangoWidthIter width_iter;
   PangoEmojiIter emoji_iter;
@@ -352,7 +354,7 @@ find_attribute (GSList        *attr_list,
   GSList *node;
 
   for (node = attr_list; node; node = node->next)
-    if (((PangoAttribute *) node->data)->klass->type == type)
+    if (((PangoAttribute *) node->data)->type == type)
       return (PangoAttribute *) node->data;
 
   return NULL;
@@ -394,13 +396,13 @@ update_attr_iterator (ItemizeState *state)
     state->lang = state->context->language;
 
   attr = find_attribute (state->extra_attrs, PANGO_ATTR_FALLBACK);
-  state->enable_fallback = (attr == NULL || ((PangoAttrInt *)attr)->value);
+  state->enable_fallback = (attr == NULL || attr->int_value);
 
   attr = find_attribute (state->extra_attrs, PANGO_ATTR_GRAVITY);
-  state->gravity = attr == NULL ? PANGO_GRAVITY_AUTO : ((PangoAttrInt *)attr)->value;
+  state->gravity = attr == NULL ? PANGO_GRAVITY_AUTO : attr->int_value;
 
   attr = find_attribute (state->extra_attrs, PANGO_ATTR_GRAVITY_HINT);
-  state->gravity_hint = attr == NULL ? state->context->gravity_hint : (PangoGravityHint)((PangoAttrInt *)attr)->value;
+  state->gravity_hint = attr == NULL ? state->context->gravity_hint : (PangoGravityHint)attr->int_value;
 
   state->changed |= FONT_CHANGED;
   if (state->lang != old_lang)
@@ -720,8 +722,6 @@ itemize_state_add_character (ItemizeState *state,
 }
 
 typedef struct {
-  PangoLanguage *lang;
-  gunichar wc;
   PangoFont *font;
   int position;
 } GetFontInfo;
@@ -733,20 +733,8 @@ get_font_foreach (PangoFontset *fontset,
 {
   GetFontInfo *info = data;
 
-  if (G_UNLIKELY (!font))
-    return FALSE;
-
-  if (pango_font_has_char (font, info->wc))
-    {
-      info->font = font;
-      return TRUE;
-    }
-
-  if (!fontset)
-    {
-      info->font = font;
-      return TRUE;
-    }
+  if (font == info->font)
+    return TRUE;
 
   info->position++;
 
@@ -777,13 +765,16 @@ get_font (ItemizeState  *state,
   if (state->enable_fallback && font_cache_get (state->cache, wc, font, position))
     return TRUE;
 
-  info.lang = state->derived_lang;
-  info.wc = wc;
   info.font = NULL;
   info.position = 0;
 
   if (state->enable_fallback)
-    pango_fontset_foreach (state->current_fonts, get_font_foreach, &info);
+    {
+      info.font = pango_fontset_get_font (state->current_fonts, wc);
+      if (info.font)
+        g_object_unref (info.font);
+      pango_fontset_foreach (state->current_fonts, get_font_foreach, &info);
+    }
 
   if (!info.font)
     info.font = get_base_font (state);
@@ -800,7 +791,7 @@ get_font (ItemizeState  *state,
 
 static PangoLanguage *
 compute_derived_language (PangoLanguage *lang,
-                          PangoScript    script)
+                          GUnicodeScript script)
 {
   PangoLanguage *derived_lang;
 
@@ -944,6 +935,7 @@ itemize_state_process_run (ItemizeState *state)
   const char *p;
   gboolean last_was_forced_break = FALSE;
   gboolean is_space;
+  gunichar prev_wc = 0;
 
   /* Only one character has type G_UNICODE_LINE_SEPARATOR in Unicode 4.0;
    * update this if that changes. */
@@ -959,7 +951,8 @@ itemize_state_process_run (ItemizeState *state)
        p = g_utf8_next_char (p))
     {
       gunichar wc = g_utf8_get_char (p);
-      gboolean is_forced_break = (wc == '\t' || wc == LINE_SEPARATOR);
+      gboolean is_forced_break = wc == '\t' || wc == '\r' || wc == '\n' ||
+                                 wc == 0x2028 || wc == 0x2029;
       PangoFont *font;
       int font_position;
 
@@ -975,12 +968,17 @@ itemize_state_process_run (ItemizeState *state)
           is_space = FALSE;
         }
 
-      itemize_state_add_character (state, font, font_position,
-                                   is_forced_break || last_was_forced_break,
-                                   p,
-                                   is_space);
+      /* Don't break between \r and \n */
+      if (prev_wc == '\r' && wc == '\n')
+        state->item->num_chars++;
+      else
+        itemize_state_add_character (state, font, font_position,
+                                     is_forced_break || last_was_forced_break,
+                                     p,
+                                     is_space);
 
       last_was_forced_break = is_forced_break;
+      prev_wc = wc;
     }
 
   /* Finish the final item from the current segment */
@@ -1057,7 +1055,7 @@ collect_font_scale (PangoContext  *context,
     {
       PangoAttribute *attr = l->data;
 
-      if (attr->klass->type == PANGO_ATTR_FONT_SCALE)
+      if (attr->type == PANGO_ATTR_FONT_SCALE)
         {
           if (attr->start_index == item->offset)
             {
@@ -1071,7 +1069,7 @@ collect_font_scale (PangoContext  *context,
               entry->attr = attr;
               *stack = g_list_prepend (*stack, entry);
 
-              switch (((PangoAttrInt *)attr)->value)
+              switch (attr->int_value)
                 {
                 case PANGO_FONT_SCALE_NONE:
                   break;
@@ -1132,7 +1130,7 @@ collect_font_scale (PangoContext  *context,
      {
        ScaleItem *entry = l->data;
        *scale *= entry->scale;
-       if (((PangoAttrInt *)entry->attr)->value != PANGO_FONT_SCALE_SMALL_CAPS)
+       if (entry->attr->int_value != PANGO_FONT_SCALE_SMALL_CAPS)
          *is_small_caps = FALSE;
        retval = TRUE;
      }
@@ -1296,10 +1294,17 @@ variant_supported (PangoItem    *item,
 static PangoVariant
 get_font_variant (PangoItem *item)
 {
-  if (item->analysis.font)
-    return pango_font_get_variant (item->analysis.font);
+  PangoFontDescription *desc;
+  PangoVariant variant = PANGO_VARIANT_NORMAL;
 
-  return PANGO_VARIANT_NORMAL;
+  if (item->analysis.font)
+    {
+      desc = pango_font_describe (item->analysis.font);
+      variant = pango_font_description_get_variant (desc);
+      pango_font_description_free (desc);
+    }
+
+  return variant;
 }
 
 static PangoTextTransform
@@ -1312,8 +1317,8 @@ find_text_transform (const PangoAnalysis *analysis)
     {
       PangoAttribute *attr = l->data;
 
-      if (attr->klass->type == PANGO_ATTR_TEXT_TRANSFORM)
-        transform = (PangoTextTransform) ((PangoAttrInt*)attr)->value;
+      if (attr->type == PANGO_ATTR_TEXT_TRANSFORM)
+        transform = (PangoTextTransform) attr->int_value;
     }
 
   return transform;
@@ -1366,7 +1371,7 @@ split_item_for_variant (const char   *text,
 
   start = text + item->offset;
   end = start + item->length;
-  offset = ((PangoItemPrivate *)item)->char_offset;
+  offset = item->char_offset;
 
   p = start;
   while (p < end)
@@ -1504,7 +1509,7 @@ reorder_items (PangoContext *context,
   /* Also cmpute the char offset for each item here */
   for (GList *l = items; l; l = l->next)
     {
-      PangoItemPrivate *item = l->data;
+      PangoItem *item = l->data;
       item->char_offset = char_offset;
       char_offset += item->num_chars;
     }
@@ -1527,8 +1532,8 @@ post_process_items (PangoContext *context,
 /* }}} */
 /* {{{ Private API */
 
-/* Like pango_itemize_with_base_dir, but takes a font description.
- * In contrast to pango_itemize_with_base_dir, this function does
+/* Like pango_itemize, but takes a font description.
+ * In contrast to pango_itemize, this function does
  * not call pango_itemize_post_process_items, so you need to do that
  * separately, after applying attributes that affect segmentation and
  * computing the log attrs.
@@ -1577,7 +1582,7 @@ pango_itemize_post_process_items (PangoContext *context,
 /* {{{ Public API */
 
 /**
- * pango_itemize_with_base_dir:
+ * pango_itemize:
  * @context: a structure holding information that affects
  *   the itemization process.
  * @base_dir: base direction to use for bidirectional processing
@@ -1586,9 +1591,13 @@ pango_itemize_post_process_items (PangoContext *context,
  * @length: the number of bytes (not characters) to process
  *   after @start_index. This must be >= 0.
  * @attrs: the set of attributes that apply to @text.
- * @cached_iter: (nullable): Cached attribute iterator
  *
- * Like `pango_itemize()`, but with an explicitly specified base direction.
+ * Breaks a piece of text into segments with consistent directional
+ * level and font.
+ *
+ * Each byte of @text will be contained in exactly one of the items in the
+ * returned list; the generated list of items will be in logical order (the
+ * start offsets of the items are ascending).
  *
  * The base direction is used when computing bidirectional levels.
  * [func@itemize] gets the base direction from the `PangoContext`
@@ -1597,17 +1606,14 @@ pango_itemize_post_process_items (PangoContext *context,
  * Return value: (transfer full) (element-type Pango.Item): a `GList` of
  *   [struct@Pango.Item] structures. The items should be freed using
  *   [method@Pango.Item.free] probably in combination with [func@GLib.List.free_full].
- *
- * Since: 1.4
  */
 GList *
-pango_itemize_with_base_dir (PangoContext      *context,
-                             PangoDirection     base_dir,
-                             const char        *text,
-                             int                start_index,
-                             int                length,
-                             PangoAttrList     *attrs,
-                             PangoAttrIterator *cached_iter)
+pango_itemize (PangoContext      *context,
+               PangoDirection     base_dir,
+               const char        *text,
+               int                start_index,
+               int                length,
+               PangoAttrList     *attrs)
 {
   GList *items;
 
@@ -1618,58 +1624,11 @@ pango_itemize_with_base_dir (PangoContext      *context,
 
   items = pango_itemize_with_font (context, base_dir,
                                    text, start_index, length,
-                                   attrs, cached_iter,
-                                   NULL);
+                                   attrs, NULL, NULL);
 
   return pango_itemize_post_process_items (context, text, NULL, items);
 }
 
-/**
- * pango_itemize:
- * @context: a structure holding information that affects
- *   the itemization process.
- * @text: the text to itemize. Must be valid UTF-8
- * @start_index: first byte in @text to process
- * @length: the number of bytes (not characters) to process
- *   after @start_index. This must be >= 0.
- * @attrs: the set of attributes that apply to @text.
- * @cached_iter: (nullable): Cached attribute iterator
- *
- * Breaks a piece of text into segments with consistent directional
- * level and font.
- *
- * Each byte of @text will be contained in exactly one of the items in the
- * returned list; the generated list of items will be in logical order (the
- * start offsets of the items are ascending).
- *
- * @cached_iter should be an iterator over @attrs currently positioned
- * at a range before or containing @start_index; @cached_iter will be
- * advanced to the range covering the position just after
- * @start_index + @length. (i.e. if itemizing in a loop, just keep passing
- * in the same @cached_iter).
- *
- * Return value: (transfer full) (element-type Pango.Item): a `GList` of
- *   [struct@Pango.Item] structures. The items should be freed using
- *   [method@Pango.Item.free] in combination with [func@GLib.List.free_full].
- */
-GList *
-pango_itemize (PangoContext      *context,
-               const char        *text,
-               int                start_index,
-               int                length,
-               PangoAttrList     *attrs,
-               PangoAttrIterator *cached_iter)
-{
-  g_return_val_if_fail (context != NULL, NULL);
-  g_return_val_if_fail (start_index >= 0, NULL);
-  g_return_val_if_fail (length >= 0, NULL);
-  g_return_val_if_fail (length == 0 || text != NULL, NULL);
-
-  return pango_itemize_with_base_dir (context, context->base_dir,
-                                      text, start_index, length,
-                                      attrs, cached_iter);
-}
-
 /* }}} */
 
 /* vim:set foldmethod=marker expandtab: */
