@@ -1,32 +1,29 @@
-/* Pango
- * pango-break.h:
- *
+/*
  * Copyright (C) 1999 Red Hat Software
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __PANGO_BREAK_H__
-#define __PANGO_BREAK_H__
+#pragma once
 
 #include <glib.h>
+#include <pango/pango-item.h>
 
 G_BEGIN_DECLS
 
-#include <pango/pango-item.h>
 
 /* Logical attributes of a character.
  */
@@ -62,20 +59,19 @@ G_BEGIN_DECLS
  *   on grapheme boundaries (where @is_cursor_position is set). In some languages,
  *   the full grapheme (e.g. letter + diacritics) is considered a unit, while in
  *   others, each decomposed character in the grapheme is a unit. In the default
- *   implementation of [func@break], this bit is set on all grapheme boundaries
- *   except those following Latin, Cyrillic or Greek base characters.
+ *   implementation of [func@default_break], this bit is set on all grapheme
+ *   boundaries except those following Latin, Cyrillic or Greek base characters.
  * @is_expandable_space: is a whitespace character that can possibly be
- *   expanded for justification purposes. (Since: 1.18)
+ *   expanded for justification purposes.
  * @is_word_boundary: is a word boundary, as defined by UAX#29.
  *   More specifically, means that this is not a position in the middle of a word.
  *   For example, both sides of a punctuation mark are considered word boundaries.
  *   This flag is particularly useful when selecting text word-by-word. This flag
  *   implements Unicode's [Word Boundaries](http://www.unicode.org/reports/tr29/)
- *   semantics. (Since: 1.22)
+ *   semantics.
  * @break_inserts_hyphen: when breaking lines before this char, insert a hyphen.
- *   Since: 1.50
  * @break_removes_preceding: when breaking lines before this char, remove the
- *   preceding char. Since 1.50
+ *   preceding char.
  *
  * The `PangoLogAttr` structure stores information about the attributes of a
  * single character.
@@ -97,18 +93,14 @@ struct _PangoLogAttr
   guint is_word_boundary            : 1;
   guint break_inserts_hyphen        : 1;
   guint break_removes_preceding     : 1;
-};
 
-PANGO_DEPRECATED_IN_1_44
-void                    pango_break             (const char    *text,
-                                                 int            length,
-                                                 PangoAnalysis *analysis,
-                                                 PangoLogAttr  *attrs,
-                                                 int            attrs_len);
+  guint reserved                    : 17;
+};
 
 PANGO_AVAILABLE_IN_ALL
 void                    pango_get_log_attrs     (const char    *text,
                                                  int            length,
+                                                 PangoAttrList *attr_list,
                                                  int            level,
                                                  PangoLanguage *language,
                                                  PangoLogAttr  *attrs,
@@ -117,11 +109,10 @@ void                    pango_get_log_attrs     (const char    *text,
 PANGO_AVAILABLE_IN_ALL
 void                    pango_default_break     (const char    *text,
                                                  int            length,
-                                                 PangoAnalysis *analysis,
                                                  PangoLogAttr  *attrs,
                                                  int            attrs_len);
 
-PANGO_AVAILABLE_IN_1_44
+PANGO_AVAILABLE_IN_ALL
 void                    pango_tailor_break      (const char    *text,
                                                  int            length,
                                                  PangoAnalysis *analysis,
@@ -129,7 +120,7 @@ void                    pango_tailor_break      (const char    *text,
                                                  PangoLogAttr  *attrs,
                                                  int            attrs_len);
 
-PANGO_AVAILABLE_IN_1_50
+PANGO_AVAILABLE_IN_ALL
 void                    pango_attr_break        (const char    *text,
                                                  int            length,
                                                  PangoAttrList *attr_list,
@@ -138,5 +129,3 @@ void                    pango_attr_break        (const char    *text,
                                                  int            attrs_len);
 
 G_END_DECLS
-
-#endif /* __PANGO_BREAK_H__ */
