@@ -10,7 +10,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -26,7 +26,7 @@
 
 #include "pango-attributes.h"
 #include "pango-impl-utils.h"
-#include "pango-utils-internal.h"
+
 
 G_DEFINE_BOXED_TYPE (PangoColor, pango_color,
                      pango_color_copy,
@@ -34,18 +34,18 @@ G_DEFINE_BOXED_TYPE (PangoColor, pango_color,
 
 /**
  * pango_color_copy:
- * @src: (nullable): color to copy, may be %NULL
+ * @src: (nullable): color to copy
  *
  * Creates a copy of @src.
  *
- * The copy should be freed with pango_color_free(). Primarily
- * used by language bindings, not that useful otherwise (since
- * colors can just be copied by assignment in C).
+ * The copy should be freed with [method@Pango.Color.free].
+ * Primarily used by language bindings, not that useful
+ * otherwise (since colors can just be copied by assignment
+ * in C).
  *
- * Return value: (nullable): the newly allocated `PangoColor`, which
- *   should be freed with [method@Pango.Color.free], or %NULL if
- *   @src was %NULL.
- **/
+ * Return value: (nullable): the newly allocated `PangoColor`,
+ *   which should be freed with [method@Pango.Color.free]
+ */
 PangoColor*
 pango_color_copy (const PangoColor *src)
 {
@@ -63,10 +63,10 @@ pango_color_copy (const PangoColor *src)
 
 /**
  * pango_color_free:
- * @color: (nullable): an allocated #PangoColor, may be %NULL
+ * @color: (nullable): an allocated `PangoColor`
  *
- * Frees a color allocated by pango_color_copy().
- **/
+ * Frees a color allocated by [method@Pango.Color.copy].
+ */
 void
 pango_color_free (PangoColor *color)
 {
@@ -76,26 +76,50 @@ pango_color_free (PangoColor *color)
   g_slice_free (PangoColor, color);
 }
 
+/**
+ * pango_color_equal:
+ * @color1: (nullable): a `PangoColor`
+ * @color2: (nullable): another `PangoColor`
+ *
+ * Compares two colors for quality.
+ *
+ * Returns: `TRUE` if the colors are equal
+ */
+gboolean
+pango_color_equal (const PangoColor *color1,
+                   const PangoColor *color2)
+{
+  return color1 == color2 ||
+         (color1 && color2 &&
+          color1->red == color2->red &&
+          color1->green == color2->green &&
+          color1->blue == color2->blue &&
+          color1->alpha == color2->alpha);
+}
+
 /**
  * pango_color_to_string:
- * @color: a #PangoColor
+ * @color: a `PangoColor`
  *
  * Returns a textual specification of @color.
  *
- * The string is in the hexadecimal form `#rrrrggggbbbb`, where
- * `r`, `g` and `b` are hex digits representing the red, green,
- * and blue components respectively.
+ * The string is in the hexadecimal form `#rrrrggggbbbbaaaa`,
+ * where `r`, `g`, `b` and `a` are hex digits representing the
+ * red, green, blue and alpha components respectively.
  *
- * Return value: a newly-allocated text string that must be freed with g_free().
- *
- * Since: 1.16
- **/
-gchar *
+ * Return value: a newly-allocated text string that must
+ *   be freed with [GLib.free]
+ */
+char *
 pango_color_to_string (const PangoColor *color)
 {
   g_return_val_if_fail (color != NULL, NULL);
 
-  return g_strdup_printf ("#%04x%04x%04x", color->red, color->green, color->blue);
+  return g_strdup_printf ("#%04x%04x%04x%04x",
+                          color->red,
+                          color->green,
+                          color->blue,
+                          color->alpha);
 }
 
 /* Color parsing
@@ -161,25 +185,25 @@ compare_xcolor_entries (const void *a, const void *b)
       int c1, c2;
       while (*s1 == ' ') s1++;
       while (*s2 == ' ') s1++;
-      c1 = (gint)(guchar) TOLOWER (*s1);
-      c2 = (gint)(guchar) TOLOWER (*s2);
+      c1 = (int)(guchar) TOLOWER (*s1);
+      c2 = (int)(guchar) TOLOWER (*s2);
       if (c1 != c2)
         return (c1 - c2);
       s1++; s2++;
     }
 
-  return ((gint) *s1) - ((gint) *s2);
+  return ((int) *s1) - ((int) *s2);
 }
 
 static gboolean
 find_color(const char *name,
-	   PangoColor *color)
+           PangoColor *color)
 {
   ColorEntry *found;
 
   found = bsearch (name, color_entries, G_N_ELEMENTS (color_entries),
-		   sizeof (ColorEntry),
-		   compare_xcolor_entries);
+                   sizeof (ColorEntry),
+                   compare_xcolor_entries);
   if (found == NULL)
     return FALSE;
 
@@ -210,40 +234,36 @@ hex (const char *spec,
 
 
 /**
- * pango_color_parse_with_alpha:
- * @color: (nullable): a `PangoColor` structure in which to store the
- *   result, or %NULL
- * @alpha: (out) (optional): return location for alpha, or %NULL
+ * pango_color_parse:
+ * @color: a `PangoColor` structure in which to store the result
  * @spec: a string specifying the new color
  *
  * Fill in the fields of a color from a string specification.
  *
- * The string can
- * either one of a large set of standard names. (Taken from the CSS Color
- * [specification](https://www.w3.org/TR/css-color-4/#named-colors),
- * or it can be a hexadecimal value in the form `#rgb`, `#rrggbb`, `#rrrgggbbb`
- * or `#rrrrggggbbbb` where `r`, `g` and `b` are hex digits of the red, green,
- * and blue components of the color, respectively. (White in the four
- * forms is `#fff`, `#ffffff`, `#fffffffff` and `#ffffffffffff`.)
+ * The string can either one of a large set of standard names.
+ * (Taken from the CSS Color [specification](https://www.w3.org/TR/css-color-4/#named-colors),
+ * or it can be a hexadecimal value in the form `#rgb`,
+ * `#rrggbb`, `#rrrgggbbb` or `#rrrrggggbbbb` where `r`, `g`
+ * and `b` are hex digits of the red, green, and blue components
+ * of the color, respectively. (White in the four forms is
+ * `#fff`, `#ffffff`, `#fffffffff` and `#ffffffffffff`.)
  *
  * Additionally, parse strings of the form `#rgba`, `#rrggbbaa`,
- * `#rrrrggggbbbbaaaa`, if @alpha is not %NULL, and set @alpha to the value
- * specified by the hex digits for `a`. If no alpha component is found
- * in @spec, @alpha is set to 0xffff (for a solid color).
+ * `#rrrrggggbbbbaaaa`, and set the alpha component of @color
+ * to the value specified by the hex digits for `a`. If no alpha
+ * component is found in @spec, alpha is set to 0xffff (for a
+ * solid color).
  *
- * Return value: %TRUE if parsing of the specifier succeeded, otherwise false.
- *
- * Since: 1.46
+ * Return value: %TRUE if parsing of the specifier succeeded,
+ *   otherwise %FALSE
  */
 gboolean
-pango_color_parse_with_alpha (PangoColor *color,
-                              guint16    *alpha,
-                              const char *spec)
+pango_color_parse (PangoColor *color,
+                   const char *spec)
 {
   g_return_val_if_fail (spec != NULL, FALSE);
 
-  if (alpha)
-    *alpha = 0xffff;
+  color->alpha = 0xffff;
 
   if (spec[0] == '#')
     {
@@ -265,8 +285,6 @@ pango_color_parse_with_alpha (PangoColor *color,
         case 4:
         case 8:
         case 16:
-          if (!alpha)
-            return FALSE;
           len /= 4;
           has_alpha = TRUE;
           break;
@@ -298,7 +316,7 @@ pango_color_parse_with_alpha (PangoColor *color,
           color->blue  = b;
         }
 
-      if (alpha && has_alpha)
+      if (has_alpha)
         {
           int bits = len * 4;
           a <<= 16 - bits;
@@ -307,7 +325,7 @@ pango_color_parse_with_alpha (PangoColor *color,
               a |= (a >> bits);
               bits *= 2;
             }
-          *alpha = a;
+          color->alpha = a;
         }
     }
   else
@@ -317,28 +335,3 @@ pango_color_parse_with_alpha (PangoColor *color,
     }
   return TRUE;
 }
-
-/**
- * pango_color_parse:
- * @color: (nullable): a `PangoColor` structure in which to store the
- *   result, or %NULL
- * @spec: a string specifying the new color
- *
- * Fill in the fields of a color from a string specification.
- *
- * The string can either
- * one of a large set of standard names. (Taken from the CSS Color
- * [specification](https://www.w3.org/TR/css-color-4/#named-colors), or it can be
- * a value in the form `#rgb`, `#rrggbb`, `#rrrgggbbb` or `#rrrrggggbbbb`, where
- * `r`, `g` and `b` are hex digits of the red, green, and blue components of the
- * color, respectively. (White in the four forms is `#fff`, `#ffffff`, `#fffffffff`
- * and `#ffffffffffff`.)
- *
- * Return value: %TRUE if parsing of the specifier succeeded, otherwise false.
- **/
-gboolean
-pango_color_parse (PangoColor *color,
-                   const char *spec)
-{
-  return pango_color_parse_with_alpha (color, NULL, spec);
-}
