@@ -28,6 +28,7 @@
 #include "pango-impl-utils.h"
 
 #include "pango-font-private.h"
+#include "pango-font-metrics-private.h"
 #include "pango-item-private.h"
 #include "pango-fontset-private.h"
 #include "pango-fontmap-private.h"
@@ -44,7 +45,8 @@
  * to look up fonts, and default values such as the default language,
  * default gravity, or default font.
  *
- * To obtain a `PangoContext`, use [method@Pango.FontMap.create_context].
+ * To obtain a `PangoContext`, use [ctor@Pango.Context.new]
+ * or [func@Pango.cairo_create_context].
  */
 
 struct _PangoContextClass
@@ -56,7 +58,21 @@ struct _PangoContextClass
 static void pango_context_finalize    (GObject       *object);
 static void context_changed           (PangoContext  *context);
 
-G_DEFINE_TYPE (PangoContext, pango_context, G_TYPE_OBJECT)
+enum {
+  PROP_FONT_MAP = 1,
+  PROP_FONT_DESCRIPTION,
+  PROP_LANGUAGE,
+  PROP_BASE_DIR,
+  PROP_BASE_GRAVITY,
+  PROP_GRAVITY_HINT,
+  PROP_MATRIX,
+  PROP_ROUND_GLYPH_POSITIONS,
+  N_PROPERTIES
+};
+
+static GParamSpec *properties[N_PROPERTIES] = { NULL, };
+
+G_DEFINE_FINAL_TYPE (PangoContext, pango_context, G_TYPE_OBJECT)
 
 static void
 pango_context_init (PangoContext *context)
@@ -80,12 +96,234 @@ pango_context_init (PangoContext *context)
   pango_font_description_set_size (context->font_desc, 12 * PANGO_SCALE);
 }
 
+static gboolean
+pango_has_mixed_deps (void)
+{
+  GModule *module;
+  gpointer func;
+  gboolean result = FALSE;
+
+  module = g_module_open (NULL, 0);
+
+  if (g_module_symbol (module, "pango_ot_info_find_script", &func))
+    result = TRUE;
+
+  g_module_close (module);
+
+  return result;
+}
+
+static void
+pango_context_set_property (GObject      *object,
+                            guint         property_id,
+                            const GValue *value,
+                            GParamSpec   *pspec)
+{
+  PangoContext *context = PANGO_CONTEXT (object);
+
+  switch (property_id)
+    {
+    case PROP_FONT_MAP:
+      pango_context_set_font_map (context, g_value_get_object (value));
+      break;
+
+    case PROP_FONT_DESCRIPTION:
+      pango_context_set_font_description (context, g_value_get_boxed (value));
+      break;
+
+    case PROP_LANGUAGE:
+      pango_context_set_language (context, g_value_get_boxed (value));
+      break;
+
+    case PROP_BASE_DIR:
+      pango_context_set_base_dir (context, g_value_get_enum (value));
+      break;
+
+    case PROP_BASE_GRAVITY:
+      pango_context_set_base_gravity (context, g_value_get_enum (value));
+      break;
+
+    case PROP_GRAVITY_HINT:
+      pango_context_set_gravity_hint (context, g_value_get_enum (value));
+      break;
+
+    case PROP_MATRIX:
+      pango_context_set_matrix (context, g_value_get_boxed (value));
+      break;
+
+    case PROP_ROUND_GLYPH_POSITIONS:
+      pango_context_set_round_glyph_positions (context, g_value_get_boolean (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
+static void
+pango_context_get_property (GObject    *object,
+                            guint       property_id,
+                            GValue     *value,
+                            GParamSpec *pspec)
+{
+  PangoContext *context = PANGO_CONTEXT (object);
+
+  switch (property_id)
+    {
+    case PROP_FONT_MAP:
+      g_value_set_object (value, pango_context_get_font_map (context));
+      break;
+
+    case PROP_FONT_DESCRIPTION:
+      g_value_set_boxed (value, pango_context_get_font_description (context));
+      break;
+
+    case PROP_LANGUAGE:
+      g_value_set_boxed (value, pango_context_get_language (context));
+      break;
+
+    case PROP_BASE_DIR:
+      g_value_set_enum (value, pango_context_get_base_dir (context));
+      break;
+
+    case PROP_BASE_GRAVITY:
+      g_value_set_enum (value, pango_context_get_base_gravity (context));
+      break;
+
+    case PROP_GRAVITY_HINT:
+      g_value_set_enum (value, pango_context_get_gravity_hint (context));
+      break;
+
+    case PROP_MATRIX:
+      g_value_set_boxed (value, pango_context_get_matrix (context));
+      break;
+
+    case PROP_ROUND_GLYPH_POSITIONS:
+      g_value_set_boolean (value, pango_context_get_round_glyph_positions (context));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    }
+}
+
 static void
 pango_context_class_init (PangoContextClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  /* Put the check for mixed linkage here, for lack of a better place */
+  if (pango_has_mixed_deps ())
+    g_error ("Pango 1.x symbols detected.\n"
+             "Using Pango 1.x and 2 in the same process is not supported.");
+
   object_class->finalize = pango_context_finalize;
+  object_class->set_property = pango_context_set_property;
+  object_class->get_property = pango_context_get_property;
+
+  /**
+   * PangoContext:font-map: (attributes org.gtk.Property.get=pango_context_get_font_map org.gtk.Property.set=pango_context_set_font_map)
+   *
+   * The font map to be searched when fonts are looked up
+   * in this context.
+   */
+  properties[PROP_FONT_MAP] =
+    g_param_spec_object ("font-map", NULL, NULL, PANGO_TYPE_FONT_MAP,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:font-description: (attributes org.gtk.Property.get=pango_context_get_font_description org.gtk.Property.set=pango_context_set_font_description)
+   *
+   * The default font descriptoin for the context.
+   */
+  properties[PROP_FONT_DESCRIPTION] =
+    g_param_spec_boxed ("font-description", NULL, NULL, PANGO_TYPE_FONT_DESCRIPTION,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:language: (attributes org.gtk.Property.get=pango_context_get_language org.gtk.Property.set=pango_context_set_language)
+   *
+   * The global language for the context.
+   */
+  properties[PROP_LANGUAGE] =
+    g_param_spec_boxed ("language", NULL, NULL, PANGO_TYPE_LANGUAGE,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:base-direction: (attributes org.gtk.Property.get=pango_context_get_base_dir org.gtk.Property.set=pango_context_set_base_dir)
+   *
+   * The base direction for the context.
+   *
+   * The base direction is used in applying the Unicode bidirectional
+   * algorithm; if the @direction is `PANGO_DIRECTION_LTR` or
+   * `PANGO_DIRECTION_RTL`, then the value will be used as the paragraph
+   * direction in the Unicode bidirectional algorithm. A value of
+   * `PANGO_DIRECTION_WEAK_LTR` or `PANGO_DIRECTION_WEAK_RTL` is used only
+   * for paragraphs that do not contain any strong characters themselves.
+   */
+  properties[PROP_BASE_DIR] =
+    g_param_spec_enum ("base-direction", NULL, NULL, PANGO_TYPE_DIRECTION,
+                       PANGO_DIRECTION_LTR,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:base-gravity: (attributes org.gtk.Property.get=pango_context_get_base_gravity org.gtk.Property.set=pango_context_set_base_gravity)
+   *
+   * The base gravity for the context.
+   *
+   * The base gravity is used in laying vertical text out.
+   */
+  properties[PROP_BASE_GRAVITY] =
+    g_param_spec_enum ("base-gravity", NULL, NULL, PANGO_TYPE_GRAVITY,
+                       PANGO_GRAVITY_SOUTH,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:gravity-hint: (attributes org.gtk.Property.get=pango_context_get_gravity_hint org.gtk.Property.set=pango_context_set_gravity_hint)
+   *
+   * The gravity hint for the context.
+   *
+   * The gravity hint is used in laying vertical text out, and
+   * is only relevant if gravity of the context as returned by
+   * [method@Pango.Context.get_gravity] is set to `PANGO_GRAVITY_EAST`
+   * or `PANGO_GRAVITY_WEST`.
+   */
+  properties[PROP_GRAVITY_HINT] =
+    g_param_spec_enum ("gravity-hint", NULL, NULL, PANGO_TYPE_GRAVITY_HINT,
+                       PANGO_GRAVITY_HINT_NATURAL,
+                       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:matrix: (attributes org.gtk.Property.get=pango_context_get_matrix org.gtk.Property.set=pango_context_set_matrix)
+   *
+   * The 'user to device' transformation that will be applied when rendering
+   * with this context.
+   *
+   * This matrix is also known as the current transformation matrix, or 'ctm'.
+   *
+   * The transformation is needed in cases where the font rendering applies
+   * hinting that depends on knowing the position of text with respect to
+   * the pixel grid. If your font rendering does not
+   */
+  properties[PROP_MATRIX] =
+    g_param_spec_boxed ("matrix", NULL, NULL, PANGO_TYPE_MATRIX,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * PangoContext:round-glyph-positions: (attributes org.gtk.Property.get=pango_context_get_round_glyph_positions org.gtk.Property.set=pango_context_set_round_glyph_positions)
+   *
+   * Determines whether font rendering with this context should
+   * round glyph positions and widths to integral positions,
+   * in device units.
+   *
+   * This is useful when the renderer can't handle subpixel
+   * positioning of glyphs.
+   */
+  properties[PROP_ROUND_GLYPH_POSITIONS] =
+    g_param_spec_boolean ("round-glyph-positions", NULL, NULL, TRUE,
+                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, properties);
 }
 
 static void
@@ -103,7 +341,7 @@ pango_context_finalize (GObject *object)
     pango_matrix_free (context->matrix);
 
   if (context->metrics)
-    pango_font_metrics_unref (context->metrics);
+    pango_font_metrics_free (context->metrics);
 
   G_OBJECT_CLASS (pango_context_parent_class)->finalize (object);
 }
@@ -113,27 +351,46 @@ pango_context_finalize (GObject *object)
  *
  * Creates a new `PangoContext` initialized to default values.
  *
- * This function is not particularly useful as it should always
- * be followed by a [method@Pango.Context.set_font_map] call, and the
- * function [method@Pango.FontMap.create_context] does these two steps
- * together and hence users are recommended to use that.
+ * If you want to use a specific [class@Pango.FontMap] other than
+ * the default one, you should use [ctor@Pango.Context.new_with_font_map]
+ * instead.
  *
  * If you are using Pango as part of a higher-level system,
  * that system may have it's own way of create a `PangoContext`.
- * For instance, the GTK toolkit has, among others,
- * `gtk_widget_get_pango_context()`. Use those instead.
+ * Pango's own cairo support for instance, has [func@Pango.cairo_create_context],
+ * and the GTK toolkit has, among others, gtk_widget_get_pango_context().
+ * Use those instead.
  *
- * Return value: the newly allocated `PangoContext`, which should
- *   be freed with g_object_unref().
+ * Return value: the newly allocated `PangoContext`
  */
 PangoContext *
 pango_context_new (void)
 {
-  PangoContext *context;
+  return g_object_new (PANGO_TYPE_CONTEXT,
+                       "font-map", pango_font_map_get_default (),
+                       NULL);
+}
 
-  context = g_object_new (PANGO_TYPE_CONTEXT, NULL);
-
-  return context;
+/**
+ * pango_context_new_with_font_map:
+ * @font_map: the `PangoFontMap` to use
+ *
+ * Creates a new `PangoContext` with the given font map.
+ *
+ * If you are using Pango as part of a higher-level system,
+ * that system may have it's own way of create a `PangoContext`.
+ * Pango's own cairo support for instance, has [func@Pango.cairo_create_context],
+ * and the GTK toolkit has, among others, gtk_widget_get_pango_context().
+ * Use those instead.
+ *
+ * Return value: the newly allocated `PangoContext`
+ */
+PangoContext *
+pango_context_new_with_font_map (PangoFontMap *font_map)
+{
+  return g_object_new (PANGO_TYPE_CONTEXT,
+                       "font-map", font_map,
+                       NULL);
 }
 
 static void
@@ -151,16 +408,20 @@ update_resolved_gravity (PangoContext *context)
  * @matrix: (nullable): a `PangoMatrix`, or %NULL to unset any existing
  * matrix. (No matrix set is the same as setting the identity matrix.)
  *
- * Sets the transformation matrix that will be applied when rendering
+ * Sets the 'user to device' transformation that will be applied when rendering
  * with this context.
  *
+ * This matrix is also known as the current transformation matrix, or 'ctm'.
+ *
+ * The transformation is needed in cases where the font rendering applies
+ * hinting that depends on knowing the position of text with respect to
+ * the pixel grid.
+ *
  * Note that reported metrics are in the user space coordinates before
  * the application of the matrix, not device-space coordinates after the
  * application of the matrix. So, they don't scale with the matrix, though
  * they may change slightly for different matrices, depending on how the
  * text is fit to the pixel grid.
- *
- * Since: 1.6
  */
 void
 pango_context_set_matrix (PangoContext      *context,
@@ -179,6 +440,7 @@ pango_context_set_matrix (PangoContext      *context,
     context->matrix = NULL;
 
   update_resolved_gravity (context);
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_MATRIX]);
 }
 
 /**
@@ -193,8 +455,6 @@ pango_context_set_matrix (PangoContext      *context,
  * Return value: (nullable): the matrix, or %NULL if no matrix has
  *   been set (which is the same as the identity matrix). The returned
  *   matrix is owned by Pango and must not be modified or freed.
- *
- * Since: 1.6
  */
 const PangoMatrix *
 pango_context_get_matrix (PangoContext *context)
@@ -236,6 +496,8 @@ pango_context_set_font_map (PangoContext *context,
 
   context->font_map = font_map;
   context->fontmap_serial = pango_font_map_get_serial (font_map);
+
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_FONT_MAP]);
 }
 
 /**
@@ -246,8 +508,6 @@ pango_context_set_font_map (PangoContext *context,
  *
  * Return value: (transfer none): the font map for the `PangoContext`.
  *   This value is owned by Pango and should not be unreferenced.
- *
- * Since: 1.6
  */
 PangoFontMap *
 pango_context_get_font_map (PangoContext *context)
@@ -257,39 +517,6 @@ pango_context_get_font_map (PangoContext *context)
   return context->font_map;
 }
 
-/**
- * pango_context_list_families:
- * @context: a `PangoContext`
- * @families: (out) (array length=n_families) (transfer container): location
- *   to store a pointer to an array of `PangoFontFamily`. This array should
- *   be freed with g_free().
- * @n_families: (out): location to store the number of elements in @descs
- *
- * List all families for a context.
- */
-void
-pango_context_list_families (PangoContext      *context,
-                             PangoFontFamily ***families,
-                             int               *n_families)
-{
-  g_return_if_fail (context != NULL);
-  g_return_if_fail (families == NULL || n_families != NULL);
-
-  if (n_families == NULL)
-    return;
-
-  if (context->font_map == NULL)
-    {
-      *n_families = 0;
-      if (families)
-        *families = NULL;
-
-      return;
-    }
-  else
-    pango_font_map_list_families (context->font_map, families, n_families);
-}
-
 /**
  * pango_context_load_font:
  * @context: a `PangoContext`
@@ -354,6 +581,8 @@ pango_context_set_font_description (PangoContext               *context,
 
       pango_font_description_free (context->font_desc);
       context->font_desc = pango_font_description_copy (desc);
+
+      g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_FONT_DESCRIPTION]);
     }
 }
 
@@ -398,6 +627,8 @@ pango_context_set_language (PangoContext  *context,
     context->language = language;
   else
     context->language = pango_language_get_default ();
+
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_LANGUAGE]);
 }
 
 /**
@@ -436,10 +667,13 @@ pango_context_set_base_dir (PangoContext   *context,
 {
   g_return_if_fail (context != NULL);
 
-  if (direction != context->base_dir)
-    context_changed (context);
+  if (direction == context->base_dir)
+    return;
+
+  context_changed (context);
 
   context->base_dir = direction;
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_BASE_DIR]);
 }
 
 /**
@@ -468,8 +702,6 @@ pango_context_get_base_dir (PangoContext *context)
  * Sets the base gravity for the context.
  *
  * The base gravity is used in laying vertical text out.
- *
- * Since: 1.16
  */
 void
 pango_context_set_base_gravity (PangoContext *context,
@@ -477,12 +709,15 @@ pango_context_set_base_gravity (PangoContext *context,
 {
   g_return_if_fail (context != NULL);
 
-  if (gravity != context->base_gravity)
-    context_changed (context);
+  if (gravity == context->base_gravity)
+    return;
+
+  context_changed (context);
 
   context->base_gravity = gravity;
 
   update_resolved_gravity (context);
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_BASE_GRAVITY]);
 }
 
 /**
@@ -494,8 +729,6 @@ pango_context_set_base_gravity (PangoContext *context,
  * See [method@Pango.Context.set_base_gravity].
  *
  * Return value: the base gravity for the context.
- *
- * Since: 1.16
  */
 PangoGravity
 pango_context_get_base_gravity (PangoContext *context)
@@ -517,8 +750,6 @@ pango_context_get_base_gravity (PangoContext *context)
  * gravity from the current context matrix.
  *
  * Return value: the resolved gravity for the context.
- *
- * Since: 1.16
  */
 PangoGravity
 pango_context_get_gravity (PangoContext *context)
@@ -539,8 +770,6 @@ pango_context_get_gravity (PangoContext *context)
  * is only relevant if gravity of the context as returned by
  * [method@Pango.Context.get_gravity] is set to %PANGO_GRAVITY_EAST
  * or %PANGO_GRAVITY_WEST.
- *
- * Since: 1.16
  */
 void
 pango_context_set_gravity_hint (PangoContext     *context,
@@ -548,10 +777,13 @@ pango_context_set_gravity_hint (PangoContext     *context,
 {
   g_return_if_fail (context != NULL);
 
-  if (hint != context->gravity_hint)
-    context_changed (context);
+  if (hint == context->gravity_hint)
+    return;
+
+  context_changed (context);
 
   context->gravity_hint = hint;
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_GRAVITY_HINT]);
 }
 
 /**
@@ -563,8 +795,6 @@ pango_context_set_gravity_hint (PangoContext     *context,
  * See [method@Pango.Context.set_gravity_hint] for details.
  *
  * Return value: the gravity hint for the context.
- *
- * Since: 1.16
  */
 PangoGravityHint
 pango_context_get_gravity_hint (PangoContext *context)
@@ -580,31 +810,21 @@ get_first_metrics_foreach (PangoFontset  *fontset,
                            PangoFont     *font,
                            gpointer       data)
 {
-  PangoFontMetrics *fontset_metrics = data;
-  PangoLanguage *language = PANGO_FONTSET_GET_CLASS (fontset)->get_language (fontset);
-  PangoFontMetrics *font_metrics = pango_font_get_metrics (font, language);
-  guint save_ref_count;
+  PangoFontMetrics **fontset_metrics = data;
+  PangoLanguage *language = pango_fontset_get_language (fontset);
 
-  /* Initialize the fontset metrics to metrics of the first font in the
-   * fontset; saving the refcount and restoring it is a bit of hack but avoids
-   * having to update this code for each metrics addition.
-   */
-  save_ref_count = fontset_metrics->ref_count;
-  *fontset_metrics = *font_metrics;
-  fontset_metrics->ref_count = save_ref_count;
+  *fontset_metrics = pango_font_get_metrics (font, language);
 
-  pango_font_metrics_unref (font_metrics);
-
-  return TRUE;                  /* Stops iteration */
+  return TRUE; /* Stops iteration */
 }
 
 static PangoFontMetrics *
 get_base_metrics (PangoFontset *fontset)
 {
-  PangoFontMetrics *metrics = pango_font_metrics_new ();
+  PangoFontMetrics *metrics = NULL;
 
   /* Initialize the metrics from the first font in the fontset */
-  pango_fontset_foreach (fontset, get_first_metrics_foreach, metrics);
+  pango_fontset_foreach (fontset, get_first_metrics_foreach, &metrics);
 
   return metrics;
 }
@@ -641,12 +861,14 @@ update_metrics_from_items (PangoFontMetrics *metrics,
           metrics->ascent = MAX (metrics->ascent, raw_metrics->ascent);
           metrics->descent = MAX (metrics->descent, raw_metrics->descent);
           metrics->height = MAX (metrics->height, raw_metrics->height);
-          pango_font_metrics_unref (raw_metrics);
+          pango_font_metrics_free (raw_metrics);
         }
 
-      pango_shape_full (text + item->offset, item->length,
-                        text, text_len,
-                        &item->analysis, glyphs);
+      pango_shape (text + item->offset, item->length,
+                   text, text_len,
+                   &item->analysis, glyphs,
+                   PANGO_SHAPE_NONE);
+
       metrics->approximate_char_width += pango_glyph_string_get_width (glyphs);
     }
 
@@ -682,7 +904,7 @@ update_metrics_from_items (PangoFontMetrics *metrics,
  * for the individual families.
  *
  * Return value: a `PangoFontMetrics` object. The caller must call
- *   [method@Pango.FontMetrics.unref] when finished using the object.
+ *   [method@Pango.FontMetrics.free] when finished using the object.
  */
 PangoFontMetrics *
 pango_context_get_metrics (PangoContext               *context,
@@ -706,7 +928,7 @@ pango_context_get_metrics (PangoContext               *context,
   if (desc == context->font_desc &&
       language == context->language &&
       context->metrics != NULL)
-    return pango_font_metrics_ref (context->metrics);
+    return pango_font_metrics_copy (context->metrics);
 
   current_fonts = pango_font_map_load_fontset (context->font_map, context, desc, language);
   metrics = get_base_metrics (current_fonts);
@@ -727,7 +949,10 @@ pango_context_get_metrics (PangoContext               *context,
 
   if (desc == context->font_desc &&
       language == context->language)
-    context->metrics = pango_font_metrics_ref (metrics);
+    {
+      pango_font_metrics_free (context->metrics);
+      context->metrics = pango_font_metrics_copy (metrics);
+    }
 
   return metrics;
 }
@@ -739,7 +964,7 @@ context_changed (PangoContext *context)
   if (context->serial == 0)
     context->serial++;
 
-  g_clear_pointer (&context->metrics, pango_font_metrics_unref);
+  g_clear_pointer (&context->metrics, pango_font_metrics_free);
 }
 
 /**
@@ -753,9 +978,7 @@ context_changed (PangoContext *context)
  * for Pango, something applications won't do. Backends should
  * call this function if they have attached extra data to the context
  * and such data is changed.
- *
- * Since: 1.32.4
- **/
+ */
 void
 pango_context_changed (PangoContext *context)
 {
@@ -794,8 +1017,6 @@ check_fontmap_changed (PangoContext *context)
  * `PangoContext` changes, like `PangoLayout`.
  *
  * Return value: The current serial number of @context.
- *
- * Since: 1.32.4
  */
 guint
 pango_context_get_serial (PangoContext *context)
@@ -818,18 +1039,17 @@ pango_context_get_serial (PangoContext *context)
  *
  * The default value is to round glyph positions, to remain
  * compatible with previous Pango behavior.
- *
- * Since: 1.44
  */
 void
 pango_context_set_round_glyph_positions (PangoContext *context,
                                          gboolean      round_positions)
 {
-  if (context->round_glyph_positions != round_positions)
-    {
-      context->round_glyph_positions = round_positions;
-      context_changed (context);
-    }
+  if (context->round_glyph_positions == round_positions)
+    return;
+
+  context->round_glyph_positions = round_positions;
+  context_changed (context);
+  g_object_notify_by_pspec (G_OBJECT (context), properties[PROP_ROUND_GLYPH_POSITIONS]);
 }
 
 /**
@@ -838,8 +1058,6 @@ pango_context_set_round_glyph_positions (PangoContext *context,
  *
  * Returns whether font rendering with this context should
  * round glyph positions and widths.
- *
- * Since: 1.44
  */
 gboolean
 pango_context_get_round_glyph_positions (PangoContext *context)
