@@ -10,7 +10,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -22,9 +22,12 @@
 #include "config.h"
 #include <string.h>
 
-#include "pango-glyph-item.h"
+#include "pango-glyph-item-private.h"
+#include "pango-glyph-iter-private.h"
+#include "pango-item-private.h"
 #include "pango-impl-utils.h"
-#include "pango-attributes-private.h"
+#include "pango-attr-list-private.h"
+#include "pango-attr-iterator-private.h"
 
 #define LTR(glyph_item) (((glyph_item)->item->analysis.level % 2) == 0)
 
@@ -44,19 +47,13 @@
  * to the length of @orig (that is, there must be at least one byte
  * assigned to each item, you can't create a zero-length item).
  *
- * This function is similar in function to pango_item_split() (and uses
- * it internally.)
- *
- * Returns: (transfer full) (nullable): the newly allocated item
- *   representing text before @split_index, which should be freed
- *   with pango_glyph_item_free().
- *
- * Since: 1.2
+ * Return value: the newly allocated item representing text before
+ *   @split_index, which should be freed with [method@Pango.GlyphItem.free]
  */
 PangoGlyphItem *
 pango_glyph_item_split (PangoGlyphItem *orig,
-			const char     *text,
-			int             split_index)
+                        const char     *text,
+                        int             split_index)
 {
   PangoGlyphItem *new;
   int i;
@@ -72,13 +69,13 @@ pango_glyph_item_split (PangoGlyphItem *orig,
   if (LTR (orig))
     {
       for (i = 0; i < orig->glyphs->num_glyphs; i++)
-	{
-	  if (orig->glyphs->log_clusters[i] >= split_index)
-	    break;
-	}
+        {
+          if (orig->glyphs->log_clusters[i] >= split_index)
+            break;
+        }
 
       if (i == orig->glyphs->num_glyphs) /* No splitting necessary */
-	return NULL;
+        return NULL;
 
       split_index = orig->glyphs->log_clusters[i];
       num_glyphs = i;
@@ -86,13 +83,13 @@ pango_glyph_item_split (PangoGlyphItem *orig,
   else
     {
       for (i = orig->glyphs->num_glyphs - 1; i >= 0; i--)
-	{
-	  if (orig->glyphs->log_clusters[i] >= split_index)
-	    break;
-	}
+        {
+          if (orig->glyphs->log_clusters[i] >= split_index)
+            break;
+        }
 
       if (i < 0) /* No splitting necessary */
-	return NULL;
+        return NULL;
 
       split_index = orig->glyphs->log_clusters[i];
       num_glyphs = orig->glyphs->num_glyphs - 1 - i;
@@ -102,7 +99,7 @@ pango_glyph_item_split (PangoGlyphItem *orig,
 
   new = g_slice_new (PangoGlyphItem);
   split_offset = g_utf8_pointer_to_offset (text + orig->item->offset,
-					   text + orig->item->offset + split_index);
+                                           text + orig->item->offset + split_index);
   new->item = pango_item_split (orig->item, split_index, split_offset);
 
   new->glyphs = pango_glyph_string_new ();
@@ -114,9 +111,9 @@ pango_glyph_item_split (PangoGlyphItem *orig,
       memcpy (new->glyphs->log_clusters, orig->glyphs->log_clusters, num_glyphs * sizeof (int));
 
       memmove (orig->glyphs->glyphs, orig->glyphs->glyphs + num_glyphs,
-	       num_remaining * sizeof (PangoGlyphInfo));
+               num_remaining * sizeof (PangoGlyphInfo));
       for (i = num_glyphs; i < orig->glyphs->num_glyphs; i++)
-	orig->glyphs->log_clusters[i - num_glyphs] = orig->glyphs->log_clusters[i] - split_index;
+        orig->glyphs->log_clusters[i - num_glyphs] = orig->glyphs->log_clusters[i] - split_index;
     }
   else
     {
@@ -124,7 +121,7 @@ pango_glyph_item_split (PangoGlyphItem *orig,
       memcpy (new->glyphs->log_clusters, orig->glyphs->log_clusters + num_remaining, num_glyphs * sizeof (int));
 
       for (i = 0; i < num_remaining; i++)
-	orig->glyphs->log_clusters[i] = orig->glyphs->log_clusters[i] - split_index;
+        orig->glyphs->log_clusters[i] = orig->glyphs->log_clusters[i] - split_index;
     }
 
   pango_glyph_string_set_size (orig->glyphs, orig->glyphs->num_glyphs - num_glyphs);
@@ -142,9 +139,7 @@ pango_glyph_item_split (PangoGlyphItem *orig,
  *
  * Make a deep copy of an existing `PangoGlyphItem` structure.
  *
- * Returns: (transfer full) (nullable): the newly allocated `PangoGlyphItem`
- *
- * Since: 1.20
+ * Return value: (nullable): the newly allocated `PangoGlyphItem`
  */
 PangoGlyphItem *
 pango_glyph_item_copy  (PangoGlyphItem *orig)
@@ -170,8 +165,6 @@ pango_glyph_item_copy  (PangoGlyphItem *orig)
  * @glyph_item: (nullable): a `PangoGlyphItem`
  *
  * Frees a `PangoGlyphItem` and resources to which it points.
- *
- * Since: 1.6
  */
 void
 pango_glyph_item_free  (PangoGlyphItem *glyph_item)
@@ -198,9 +191,7 @@ G_DEFINE_BOXED_TYPE (PangoGlyphItem, pango_glyph_item,
  *
  * Make a shallow copy of an existing `PangoGlyphItemIter` structure.
  *
- * Returns: (transfer full) (nullable): the newly allocated `PangoGlyphItemIter`
- *
- * Since: 1.22
+ * Return value: (nullable): the newly allocated `PangoGlyphItemIter`
  */
 PangoGlyphItemIter *
 pango_glyph_item_iter_copy (PangoGlyphItemIter *orig)
@@ -221,9 +212,7 @@ pango_glyph_item_iter_copy (PangoGlyphItemIter *orig)
  * pango_glyph_item_iter_free:
  * @iter: (nullable): a `PangoGlyphItemIter`
  *
- * Frees a `PangoGlyphItem`Iter.
- *
- * Since: 1.22
+ * Frees a `PangoGlyphItemIter`.
  */
 void
 pango_glyph_item_iter_free  (PangoGlyphItemIter *iter)
@@ -246,10 +235,8 @@ G_DEFINE_BOXED_TYPE (PangoGlyphItemIter, pango_glyph_item_iter,
  *
  * See `PangoGlyphItemIter` for details of cluster orders.
  *
- * Returns: %TRUE if the iterator was advanced,
+ * Return value: %TRUE if the iterator was advanced,
  *   %FALSE if we were already on the  last cluster.
- *
- * Since: 1.22
  */
 gboolean
 pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
@@ -262,12 +249,12 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
   if (LTR (iter->glyph_item))
     {
       if (glyph_index == glyphs->num_glyphs)
-	return FALSE;
+        return FALSE;
     }
   else
     {
       if (glyph_index < 0)
-	return FALSE;
+        return FALSE;
     }
 
   iter->start_glyph = iter->end_glyph;
@@ -278,47 +265,47 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
     {
       cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
-	{
-	  glyph_index++;
+        {
+          glyph_index++;
 
-	  if (glyph_index == glyphs->num_glyphs)
-	    {
-	      iter->end_index = item->offset + item->length;
-	      iter->end_char = item->num_chars;
-	      break;
-	    }
+          if (glyph_index == glyphs->num_glyphs)
+            {
+              iter->end_index = item->offset + item->length;
+              iter->end_char = item->num_chars;
+              break;
+            }
 
-	  if (glyphs->log_clusters[glyph_index] > cluster)
-	    {
-	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
-	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
-					       iter->end_index - iter->start_index);
-	      break;
-	    }
-	}
+          if (glyphs->log_clusters[glyph_index] > cluster)
+            {
+              iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
+              iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
+                                               iter->end_index - iter->start_index);
+              break;
+            }
+        }
     }
-  else			/* RTL */
+  else                  /* RTL */
     {
       cluster = glyphs->log_clusters[glyph_index];
       while (TRUE)
-	{
-	  glyph_index--;
+        {
+          glyph_index--;
 
-	  if (glyph_index < 0)
-	    {
-	      iter->end_index = item->offset + item->length;
-	      iter->end_char = item->num_chars;
-	      break;
-	    }
+          if (glyph_index < 0)
+            {
+              iter->end_index = item->offset + item->length;
+              iter->end_char = item->num_chars;
+              break;
+            }
 
-	  if (glyphs->log_clusters[glyph_index] > cluster)
-	    {
-	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
-	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
-					       iter->end_index - iter->start_index);
-	      break;
-	    }
-	}
+          if (glyphs->log_clusters[glyph_index] > cluster)
+            {
+              iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
+              iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
+                                               iter->end_index - iter->start_index);
+              break;
+            }
+        }
     }
 
   iter->end_glyph = glyph_index;
@@ -336,10 +323,8 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
  * Moves the iterator to the preceding cluster in the glyph item.
  * See `PangoGlyphItemIter` for details of cluster orders.
  *
- * Returns: %TRUE if the iterator was moved,
+ * Return value: %TRUE if the iterator was moved,
  *   %FALSE if we were already on the first cluster.
- *
- * Since: 1.22
  */
 gboolean
 pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
@@ -352,12 +337,12 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
   if (LTR (iter->glyph_item))
     {
       if (glyph_index == 0)
-	return FALSE;
+        return FALSE;
     }
   else
     {
       if (glyph_index == glyphs->num_glyphs - 1)
-	return FALSE;
+        return FALSE;
 
     }
 
@@ -369,49 +354,49 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
     {
       cluster = glyphs->log_clusters[glyph_index - 1];
       while (TRUE)
-	{
-	  if (glyph_index == 0)
-	    {
-	      iter->start_index = item->offset;
-	      iter->start_char = 0;
-	      break;
-	    }
+        {
+          if (glyph_index == 0)
+            {
+              iter->start_index = item->offset;
+              iter->start_char = 0;
+              break;
+            }
 
-	  glyph_index--;
+          glyph_index--;
 
-	  if (glyphs->log_clusters[glyph_index] < cluster)
-	    {
-	      glyph_index++;
-	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
-	      iter->start_char -= pango_utf8_strlen (iter->text + iter->start_index,
-						 iter->end_index - iter->start_index);
-	      break;
-	    }
-	}
+          if (glyphs->log_clusters[glyph_index] < cluster)
+            {
+              glyph_index++;
+              iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
+              iter->start_char -= pango_utf8_strlen (iter->text + iter->start_index,
+                                                 iter->end_index - iter->start_index);
+              break;
+            }
+        }
     }
-  else			/* RTL */
+  else                  /* RTL */
     {
       cluster = glyphs->log_clusters[glyph_index + 1];
       while (TRUE)
-	{
-	  if (glyph_index == glyphs->num_glyphs - 1)
-	    {
-	      iter->start_index = item->offset;
-	      iter->start_char = 0;
-	      break;
-	    }
+        {
+          if (glyph_index == glyphs->num_glyphs - 1)
+            {
+              iter->start_index = item->offset;
+              iter->start_char = 0;
+              break;
+            }
 
-	  glyph_index++;
+          glyph_index++;
 
-	  if (glyphs->log_clusters[glyph_index] < cluster)
-	    {
-	      glyph_index--;
-	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
-	      iter->start_char -= pango_utf8_strlen (iter->text + iter->start_index,
-						 iter->end_index - iter->start_index);
-	      break;
-	    }
-	}
+          if (glyphs->log_clusters[glyph_index] < cluster)
+            {
+              glyph_index--;
+              iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
+              iter->start_char -= pango_utf8_strlen (iter->text + iter->start_index,
+                                                 iter->end_index - iter->start_index);
+              break;
+            }
+        }
     }
 
   iter->start_glyph = glyph_index;
@@ -433,14 +418,12 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
  *
  * See `PangoGlyphItemIter` for details of cluster orders.
  *
- * Returns: %FALSE if there are no clusters in the glyph item
- *
- * Since: 1.22
+ * Return value: %FALSE if there are no clusters in the glyph item
  */
 gboolean
 pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
-				  PangoGlyphItem      *glyph_item,
-				  const char          *text)
+                                  PangoGlyphItem      *glyph_item,
+                                  const char          *text)
 {
   iter->glyph_item = glyph_item;
   iter->text = text;
@@ -472,14 +455,12 @@ pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
  *
  * See `PangoGlyphItemIter` for details of cluster orders.
  *
- * Returns: %FALSE if there are no clusters in the glyph item
- *
- * Since: 1.22
+ * Return value: %FALSE if there are no clusters in the glyph item
  */
 gboolean
 pango_glyph_item_iter_init_end (PangoGlyphItemIter  *iter,
-				PangoGlyphItem      *glyph_item,
-				const char          *text)
+                                PangoGlyphItem      *glyph_item,
+                                const char          *text)
 {
   iter->glyph_item = glyph_item;
   iter->text = text;
@@ -511,7 +492,7 @@ typedef struct
  */
 static void
 append_attrs (PangoGlyphItem *glyph_item,
-	      GSList         *attrs)
+              GSList         *attrs)
 {
   glyph_item->item->analysis.extra_attrs =
     g_slist_concat (glyph_item->item->analysis.extra_attrs, attrs);
@@ -582,17 +563,15 @@ split_before_cluster_start (ApplyAttrsState *state)
  * This function takes ownership of @glyph_item; it will be reused
  * as one of the elements in the list.
  *
- * Returns: (transfer full) (element-type Pango.GlyphItem): a
+ * Return value: (transfer full) (element-type Pango.GlyphItem): a
  *   list of glyph items resulting from splitting @glyph_item. Free
  *   the elements using [method@Pango.GlyphItem.free], the list using
- *   g_slist_free().
- *
- * Since: 1.2
+ *   [func@GLib.SList.free]
  */
 GSList *
 pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
-			      const char       *text,
-			      PangoAttrList    *list)
+                              const char       *text,
+                              PangoAttrList    *list)
 {
   PangoAttrIterator iter;
   GSList *result = NULL;
@@ -617,12 +596,12 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 
   /* Advance the attr iterator to the start of the item
    */
-  _pango_attr_list_get_iterator (list, &iter);
+  pango_attr_list_init_iterator (list, &iter);
   do
     {
       pango_attr_iterator_range (&iter, &range_start, &range_end);
       if (range_end > glyph_item->item->offset)
-	break;
+        break;
     }
   while (pango_attr_iterator_next (&iter));
 
@@ -652,11 +631,11 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
        * of the last cluster
        */
       if (start_new_segment)
-	{
-	  result = g_slist_prepend (result,
-				    split_before_cluster_start (&state));
-	  state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
-	}
+        {
+          result = g_slist_prepend (result,
+                                    split_before_cluster_start (&state));
+          state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
+        }
 
       start_new_segment = FALSE;
 
@@ -665,42 +644,42 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
        * intersects the next cluster.
        */
       do
-	{
-	  if (range_end > state.iter.end_index) /* Range intersects next cluster */
-	    break;
+        {
+          if (range_end > state.iter.end_index) /* Range intersects next cluster */
+            break;
 
-	  /* Since ranges end in this cluster, the next cluster goes into a
-	   * separate segment
-	   */
-	  start_new_segment = TRUE;
+          /* Since ranges end in this cluster, the next cluster goes into a
+           * separate segment
+           */
+          start_new_segment = TRUE;
 
-	  have_next = pango_attr_iterator_next (&iter);
-	  pango_attr_iterator_range (&iter, &range_start, &range_end);
+          have_next = pango_attr_iterator_next (&iter);
+          pango_attr_iterator_range (&iter, &range_start, &range_end);
 
-	  if (range_start >= state.iter.end_index) /* New range doesn't intersect this cluster */
-	    {
-	      /* No gap between ranges, so previous range must of ended
-	       * at cluster boundary.
-	       */
-	      g_assert (range_start == state.iter.end_index && start_new_segment);
-	      break;
-	    }
+          if (range_start >= state.iter.end_index) /* New range doesn't intersect this cluster */
+            {
+              /* No gap between ranges, so previous range must of ended
+               * at cluster boundary.
+               */
+              g_assert (range_start == state.iter.end_index && start_new_segment);
+              break;
+            }
 
-	  /* If any ranges start *inside* this cluster, then we need
-	   * to split the previous cluster into a separate segment
-	   */
-	  if (range_start > state.iter.start_index &&
-	      state.iter.start_index != glyph_item->item->offset)
-	    {
-	      GSList *new_attrs = attr_slist_copy (state.segment_attrs);
-	      result = g_slist_prepend (result,
-					split_before_cluster_start (&state));
-	      state.segment_attrs = new_attrs;
-	    }
+          /* If any ranges start *inside* this cluster, then we need
+           * to split the previous cluster into a separate segment
+           */
+          if (range_start > state.iter.start_index &&
+              state.iter.start_index != glyph_item->item->offset)
+            {
+              GSList *new_attrs = attr_slist_copy (state.segment_attrs);
+              result = g_slist_prepend (result,
+                                        split_before_cluster_start (&state));
+              state.segment_attrs = new_attrs;
+            }
 
-	  state.segment_attrs = g_slist_concat (state.segment_attrs,
-						pango_attr_iterator_get_attrs (&iter));
-	}
+          state.segment_attrs = g_slist_concat (state.segment_attrs,
+                                                pango_attr_iterator_get_attrs (&iter));
+        }
       while (have_next);
     }
 
@@ -713,7 +692,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
   if (LTR (glyph_item))
     result = g_slist_reverse (result);
 
-  _pango_attr_iterator_destroy (&iter);
+  pango_attr_iterator_clear (&iter);
 
   return result;
 }
@@ -733,14 +712,12 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
  *
  * Adds spacing between the graphemes of @glyph_item to
  * give the effect of typographic letter spacing.
- *
- * Since: 1.6
  */
 void
 pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
-			       const char     *text,
-			       PangoLogAttr   *log_attrs,
-			       int             letter_spacing)
+                               const char     *text,
+                               PangoLogAttr   *log_attrs,
+                               int             letter_spacing)
 {
   PangoGlyphItemIter iter;
   PangoGlyphInfo *glyphs = glyph_item->glyphs->glyphs;
@@ -774,29 +751,29 @@ pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
         }
 
       if (iter.start_glyph < iter.end_glyph) /* LTR */
-	{
-	  if (iter.start_char > 0)
-	    {
-	      glyphs[iter.start_glyph].geometry.width    += space_left ;
-	      glyphs[iter.start_glyph].geometry.x_offset += space_left ;
-	    }
-	  if (iter.end_char < glyph_item->item->num_chars)
-	    {
-	      glyphs[iter.end_glyph-1].geometry.width    += space_right;
-	    }
-	}
-      else			                 /* RTL */
-	{
-	  if (iter.start_char > 0)
-	    {
-	      glyphs[iter.start_glyph].geometry.width    += space_right;
-	    }
-	  if (iter.end_char < glyph_item->item->num_chars)
-	    {
-	      glyphs[iter.end_glyph+1].geometry.x_offset += space_left ;
-	      glyphs[iter.end_glyph+1].geometry.width    += space_left ;
-	    }
-	}
+        {
+          if (iter.start_char > 0)
+            {
+              glyphs[iter.start_glyph].geometry.width    += space_left ;
+              glyphs[iter.start_glyph].geometry.x_offset += space_left ;
+            }
+          if (iter.end_char < glyph_item->item->num_chars)
+            {
+              glyphs[iter.end_glyph-1].geometry.width    += space_right;
+            }
+        }
+      else                                       /* RTL */
+        {
+          if (iter.start_char > 0)
+            {
+              glyphs[iter.start_glyph].geometry.width    += space_right;
+            }
+          if (iter.end_char < glyph_item->item->num_chars)
+            {
+              glyphs[iter.end_glyph+1].geometry.x_offset += space_left ;
+              glyphs[iter.end_glyph+1].geometry.width    += space_left ;
+            }
+        }
     }
 }
 
@@ -817,13 +794,11 @@ pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
  * entire cluster is divided equally among the characters.
  *
  * See also [method@Pango.GlyphString.get_logical_widths].
- *
- * Since: 1.26
  */
 void
 pango_glyph_item_get_logical_widths (PangoGlyphItem *glyph_item,
-				     const char     *text,
-				     int            *logical_widths)
+                                     const char     *text,
+                                     int            *logical_widths)
 {
   PangoGlyphItemIter iter;
   gboolean has_cluster;
@@ -837,26 +812,26 @@ pango_glyph_item_get_logical_widths (PangoGlyphItem *glyph_item,
       int glyph_index, char_index, num_chars, cluster_width = 0, char_width;
 
       for (glyph_index  = iter.start_glyph;
-	   glyph_index != iter.end_glyph;
-	   glyph_index += dir)
+           glyph_index != iter.end_glyph;
+           glyph_index += dir)
         {
-	  cluster_width += glyph_item->glyphs->glyphs[glyph_index].geometry.width;
-	}
+          cluster_width += glyph_item->glyphs->glyphs[glyph_index].geometry.width;
+        }
 
       num_chars = iter.end_char - iter.start_char;
       if (num_chars) /* pedantic */
         {
-	  char_width = cluster_width / num_chars;
+          char_width = cluster_width / num_chars;
 
-	  for (char_index = iter.start_char;
-	       char_index < iter.end_char;
-	       char_index++)
-	    {
-	      logical_widths[char_index] = char_width;
-	    }
+          for (char_index = iter.start_char;
+               char_index < iter.end_char;
+               char_index++)
+            {
+              logical_widths[char_index] = char_width;
+            }
 
-	  /* add any residues to the first char */
-	  logical_widths[iter.start_char] += cluster_width - (char_width * num_chars);
-	}
+          /* add any residues to the first char */
+          logical_widths[iter.start_char] += cluster_width - (char_width * num_chars);
+        }
     }
 }
