@@ -1,26 +1,23 @@
-/* Pango
- * pango-gravity.h: Gravity routines
- *
+/*
  * Copyright (C) 2006, 2007 Red Hat Software
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __PANGO_GRAVITY_H__
-#define __PANGO_GRAVITY_H__
+#pragma once
 
 #include <glib.h>
 
@@ -28,10 +25,10 @@ G_BEGIN_DECLS
 
 /**
  * PangoGravity:
- * @PANGO_GRAVITY_SOUTH: Glyphs stand upright (default)
- * @PANGO_GRAVITY_EAST: Glyphs are rotated 90 degrees clockwise
- * @PANGO_GRAVITY_NORTH: Glyphs are upside-down
- * @PANGO_GRAVITY_WEST: Glyphs are rotated 90 degrees counter-clockwise
+ * @PANGO_GRAVITY_SOUTH: Glyphs stand upright (default) <img align="right" valign="center" src="m-south.png">
+ * @PANGO_GRAVITY_EAST: Glyphs are rotated 90 degrees counter-clockwise. <img align="right" valign="center" src="m-east.png">
+ * @PANGO_GRAVITY_NORTH: Glyphs are upside-down. <img align="right" valign="cener" src="m-north.png">
+ * @PANGO_GRAVITY_WEST: Glyphs are rotated 90 degrees clockwise. <img align="right" valign="center" src="m-west.png">
  * @PANGO_GRAVITY_AUTO: Gravity is resolved from the context matrix
  *
  * `PangoGravity` represents the orientation of glyphs in a segment
@@ -47,8 +44,6 @@ G_BEGIN_DECLS
  * [method@Pango.Context.get_base_gravity].
  *
  * See also: [enum@Pango.GravityHint]
- *
- * Since: 1.16
  */
 typedef enum {
   PANGO_GRAVITY_SOUTH,
@@ -75,8 +70,6 @@ typedef enum {
  * That is, English excerpts in a vertical paragraph for example.
  *
  * See also [enum@Pango.Gravity]
- *
- * Since: 1.16
  */
 typedef enum {
   PANGO_GRAVITY_HINT_NATURAL,
@@ -92,11 +85,9 @@ typedef enum {
  *
  * Returns: %TRUE if @gravity is %PANGO_GRAVITY_EAST or %PANGO_GRAVITY_WEST,
  *   %FALSE otherwise.
- *
- * Since: 1.16
  */
 #define PANGO_GRAVITY_IS_VERTICAL(gravity) \
-	((gravity) == PANGO_GRAVITY_EAST || (gravity) == PANGO_GRAVITY_WEST)
+        ((gravity) == PANGO_GRAVITY_EAST || (gravity) == PANGO_GRAVITY_WEST)
 
 /**
  * PANGO_GRAVITY_IS_IMPROPER:
@@ -107,31 +98,27 @@ typedef enum {
  *
  * Returns: %TRUE if @gravity is %PANGO_GRAVITY_WEST or %PANGO_GRAVITY_NORTH,
  *   %FALSE otherwise.
- *
- * Since: 1.32
  */
 #define PANGO_GRAVITY_IS_IMPROPER(gravity) \
-	((gravity) == PANGO_GRAVITY_WEST || (gravity) == PANGO_GRAVITY_NORTH)
+        ((gravity) == PANGO_GRAVITY_WEST || (gravity) == PANGO_GRAVITY_NORTH)
 
 #include <pango/pango-matrix.h>
 #include <pango/pango-script.h>
 
-PANGO_AVAILABLE_IN_1_16
+PANGO_AVAILABLE_IN_ALL
 double       pango_gravity_to_rotation    (PangoGravity       gravity) G_GNUC_CONST;
-PANGO_AVAILABLE_IN_1_16
+PANGO_AVAILABLE_IN_ALL
 PangoGravity pango_gravity_get_for_matrix (const PangoMatrix *matrix) G_GNUC_PURE;
-PANGO_AVAILABLE_IN_1_16
-PangoGravity pango_gravity_get_for_script (PangoScript        script,
-					   PangoGravity       base_gravity,
-					   PangoGravityHint   hint) G_GNUC_CONST;
-PANGO_AVAILABLE_IN_1_26
+PANGO_AVAILABLE_IN_ALL
+PangoGravity pango_gravity_get_for_script (GUnicodeScript     script,
+                                           PangoGravity       base_gravity,
+                                           PangoGravityHint   hint) G_GNUC_CONST;
+PANGO_AVAILABLE_IN_ALL
 PangoGravity pango_gravity_get_for_script_and_width
-					  (PangoScript        script,
-					   gboolean           wide,
-					   PangoGravity       base_gravity,
-					   PangoGravityHint   hint) G_GNUC_CONST;
+                                          (GUnicodeScript     script,
+                                           gboolean           wide,
+                                           PangoGravity       base_gravity,
+                                           PangoGravityHint   hint) G_GNUC_CONST;
 
 
 G_END_DECLS
-
-#endif /* __PANGO_GRAVITY_H__ */
