@@ -1,32 +1,27 @@
-/* Pango
- * pango-impl-utils.h: Macros for get_type() functions
- * Inspired by Jody Goldberg's gsf-impl-utils.h
- *
+/*
  * Copyright (C) 2003 Red Hat Software
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __PANGO_IMPL_UTILS_H__
-#define __PANGO_IMPL_UTILS_H__
+#pragma once
 
 #include <glib.h>
 #include <glib-object.h>
 #include <pango/pango.h>
-#include <hb-ot.h>
 
 G_BEGIN_DECLS
 
@@ -34,21 +29,6 @@ G_BEGIN_DECLS
 /* String interning for static strings */
 #define I_(string) g_intern_static_string (string)
 
-
-/* Some functions for handling PANGO_ATTR_SHAPE */
-void _pango_shape_shape (const char       *text,
-			 unsigned int      n_chars,
-			 PangoRectangle   *shape_ink,
-			 PangoRectangle   *shape_logical,
-			 PangoGlyphString *glyphs);
-
-void _pango_shape_get_extents (gint              n_chars,
-			       PangoRectangle   *shape_ink,
-			       PangoRectangle   *shape_logical,
-			       PangoRectangle   *ink_rect,
-			       PangoRectangle   *logical_rect);
-
-
 /* We define these functions static here because we don't want to add public API
  * for them (if anything, it belongs to glib, but glib found it trivial enough
  * not to add API for).  At some point metrics calculations will be
@@ -63,7 +43,7 @@ pango_unichar_width (gunichar c)
 }
 
 static G_GNUC_UNUSED glong
-pango_utf8_strwidth (const gchar *p)
+pango_utf8_strwidth (const char *p)
 {
   glong len = 0;
   g_return_val_if_fail (p != NULL, 0);
@@ -80,10 +60,10 @@ pango_utf8_strwidth (const gchar *p)
 /* Glib's g_utf8_strlen() is broken and stops at embedded NUL's.
  * Wrap it here. */
 static G_GNUC_UNUSED glong
-pango_utf8_strlen (const gchar *p, gssize max)
+pango_utf8_strlen (const char *p, gssize max)
 {
   glong len = 0;
-  const gchar *start = p;
+  const char *start = p;
   g_return_val_if_fail (p != NULL || max == 0, 0);
 
   if (max <= 0)
@@ -117,7 +97,7 @@ pango_glyph_string_reverse_range (PangoGlyphString *glyphs,
   for (i = start, j = end - 1; i < j; i++, j--)
     {
       PangoGlyphInfo glyph_info;
-      gint log_cluster;
+      int log_cluster;
 
       glyph_info = glyphs->glyphs[i];
       glyphs->glyphs[i] = glyphs->glyphs[j];
@@ -203,9 +183,7 @@ static struct {
 static inline G_GNUC_UNUSED const char *
 pango_get_ignorable (gunichar ch)
 {
-  int i;
-
-  for (i = 0; i < G_N_ELEMENTS (ignorables); i++)
+  for (guint i = 0; i < G_N_ELEMENTS (ignorables); i++)
     {
       if (ch < ignorables[i].ch)
         return NULL;
@@ -213,6 +191,7 @@ pango_get_ignorable (gunichar ch)
       if (ch == ignorables[i].ch)
         return ignorables[i].nick;
     }
+
   return NULL;
 }
 
@@ -256,37 +235,4 @@ pango_get_ignorable_size (gunichar  ch,
 # define g_memdup2(mem,size)    g_memdup((mem),(size))
 #endif
 
-static inline void
-pango_parse_variations (const char            *variations,
-                        hb_ot_var_axis_info_t *axes,
-                        int                    n_axes,
-                        float                 *coords)
-{
-  const char *p;
-  const char *end;
-  hb_variation_t var;
-  int i;
-
-  p = variations;
-  while (p && *p)
-    {
-      end = strchr (p, ',');
-      if (hb_variation_from_string (p, end ? end - p: -1, &var))
-        {
-          for (i = 0; i < n_axes; i++)
-            {
-              if (axes[i].tag == var.tag)
-                {
-                  coords[axes[i].axis_index] = var.value;
-                  break;
-                }
-            }
-        }
-
-      p = end ? end + 1 : NULL;
-    }
-}
-
 G_END_DECLS
-
-#endif /* __PANGO_IMPL_UTILS_H__ */
