@@ -21,9 +21,27 @@
 
 #include "config.h"
 #include "pango-attributes.h"
+#include "pango-attr-private.h"
 #include "pango-item-private.h"
 #include "pango-impl-utils.h"
 
+/**
+ * PangoAnalysis:
+ *
+ * The `PangoAnalysis` structure stores information about
+ * the properties of a segment of text.
+ */
+
+/**
+ * PangoItem:
+ *
+ * The `PangoItem` structure stores information about
+ * a segment of text.
+ *
+ * You typically obtain `PangoItems` by itemizing a piece
+ * of text with [func@Pango.itemize].
+ */
+
 /**
  * pango_item_new:
  *
@@ -35,9 +53,7 @@
 PangoItem *
 pango_item_new (void)
 {
-  PangoItemPrivate *result = g_slice_new0 (PangoItemPrivate);
-
-  result->analysis.flags |= PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET;
+  PangoItem *result = g_slice_new0 (PangoItem);
 
   return (PangoItem *)result;
 }
@@ -64,12 +80,11 @@ pango_item_copy (PangoItem *item)
   result->offset = item->offset;
   result->length = item->length;
   result->num_chars = item->num_chars;
-  if (item->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    ((PangoItemPrivate *)result)->char_offset = ((PangoItemPrivate *)item)->char_offset;
+  result->char_offset = item->char_offset;
 
   result->analysis = item->analysis;
-  if (result->analysis.lang_engine)
-    g_object_ref (result->analysis.lang_engine);
+  if (result->analysis.size_font)
+    g_object_ref (result->analysis.size_font);
 
   if (result->analysis.font)
     g_object_ref (result->analysis.font);
@@ -105,16 +120,13 @@ pango_item_free (PangoItem *item)
       g_slist_free (item->analysis.extra_attrs);
     }
 
-  if (item->analysis.lang_engine)
-    g_object_unref (item->analysis.lang_engine);
+  if (item->analysis.size_font)
+    g_object_unref (item->analysis.size_font);
 
   if (item->analysis.font)
     g_object_unref (item->analysis.font);
 
-  if (item->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    g_slice_free (PangoItemPrivate, (PangoItemPrivate *)item);
-  else
-    g_slice_free (PangoItem, item);
+  g_slice_free (PangoItem, item);
 }
 
 G_DEFINE_BOXED_TYPE (PangoItem, pango_item,
@@ -164,8 +176,7 @@ pango_item_split (PangoItem *orig,
   orig->offset += split_index;
   orig->length -= split_index;
   orig->num_chars -= split_offset;
-  if (orig->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    ((PangoItemPrivate *)orig)->char_offset += split_offset;
+  orig->char_offset += split_offset;
 
   return new_item;
 }
@@ -190,9 +201,7 @@ pango_item_unsplit (PangoItem *orig,
   orig->offset -= split_index;
   orig->length += split_index;
   orig->num_chars += split_offset;
-
-  if (orig->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    ((PangoItemPrivate *)orig)->char_offset -= split_offset;
+  orig->char_offset -= split_offset;
 }
 
 static int
@@ -224,8 +233,6 @@ compare_attr (gconstpointer p1, gconstpointer p2)
  * and will be advanced past it. This function is meant to be called
  * in a loop over the items resulting from itemization, while passing
  * the iter to each call.
- *
- * Since: 1.44
  */
 void
 pango_item_apply_attrs (PangoItem         *item,
@@ -271,19 +278,27 @@ pango_analysis_collect_features (const PangoAnalysis *analysis,
 {
   GSList *l;
 
-  pango_font_get_features (analysis->font, features, length, num_features);
+  if (PANGO_IS_HB_FONT (analysis->font))
+    {
+      const hb_feature_t *font_features;
+      guint n_font_features;
+
+      font_features = pango_hb_font_get_features (PANGO_HB_FONT (analysis->font),
+                                                  &n_font_features);
+      *num_features = MIN (length, n_font_features);
+      memcpy (features, font_features, sizeof (hb_feature_t) * *num_features);
+   }
 
   for (l = analysis->extra_attrs; l && *num_features < length; l = l->next)
     {
       PangoAttribute *attr = l->data;
-      if (attr->klass->type == PANGO_ATTR_FONT_FEATURES)
+      if (attr->type == PANGO_ATTR_FONT_FEATURES)
         {
-          PangoAttrFontFeatures *fattr = (PangoAttrFontFeatures *) attr;
-          const gchar *feat;
-          const gchar *end;
+          const char *feat;
+          const char *end;
           int len;
 
-          feat = fattr->features;
+          feat = attr->str_value;
 
           while (feat != NULL && *num_features < length)
             {
@@ -311,7 +326,7 @@ pango_analysis_collect_features (const PangoAnalysis *analysis,
   for (l = analysis->extra_attrs; l && *num_features < length; l = l->next)
     {
       PangoAttribute *attr = l->data;
-      if (attr->klass->type == PANGO_ATTR_LETTER_SPACING)
+      if (attr->type == PANGO_ATTR_LETTER_SPACING)
         {
           hb_tag_t tags[] = {
             HB_TAG('l','i','g','a'),
@@ -346,13 +361,11 @@ void
 pango_analysis_set_size_font (PangoAnalysis *analysis,
                               PangoFont     *font)
 {
-  PangoAnalysisPrivate *priv = (PangoAnalysisPrivate *)analysis;
-
-  if (priv->size_font)
-    g_object_unref (priv->size_font);
-  priv->size_font = font;
-  if (priv->size_font)
-    g_object_ref (priv->size_font);
+  if (analysis->size_font)
+    g_object_unref (analysis->size_font);
+  analysis->size_font = font;
+  if (analysis->size_font)
+    g_object_ref (analysis->size_font);
 }
 
 /*< private >
@@ -368,32 +381,272 @@ pango_analysis_set_size_font (PangoAnalysis *analysis,
 PangoFont *
 pango_analysis_get_size_font (const PangoAnalysis *analysis)
 {
-  PangoAnalysisPrivate *priv = (PangoAnalysisPrivate *)analysis;
+  return analysis->size_font;
+}
 
-  return priv->size_font;
+/*< private >
+ * pango_item_get_properties:
+ * @item: a `PangoItem`
+ * @properties: `ItemProperties` struct to populate
+ *
+ * Extract useful information from the @item's attributes.
+ *
+ * Note that letter-spacing and shape are required to be constant
+ * across items. But underline and strikethrough can vary across
+ * an item, so we collect all the values that we find.
+ */
+void
+pango_item_get_properties (PangoItem      *item,
+                           ItemProperties *properties)
+{
+  GSList *tmp_list = item->analysis.extra_attrs;
+
+  properties->uline_style = PANGO_LINE_STYLE_NONE;
+  properties->uline_position = PANGO_UNDERLINE_POSITION_NORMAL;
+  properties->oline_style = PANGO_LINE_STYLE_NONE;
+  properties->strikethrough_style = PANGO_LINE_STYLE_NONE;
+  properties->showing_space = FALSE;
+  properties->no_paragraph_break = FALSE;
+  properties->letter_spacing = 0;
+  properties->line_height = 0.0;
+  properties->absolute_line_height = 0;
+  properties->line_spacing = 0;
+  properties->shape = NULL;
+
+  while (tmp_list)
+    {
+      PangoAttribute *attr = tmp_list->data;
+
+      switch ((int) attr->type)
+        {
+        case PANGO_ATTR_UNDERLINE:
+          properties->uline_style = attr->int_value;
+          break;
+
+        case PANGO_ATTR_UNDERLINE_POSITION:
+          properties->uline_position = attr->int_value;
+          break;
+
+        case PANGO_ATTR_OVERLINE:
+          properties->oline_style = attr->int_value;
+          break;
+
+        case PANGO_ATTR_STRIKETHROUGH:
+          properties->strikethrough_style = attr->int_value;
+          break;
+
+        case PANGO_ATTR_LETTER_SPACING:
+          properties->letter_spacing = attr->int_value;
+          break;
+
+        case PANGO_ATTR_LINE_HEIGHT:
+          properties->line_height = attr->double_value;
+          break;
+
+        case PANGO_ATTR_ABSOLUTE_LINE_HEIGHT:
+          properties->absolute_line_height = attr->int_value;
+          break;
+
+        case PANGO_ATTR_LINE_SPACING:
+          properties->line_spacing = attr->int_value;
+          break;
+
+        case PANGO_ATTR_SHOW:
+          properties->showing_space = (attr->int_value & PANGO_SHOW_SPACES) != 0;
+          break;
+
+        case PANGO_ATTR_PARAGRAPH:
+          properties->no_paragraph_break = TRUE;
+          break;
+
+        case PANGO_ATTR_SHAPE:
+          properties->shape = attr;
+          break;
+
+        default:
+          break;
+        }
+      tmp_list = tmp_list->next;
+    }
+}
+
+/**
+ * pango_analysis_get_font:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns the font that will be used for text
+ * with this `PangoAnalysis`.
+ *
+ * Return value: (transfer none): the `PangoFont`
+ */
+PangoFont *
+pango_analysis_get_font (const PangoAnalysis *analysis)
+{
+  return analysis->font;
+}
+
+/**
+ * pango_analysis_get_bidi_level:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns the bidi embedding level for text
+ * with this `PangoAnalysis`.
+ *
+ * Return value: the bidi embedding level
+ */
+int
+pango_analysis_get_bidi_level (const PangoAnalysis *analysis)
+{
+  return analysis->level;
+}
+
+/**
+ * pango_analysis_get_gravity:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns the gravity for text with this `PangoAnalysis`.
+ *
+ * Return value: the gravity
+ */
+PangoGravity
+pango_analysis_get_gravity (const PangoAnalysis *analysis)
+{
+  return (PangoGravity) analysis->gravity;
+}
+
+/**
+ * pango_analysis_get_flags:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns flags for this `PangoAnalysis`.
+ *
+ * Possible flag values are
+ * `PANGO_ANALYSIS_FLAG_CENTERED_BASELINE`,
+ * `PANGO_ANALYSIS_FLAG_IS_ELLIPSIS` and
+ * `PANGO_ANALYSIS_FLAG_NEED_HYPHEN`.
+ *
+ * Return value: the flags
+ */
+guint
+pango_analysis_get_flags (const PangoAnalysis *analysis)
+{
+  return analysis->flags;
+}
+
+/**
+ * pango_analysis_get_script:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns the script for text with this `PangoAnalysis`.
+ *
+ * Return value: the script
+ */
+GUnicodeScript
+pango_analysis_get_script (const PangoAnalysis *analysis)
+{
+  return (GUnicodeScript) analysis->script;
+}
+
+/**
+ * pango_analysis_get_language:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns the language for text with this `PangoAnalysis`.
+ *
+ * Return value: the script
+ */
+PangoLanguage *
+pango_analysis_get_language (const PangoAnalysis *analysis)
+{
+  return analysis->language;
+}
+
+/**
+ * pango_analysis_get_extra_attributes:
+ * @analysis: a `PangoAnalysis`
+ *
+ * Returns attributes to apply to text with this
+ * `PangoAnalysis`.
+ *
+ * Return value: (transfer none) (element-type PangoAttribute):
+ *   a `GSList` with `PangoAttribute` values
+ */
+GSList *
+pango_analysis_get_extra_attributes (const PangoAnalysis *analysis)
+{
+  return analysis->extra_attrs;
+}
+
+/**
+ * pango_item_get_analysis:
+ * @item: a `PangoItem`
+ *
+ * Returns the `PangoAnalysis` of @item.
+ *
+ * Return value: (transfer none): a `PangoAnalysis`
+ */
+const PangoAnalysis *
+pango_item_get_analysis (PangoItem *item)
+{
+  return &item->analysis;
+}
+
+/**
+ * pango_item_get_byte_offset:
+ * @item: a `PangoItem`
+ *
+ * Returns the byte offset of this items
+ * text in the overall paragraph text.
+ *
+ * Return value: the byte offset
+ */
+int
+pango_item_get_byte_offset (PangoItem *item)
+{
+  return item->offset;
+}
+
+/**
+ * pango_item_get_byte_length:
+ * @item: a `PangoItem`
+ *
+ * Returns the length of this items
+ * text in bytes.
+ *
+ * Return value: the length of @item
+ */
+int
+pango_item_get_byte_length (PangoItem *item)
+{
+  return item->length;
 }
 
 /**
  * pango_item_get_char_offset:
  * @item: a `PangoItem`
  *
- * Returns the character offset of the item from the beginning
- * of the itemized text.
+ * Returns the offset of this items text
+ * in the overall paragraph text, in characters.
  *
- * If the item has not been obtained from Pango's itemization
- * machinery, then the character offset is not available. In
- * that case, this function returns -1.
- *
- * Returns: the character offset of the item from the beginning
- *   of the itemized text, or -1
- *
- * Since: 1.54
+ * Returns value: the character offset
  */
 int
 pango_item_get_char_offset (PangoItem *item)
 {
-  if (item->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    return ((PangoItemPrivate *)item)->char_offset;
-  else
-    return -1;
+  return item->char_offset;
+}
+
+/**
+ * pango_item_get_char_length:
+ * @item: a `PangoItem`
+ *
+ * Returns the number of characters in this
+ * items text.
+ *
+ * Return value: the number of characters in @item
+ */
+int
+pango_item_get_char_length (PangoItem *item)
+{
+  return item->num_chars;
 }
