@@ -37,8 +37,6 @@ G_DEFINE_BOXED_TYPE (PangoMatrix, pango_matrix,
  * Copies a `PangoMatrix`.
  *
  * Return value: (nullable): the newly allocated `PangoMatrix`
- *
- * Since: 1.6
  */
 PangoMatrix *
 pango_matrix_copy (const PangoMatrix *matrix)
@@ -60,8 +58,6 @@ pango_matrix_copy (const PangoMatrix *matrix)
  * @matrix: (nullable): a `PangoMatrix`, may be %NULL
  *
  * Free a `PangoMatrix`.
- *
- * Since: 1.6
  */
 void
 pango_matrix_free (PangoMatrix *matrix)
@@ -81,8 +77,6 @@ pango_matrix_free (PangoMatrix *matrix)
  * Changes the transformation represented by @matrix to be the
  * transformation given by first translating by (@tx, @ty)
  * then applying the original transformation.
- *
- * Since: 1.6
  */
 void
 pango_matrix_translate (PangoMatrix *matrix,
@@ -105,8 +99,6 @@ pango_matrix_translate (PangoMatrix *matrix,
  * transformation given by first scaling by @sx in the X direction
  * and @sy in the Y direction then applying the original
  * transformation.
- *
- * Since: 1.6
  */
 void
 pango_matrix_scale (PangoMatrix *matrix,
@@ -129,15 +121,13 @@ pango_matrix_scale (PangoMatrix *matrix,
  * Changes the transformation represented by @matrix to be the
  * transformation given by first rotating by @degrees degrees
  * counter-clockwise then applying the original transformation.
- *
- * Since: 1.6
  */
 void
 pango_matrix_rotate (PangoMatrix *matrix,
 		     double       degrees)
 {
   PangoMatrix tmp;
-  gdouble r, s, c;
+  double r, s, c;
 
   g_return_if_fail (matrix != NULL);
 
@@ -163,8 +153,6 @@ pango_matrix_rotate (PangoMatrix *matrix,
  * Changes the transformation represented by @matrix to be the
  * transformation given by first applying transformation
  * given by @new_matrix then applying the original transformation.
- *
- * Since: 1.6
  */
 void
 pango_matrix_concat (PangoMatrix       *matrix,
@@ -196,8 +184,6 @@ pango_matrix_concat (PangoMatrix       *matrix,
  *
  * Return value: the scale factor of @matrix on the height of the font,
  *   or 1.0 if @matrix is %NULL.
- *
- * Since: 1.12
  */
 double
 pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
@@ -220,8 +206,6 @@ pango_matrix_get_font_scale_factor (const PangoMatrix *matrix)
  * vector that the X coordinate is mapped to.
  *
  * Note that output numbers will always be non-negative.
- *
- * Since: 1.38
  **/
 void
 pango_matrix_get_font_scale_factors (const PangoMatrix *matrix,
@@ -262,6 +246,32 @@ pango_matrix_get_font_scale_factors (const PangoMatrix *matrix,
     *yscale = minor;
 }
 
+#define RAD_TO_DEG(x) ((x)/G_PI * 180)
+
+/**
+ * pango_matrix_get_rotation:
+ * @matrix: a `PangoMatrix`
+ *
+ * Returns the angle (in degrees) that this
+ * matrix rotates the X axis by.
+ *
+ * For font matrices, this is typically zero.
+ *
+ * Returns: the rotation of @matrix
+ */
+double
+pango_matrix_get_rotation (const PangoMatrix *matrix)
+{
+  double x, y;
+
+  x = 1;
+  y = 0;
+
+  pango_matrix_transform_distance (matrix, &x, &y);
+
+  return RAD_TO_DEG (acos (x /  sqrtf (x*x + y*y)));
+}
+
 /**
  * pango_matrix_get_slant_ratio:
  * @matrix: a `PangoMatrix`
@@ -276,24 +286,25 @@ pango_matrix_get_font_scale_factors (const PangoMatrix *matrix,
  * this is simply λ.
  *
  * Returns: the slant ratio of @matrix
- *
- * Since: 1.50
  */
 double
 pango_matrix_get_slant_ratio (const PangoMatrix *matrix)
 {
-  double x0, y0;
-  double x1, y1;
+  if (matrix)
+    {
+      double a = matrix->xx;
+      double b = matrix->xy;
+      double c = matrix->yx;
+      double d = matrix->yy;
 
-  x0 = 0;
-  y0 = 1;
-  pango_matrix_transform_distance (matrix, &x0, &y0);
+      if (c != 0 || d != 0)
+        {
+          double s = sqrtf (c * c + d * d);
+          return (a*c + b*d) / (s*s);
+        }
+    }
 
-  x1 = 1;
-  y1 = 0;
-  pango_matrix_transform_distance (matrix, &x1, &y1);
-
-  return (x0 * x1 + y0 * y1) / (x0 * x0 + y0 * y0);
+  return 0;
 }
 
 /**
@@ -317,8 +328,6 @@ pango_matrix_get_slant_ratio (const PangoMatrix *matrix)
  * always transforms to the same vector. If (@x1,@y1) transforms
  * to (@x2,@y2) then (@x1+@dx1,@y1+@dy1) will transform to
  * (@x1+@dx2,@y1+@dy2) for all values of @x1 and @x2.
- *
- * Since: 1.16
  */
 void
 pango_matrix_transform_distance (const PangoMatrix *matrix,
@@ -344,9 +353,7 @@ pango_matrix_transform_distance (const PangoMatrix *matrix,
  * @y: (inout): in/out Y position
  *
  * Transforms the point (@x, @y) by @matrix.
- *
- * Since: 1.16
- **/
+ */
 void
 pango_matrix_transform_point (const PangoMatrix *matrix,
 			      double            *x,
@@ -384,8 +391,6 @@ pango_matrix_transform_point (const PangoMatrix *matrix,
  * to pixels first and then transform, for example when the transformed
  * coordinates may overflow in Pango units (large matrix translation for
  * example).
- *
- * Since: 1.16
  */
 void
 pango_matrix_transform_rectangle (const PangoMatrix *matrix,
@@ -456,8 +461,6 @@ pango_matrix_transform_rectangle (const PangoMatrix *matrix,
  * For better accuracy, you should use [method@Pango.Matrix.transform_rectangle]
  * on original rectangle in Pango units and convert to pixels afterward
  * using [func@extents_to_pixels]'s first argument.
- *
- * Since: 1.16
  */
 void
 pango_matrix_transform_pixel_rectangle (const PangoMatrix *matrix,
@@ -513,3 +516,12 @@ pango_matrix_transform_pixel_rectangle (const PangoMatrix *matrix,
   rect->width  = ceil (max_x - rect->x);
   rect->height = ceil (max_y - rect->y);
 }
+
+gboolean
+pango_matrix_equal (const PangoMatrix *m1,
+                    const PangoMatrix *m2)
+{
+  return m1->xx == m2->xx && m1->xy == m2->xy &&
+         m1->yx == m2->yx && m1->yy == m2->yy &&
+         m1->x0 == m2->x0 && m1->y0 == m2->y0;
+}
