@@ -1,27 +1,27 @@
-/* Pango
- * pango-renderer.h: Base class for rendering
- *
+/*
  * Copyright (C) 2004, Red Hat, Inc.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
-#ifndef __PANGO_RENDERER_H_
-#define __PANGO_RENDERER_H_
+
+#pragma once
 
 #include <pango/pango-layout.h>
+#include <pango/pango-lines.h>
+#include <pango/pango-glyph.h>
 
 G_BEGIN_DECLS
 
@@ -34,7 +34,6 @@ G_BEGIN_DECLS
 
 typedef struct _PangoRenderer        PangoRenderer;
 typedef struct _PangoRendererClass   PangoRendererClass;
-typedef struct _PangoRendererPrivate PangoRendererPrivate;
 
 /**
  * PangoRenderPart:
@@ -44,11 +43,9 @@ typedef struct _PangoRendererPrivate PangoRendererPrivate;
  * @PANGO_RENDER_PART_STRIKETHROUGH: strikethrough lines
  * @PANGO_RENDER_PART_OVERLINE: overlines
  *
- * #PangoRenderPart defines different items to render for such
+ * `PangoRenderPart` defines different items to render for such
  * purposes as setting colors.
- *
- * Since: 1.8
- **/
+ */
 /* When extending, note N_RENDER_PARTS #define in pango-renderer.c */
 typedef enum
 {
@@ -61,43 +58,26 @@ typedef enum
 
 /**
  * PangoRenderer:
- * @matrix: (nullable): the current transformation matrix for
- *    the Renderer; may be %NULL, which should be treated the
- *    same as the identity matrix.
  *
- * #PangoRenderer is a base class for objects that are used to
- * render Pango objects such as #PangoGlyphString and
- * #PangoLayout.
+ * `PangoRenderer` is a base class for objects that can render text
+ * provided as `PangoGlyphString` or `PangoLayout`.
  *
- * Since: 1.8
- **/
+ * By subclassing `PangoRenderer` and overriding operations such as
+ * @draw_glyphs and @draw_rectangle, renderers for particular font
+ * backends and destinations can be created.
+ */
 struct _PangoRenderer
 {
-  /*< private >*/
   GObject parent_instance;
-
-  PangoUnderline underline;
-  PangoOverline overline;
-  gboolean strikethrough;
-  int active_count;
-
-  /*< public >*/
-  PangoMatrix *matrix;          /* May be NULL */
-
-  /*< private >*/
-  PangoRendererPrivate *priv;
 };
 
 /**
  * PangoRendererClass:
- * @draw_glyphs: draws a #PangoGlyphString
+ * @draw_glyphs: draws a `PangoGlyphString`
  * @draw_rectangle: draws a rectangle
  * @draw_error_underline: draws a squiggly line that approximately
  * covers the given rectangle in the style of an underline used to
  * indicate a spelling error.
- * @draw_shape: draw content for a glyph shaped with #PangoAttrShape.
- *   @x, @y are the coordinates of the left edge of the baseline,
- *   in user coordinates.
  * @draw_trapezoid: draws a trapezoidal filled area
  * @draw_glyph: draws a single glyph
  * @part_changed: do renderer specific processing when rendering
@@ -105,9 +85,9 @@ struct _PangoRenderer
  * @begin: Do renderer-specific initialization before drawing
  * @end: Do renderer-specific cleanup after drawing
  * @prepare_run: updates the renderer for a new run
- * @draw_glyph_item: draws a #PangoGlyphItem
+ * @draw_glyph_item: draws a `PangoGlyphItem`
  *
- * Class structure for #PangoRenderer.
+ * Class structure for `PangoRenderer`.
  *
  * The following vfuncs take user space coordinates in Pango units
  * and have default implementations:
@@ -123,15 +103,12 @@ struct _PangoRenderer
  * and must be implemented:
  * - draw_trapezoid
  * - draw_glyph
- *
- * Since: 1.8
  */
 struct _PangoRendererClass
 {
   /*< private >*/
   GObjectClass parent_class;
 
-  /* vtable - not signals */
   /*< public >*/
 
   void (*draw_glyphs)          (PangoRenderer    *renderer,
@@ -151,10 +128,11 @@ struct _PangoRendererClass
                                 int               width,
                                 int               height);
   void (*draw_shape)           (PangoRenderer    *renderer,
-                                PangoAttrShape   *attr,
+                                PangoRectangle   *ink_rect,
+                                PangoRectangle   *logical_rect,
+                                gpointer          data,
                                 int               x,
                                 int               y);
-
   void (*draw_trapezoid)       (PangoRenderer    *renderer,
                                 PangoRenderPart   part,
                                 double            y1_,
@@ -176,61 +154,60 @@ struct _PangoRendererClass
   void (*end)                  (PangoRenderer    *renderer);
 
   void (*prepare_run)          (PangoRenderer    *renderer,
-                                PangoLayoutRun   *run);
+                                PangoRun         *run);
 
-  void (*draw_glyph_item)      (PangoRenderer    *renderer,
+  void (*draw_run)             (PangoRenderer    *renderer,
                                 const char       *text,
-                                PangoGlyphItem   *glyph_item,
+                                PangoRun         *run,
                                 int               x,
                                 int               y);
 
   /*< private >*/
 
   /* Padding for future expansion */
-  void (*_pango_reserved2) (void);
-  void (*_pango_reserved3) (void);
-  void (*_pango_reserved4) (void);
+  gpointer _pango_reserved[8];
 };
 
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 GType pango_renderer_get_type            (void) G_GNUC_CONST;
 
-PANGO_AVAILABLE_IN_1_8
-void pango_renderer_draw_layout          (PangoRenderer    *renderer,
-                                          PangoLayout      *layout,
+PANGO_AVAILABLE_IN_ALL
+void pango_renderer_draw_lines           (PangoRenderer    *renderer,
+                                          PangoLines       *lines,
                                           int               x,
                                           int               y);
-PANGO_AVAILABLE_IN_1_8
-void pango_renderer_draw_layout_line     (PangoRenderer    *renderer,
-                                          PangoLayoutLine  *line,
+
+PANGO_AVAILABLE_IN_ALL
+void pango_renderer_draw_line            (PangoRenderer    *renderer,
+                                          PangoLine        *line,
                                           int               x,
                                           int               y);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_draw_glyphs          (PangoRenderer    *renderer,
                                           PangoFont        *font,
                                           PangoGlyphString *glyphs,
                                           int               x,
                                           int               y);
-PANGO_AVAILABLE_IN_1_22
-void pango_renderer_draw_glyph_item      (PangoRenderer    *renderer,
+PANGO_AVAILABLE_IN_ALL
+void pango_renderer_draw_run             (PangoRenderer    *renderer,
                                           const char       *text,
-                                          PangoGlyphItem   *glyph_item,
+                                          PangoRun         *run,
                                           int               x,
                                           int               y);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_draw_rectangle       (PangoRenderer    *renderer,
                                           PangoRenderPart   part,
                                           int               x,
                                           int               y,
                                           int               width,
                                           int               height);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_draw_error_underline (PangoRenderer    *renderer,
                                           int               x,
                                           int               y,
                                           int               width,
                                           int               height);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_draw_trapezoid       (PangoRenderer    *renderer,
                                           PangoRenderPart   part,
                                           double            y1_,
@@ -239,50 +216,43 @@ void pango_renderer_draw_trapezoid       (PangoRenderer    *renderer,
                                           double            y2,
                                           double            x12,
                                           double            x22);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_draw_glyph           (PangoRenderer    *renderer,
                                           PangoFont        *font,
                                           PangoGlyph        glyph,
                                           double            x,
                                           double            y);
 
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_activate             (PangoRenderer    *renderer);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_deactivate           (PangoRenderer    *renderer);
 
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void pango_renderer_part_changed         (PangoRenderer   *renderer,
                                           PangoRenderPart  part);
 
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void        pango_renderer_set_color     (PangoRenderer    *renderer,
                                           PangoRenderPart   part,
                                           const PangoColor *color);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 PangoColor *pango_renderer_get_color     (PangoRenderer    *renderer,
                                           PangoRenderPart   part);
 
-PANGO_AVAILABLE_IN_1_38
-void        pango_renderer_set_alpha     (PangoRenderer    *renderer,
-                                          PangoRenderPart   part,
-                                          guint16           alpha);
-PANGO_AVAILABLE_IN_1_38
-guint16     pango_renderer_get_alpha     (PangoRenderer    *renderer,
-                                          PangoRenderPart   part);
-
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 void               pango_renderer_set_matrix      (PangoRenderer     *renderer,
                                                    const PangoMatrix *matrix);
-PANGO_AVAILABLE_IN_1_8
+PANGO_AVAILABLE_IN_ALL
 const PangoMatrix *pango_renderer_get_matrix      (PangoRenderer     *renderer);
 
-PANGO_AVAILABLE_IN_1_20
-PangoLayout       *pango_renderer_get_layout      (PangoRenderer     *renderer);
-PANGO_AVAILABLE_IN_1_20
-PangoLayoutLine   *pango_renderer_get_layout_line (PangoRenderer     *renderer);
+PANGO_AVAILABLE_IN_ALL
+PangoLines *       pango_renderer_get_lines       (PangoRenderer     *renderer);
+
+PANGO_AVAILABLE_IN_ALL
+PangoLine *        pango_renderer_get_layout_line (PangoRenderer     *renderer);
+
+PANGO_AVAILABLE_IN_ALL
+PangoContext *     pango_renderer_get_context     (PangoRenderer     *renderer);
 
 G_END_DECLS
-
-#endif /* __PANGO_RENDERER_H_ */
-
