@@ -22,10 +22,15 @@
 #include "config.h"
 
 #include <pango/pango-layout.h>
-#include <pango/pango-layout-private.h>
 #include <pango/pango-context-private.h>
 #include <pango/pango-enum-types.h>
 #include <pango/pango-font-private.h>
+#include <pango/pango-line-private.h>
+#include <pango/pango-hbface.h>
+#include <pango/pango-hbfont.h>
+#include <pango/pango-attributes.h>
+#include <pango/pango-attr-private.h>
+#include <pango/pango-item-private.h>
 
 #include <hb-ot.h>
 #include "pango/json/gtkjsonparserprivate.h"
@@ -69,21 +74,19 @@ static const char *stretch_names[] = {
   NULL
 };
 
-static const char *underline_names[] = {
+static const char *line_style_names[] = {
   "none",
   "single",
   "double",
-  "low",
-  "error",
-  "single-line",
-  "double-line",
-  "error-line",
+  "dashed",
+  "dotted",
+  "wavy",
   NULL
 };
 
-static const char *overline_names[] = {
-  "none",
-  "single",
+static const char *underline_position_names[] = {
+  "normal",
+  "under",
   NULL
 };
 
@@ -162,56 +165,42 @@ get_weight_name (int weight)
   return NULL;
 }
 
-static const char *attr_type_names[] = {
-  "invalid",
-  "language",
-  "family",
-  "style",
-  "weight",
-  "variant",
-  "stretch",
-  "size",
-  "font-desc",
-  "foreground",
-  "background",
-  "underline",
-  "strikethrough",
-  "rise",
-  "shape",
-  "scale",
-  "fallback",
-  "letter-spacing",
-  "underline-color",
-  "strikethrough-color",
-  "absolute-size",
-  "gravity",
-  "gravity-hint",
-  "font-features",
-  "foreground-alpha",
-  "background-alpha",
-  "allow-breaks",
-  "show",
-  "insert-hyphens",
-  "overline",
-  "overline-color",
-  "line-height",
-  "absolute-line-height",
-  "text-transform",
-  "word",
-  "sentence",
-  "baseline-shift",
-  "font-scale",
-  NULL
-};
-
-static const char *
-get_script_name (PangoScript script)
+static PangoAttrType
+get_attr_type (const char *nick)
 {
   GEnumClass *enum_class;
-  GEnumValue *enum_value;
+  GEnumValue *enum_value = NULL;
 
-  enum_class = g_type_class_ref (PANGO_TYPE_SCRIPT);
-  enum_value = g_enum_get_value (enum_class, script);
+  enum_class = g_type_class_ref (PANGO_TYPE_ATTR_TYPE);
+  for (int i = 0; i < enum_class->n_values; i++)
+    {
+      enum_value = &enum_class->values[i];
+      if (strcmp (enum_value->value_nick, nick) == 0)
+        break;
+      enum_value = NULL;
+    }
+  g_type_class_unref (enum_class);
+
+  if (enum_value)
+    return enum_value->value;
+
+  return 0;
+}
+
+static const char *
+get_attr_type_name (PangoAttrType type)
+{
+  GEnumClass *enum_class;
+  GEnumValue *enum_value = NULL;
+
+  enum_class = g_type_class_ref (PANGO_TYPE_ATTR_TYPE);
+  for (int i = 0; i < enum_class->n_values; i++)
+    {
+      enum_value = &enum_class->values[i];
+      if (enum_value->value == type)
+        break;
+      enum_value = NULL;
+    }
   g_type_class_unref (enum_class);
 
   if (enum_value)
@@ -220,6 +209,14 @@ get_script_name (PangoScript script)
   return NULL;
 }
 
+static void
+get_script_name (GUnicodeScript  script,
+                 char           *buf)
+{
+  guint32 tag = g_unicode_script_to_iso15924 (script);
+  hb_tag_to_string (tag, buf);
+}
+
 static const char *tab_align_names[] = {
   "left",
   "right",
@@ -239,13 +236,6 @@ static const char *direction_names[] = {
   NULL
 };
 
-static const char *alignment_names[] = {
-  "left",
-  "center",
-  "right",
-  NULL
-};
-
 static const char *wrap_names[] = {
   "word",
   "char",
@@ -253,6 +243,15 @@ static const char *wrap_names[] = {
   NULL
 };
 
+static const char *alignment_names[] = {
+  "left",
+  "center",
+  "right",
+  "natural",
+  "justify",
+  NULL
+};
+
 static const char *ellipsize_names[] = {
   "none",
   "start",
@@ -264,6 +263,23 @@ static const char *ellipsize_names[] = {
 /* }}} */
 /* {{{ Serialization */
 
+static char *
+font_description_to_string (PangoFontDescription *desc)
+{
+  PangoFontDescription *copy;
+  char *s;
+
+  /* Leave out the faceid for now, since it would make serialization
+   * backend-dependent.
+   */
+  copy = pango_font_description_copy_static (desc);
+  pango_font_description_unset_fields (copy, PANGO_FONT_MASK_FACEID);
+  s = pango_font_description_to_string (copy);
+  pango_font_description_free (copy);
+
+  return s;
+}
+
 static void
 add_attribute (GtkJsonPrinter *printer,
                PangoAttribute *attr)
@@ -276,114 +292,108 @@ add_attribute (GtkJsonPrinter *printer,
     gtk_json_printer_add_integer (printer, "start", (int)attr->start_index);
   if (attr->end_index != PANGO_ATTR_INDEX_TO_TEXT_END)
     gtk_json_printer_add_integer (printer, "end", (int)attr->end_index);
-  gtk_json_printer_add_string (printer, "type", attr_type_names[attr->klass->type]);
+  gtk_json_printer_add_string (printer, "type", get_attr_type_name (attr->type));
 
-  switch (attr->klass->type)
+  switch (PANGO_ATTR_VALUE_TYPE (attr))
     {
+    case PANGO_ATTR_VALUE_STRING:
+      gtk_json_printer_add_string (printer, "value", attr->str_value);
+      break;
+
+    case PANGO_ATTR_VALUE_INT:
+      switch ((int)attr->type)
+        {
+        case PANGO_ATTR_STYLE:
+          gtk_json_printer_add_string (printer, "value", style_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_VARIANT:
+          gtk_json_printer_add_string (printer, "value", variant_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_STRETCH:
+          gtk_json_printer_add_string (printer, "value", stretch_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_UNDERLINE:
+        case PANGO_ATTR_STRIKETHROUGH:
+        case PANGO_ATTR_OVERLINE:
+          gtk_json_printer_add_string (printer, "value", line_style_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_UNDERLINE_POSITION:
+          gtk_json_printer_add_string (printer, "value", underline_position_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_GRAVITY:
+          gtk_json_printer_add_string (printer, "value", gravity_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_GRAVITY_HINT:
+          gtk_json_printer_add_string (printer, "value", gravity_hint_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_TEXT_TRANSFORM:
+          gtk_json_printer_add_string (printer, "value", text_transform_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_FONT_SCALE:
+          gtk_json_printer_add_string (printer, "value", font_scale_names[attr->int_value]);
+          break;
+
+        case PANGO_ATTR_WEIGHT:
+          {
+            const char *name = get_weight_name (attr->int_value);
+            if (name)
+              gtk_json_printer_add_string (printer, "value", name);
+            else
+              gtk_json_printer_add_integer (printer, "value", attr->int_value);
+          }
+          break;
+
+        case PANGO_ATTR_BASELINE_SHIFT:
+          gtk_json_printer_add_string (printer, "value", baseline_shift_names[attr->int_value]);
+          break;
+
+        default:
+          gtk_json_printer_add_integer (printer, "value", attr->int_value);
+          break;
+        }
+
+      break;
+
+    case PANGO_ATTR_VALUE_BOOLEAN:
+      gtk_json_printer_add_boolean (printer, "value", attr->boolean_value);
+      break;
+
+    case PANGO_ATTR_VALUE_LANGUAGE:
+      gtk_json_printer_add_string (printer, "value", pango_language_to_string (attr->lang_value));
+      break;
+
+    case PANGO_ATTR_VALUE_FONT_DESC:
+      str = font_description_to_string (attr->font_value);
+      gtk_json_printer_add_string (printer, "value", str);
+      g_free (str);
+      break;
+
+    case PANGO_ATTR_VALUE_COLOR:
+      str = pango_color_to_string (&attr->color_value);
+      gtk_json_printer_add_string (printer, "value", str);
+      g_free (str);
+      break;
+
+    case PANGO_ATTR_VALUE_FLOAT:
+      gtk_json_printer_add_number (printer, "value", attr->double_value);
+      break;
+
+    case PANGO_ATTR_VALUE_POINTER:
+      str = pango_attr_value_serialize (attr);
+      gtk_json_printer_add_string (printer, "value", str);
+      g_free (str);
+      break;
+
     default:
-    case PANGO_ATTR_INVALID:
       g_assert_not_reached ();
-    case PANGO_ATTR_LANGUAGE:
-      gtk_json_printer_add_string (printer, "value", pango_language_to_string (((PangoAttrLanguage*)attr)->value));
-      break;
-    case PANGO_ATTR_FAMILY:
-    case PANGO_ATTR_FONT_FEATURES:
-      gtk_json_printer_add_string (printer, "value", ((PangoAttrString*)attr)->value);
-      break;
-    case PANGO_ATTR_STYLE:
-      gtk_json_printer_add_string (printer, "value", style_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_VARIANT:
-      gtk_json_printer_add_string (printer, "value", variant_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_STRETCH:
-      gtk_json_printer_add_string (printer, "value", stretch_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_UNDERLINE:
-      gtk_json_printer_add_string (printer, "value", underline_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_OVERLINE:
-      gtk_json_printer_add_string (printer, "value", overline_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_GRAVITY:
-      gtk_json_printer_add_string (printer, "value", gravity_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_GRAVITY_HINT:
-      gtk_json_printer_add_string (printer, "value", gravity_hint_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_TEXT_TRANSFORM:
-      gtk_json_printer_add_string (printer, "value", text_transform_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_FONT_SCALE:
-      gtk_json_printer_add_string (printer, "value", font_scale_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-    case PANGO_ATTR_WEIGHT:
-      {
-        const char *name = get_weight_name (((PangoAttrInt*)attr)->value);
-        if (name)
-          gtk_json_printer_add_string (printer, "value", name);
-        else
-          gtk_json_printer_add_integer (printer, "value", ((PangoAttrInt*)attr)->value);
-      }
-      break;
-
-    case PANGO_ATTR_BASELINE_SHIFT:
-      gtk_json_printer_add_string (printer, "value", baseline_shift_names[((PangoAttrInt*)attr)->value]);
-      break;
-
-
-    case PANGO_ATTR_SIZE:
-    case PANGO_ATTR_RISE:
-    case PANGO_ATTR_LETTER_SPACING:
-    case PANGO_ATTR_ABSOLUTE_SIZE:
-    case PANGO_ATTR_FOREGROUND_ALPHA:
-    case PANGO_ATTR_BACKGROUND_ALPHA:
-    case PANGO_ATTR_SHOW:
-    case PANGO_ATTR_WORD:
-    case PANGO_ATTR_SENTENCE:
-    case PANGO_ATTR_ABSOLUTE_LINE_HEIGHT:
-      gtk_json_printer_add_integer (printer, "value", ((PangoAttrInt*)attr)->value);
-      break;
-
-    case PANGO_ATTR_FONT_DESC:
-      str = pango_font_description_to_string (((PangoAttrFontDesc*)attr)->desc);
-      gtk_json_printer_add_string (printer, "value", str);
-      g_free (str);
-      break;
-
-    case PANGO_ATTR_FOREGROUND:
-    case PANGO_ATTR_BACKGROUND:
-    case PANGO_ATTR_UNDERLINE_COLOR:
-    case PANGO_ATTR_OVERLINE_COLOR:
-    case PANGO_ATTR_STRIKETHROUGH_COLOR:
-      str = pango_color_to_string (&((PangoAttrColor*)attr)->color);
-      gtk_json_printer_add_string (printer, "value", str);
-      g_free (str);
-      break;
-
-    case PANGO_ATTR_STRIKETHROUGH:
-    case PANGO_ATTR_FALLBACK:
-    case PANGO_ATTR_ALLOW_BREAKS:
-    case PANGO_ATTR_INSERT_HYPHENS:
-      gtk_json_printer_add_boolean (printer, "value", ((PangoAttrInt*)attr)->value != 0);
-      break;
-
-    case PANGO_ATTR_SHAPE:
-      gtk_json_printer_add_string (printer, "value", "shape");
-      break;
-
-    case PANGO_ATTR_SCALE:
-    case PANGO_ATTR_LINE_HEIGHT:
-      gtk_json_printer_add_number (printer, "value", ((PangoAttrFloat*)attr)->value);
     }
 
   gtk_json_printer_end (printer);
@@ -457,7 +467,7 @@ add_context (GtkJsonPrinter *printer,
    * context gets updated as expected.
    */
 
-  str = pango_font_description_to_string (context->font_desc);
+  str = font_description_to_string (context->font_desc);
   gtk_json_printer_add_string (printer, "font", str);
   g_free (str);
 
@@ -486,15 +496,12 @@ add_context (GtkJsonPrinter *printer,
 }
 
 static void
-add_log_attrs (GtkJsonPrinter *printer,
-               PangoLayout    *layout)
+add_log_attrs (GtkJsonPrinter      *printer,
+               const PangoLogAttr  *log_attrs,
+               int                  n_attrs)
 {
-  const PangoLogAttr *log_attrs;
-  int n_attrs;
-
   gtk_json_printer_start_array (printer, "log-attrs");
 
-  log_attrs = pango_layout_get_log_attrs_readonly (layout, &n_attrs);
   for (int i = 0; i < n_attrs; i++)
     {
       gtk_json_printer_start_object (printer, NULL);
@@ -547,13 +554,13 @@ add_font (GtkJsonPrinter *printer,
   const char *data;
   guint length;
   const int *coords;
-  hb_feature_t features[32];
+  const hb_feature_t *features;
   PangoMatrix matrix;
 
   gtk_json_printer_start_object (printer, member);
 
   desc = pango_font_describe (font);
-  str = pango_font_description_to_string (desc);
+  str = font_description_to_string (desc);
   gtk_json_printer_add_string (printer, "description", str);
   g_free (str);
   pango_font_description_free (desc);
@@ -596,7 +603,8 @@ add_font (GtkJsonPrinter *printer,
     }
 
   length = 0;
-  pango_font_get_features (font, features, G_N_ELEMENTS (features), &length);
+  if (PANGO_IS_HB_FONT (font))
+    features = pango_hb_font_get_features (PANGO_HB_FONT (font), &length);
   if (length > 0)
     {
       gtk_json_printer_start_object (printer, "features");
@@ -612,8 +620,8 @@ add_font (GtkJsonPrinter *printer,
       gtk_json_printer_end (printer);
     }
 
-  pango_font_get_matrix (font, &matrix);
-  if (memcmp (&matrix, &(PangoMatrix)PANGO_MATRIX_INIT, sizeof (PangoMatrix)) != 0)
+  pango_font_get_transform (font, &matrix);
+  if (!pango_matrix_equal (&matrix, &(const PangoMatrix)PANGO_MATRIX_INIT))
     {
       gtk_json_printer_start_array (printer, "matrix");
       gtk_json_printer_add_number (printer, NULL, matrix.xx);
@@ -634,27 +642,28 @@ add_font (GtkJsonPrinter *printer,
 
 static void
 add_run (GtkJsonPrinter *printer,
-         PangoLayout    *layout,
-         PangoLayoutRun *run)
+         const char     *text,
+         PangoGlyphItem *run)
 {
   char *str;
+  char buf[5] = { 0, };
 
   gtk_json_printer_start_object (printer, NULL);
 
   gtk_json_printer_add_integer (printer, "offset", run->item->offset);
   gtk_json_printer_add_integer (printer, "length", run->item->length);
 
-  str = g_strndup (layout->text + run->item->offset, run->item->length);
+  str = g_strndup (text + run->item->offset, run->item->length);
   gtk_json_printer_add_string (printer, "text", str);
   g_free (str);
 
   gtk_json_printer_add_integer (printer, "bidi-level", run->item->analysis.level);
   gtk_json_printer_add_string (printer, "gravity", gravity_names[run->item->analysis.gravity]);
   gtk_json_printer_add_string (printer, "language", pango_language_to_string (run->item->analysis.language));
-  gtk_json_printer_add_string (printer, "script", get_script_name (run->item->analysis.script));
+  get_script_name (run->item->analysis.script, buf);
+  gtk_json_printer_add_string (printer, "script", buf);
 
-  if (run->item->analysis.font)
-    add_font (printer, "font", run->item->analysis.font);
+  add_font (printer, "font", run->item->analysis.font);
 
   gtk_json_printer_add_integer (printer, "flags", run->item->analysis.flags & ANALYSIS_FLAGS);
 
@@ -708,50 +717,73 @@ add_run (GtkJsonPrinter *printer,
 #undef ANALYSIS_FLAGS
 
 static void
-add_line (GtkJsonPrinter  *printer,
-          PangoLayoutLine *line)
+line_to_json (GtkJsonPrinter  *printer,
+              PangoLine       *line,
+              int              x,
+              int              y)
 {
   gtk_json_printer_start_object (printer, NULL);
 
+  gtk_json_printer_start_array (printer, "position");
+  gtk_json_printer_add_number (printer, NULL, x);
+  gtk_json_printer_add_number (printer, NULL, y);
+  gtk_json_printer_end (printer);
+
+  gtk_json_printer_start_object (printer, "line");
+
   gtk_json_printer_add_integer (printer, "start-index", line->start_index);
   gtk_json_printer_add_integer (printer, "length", line->length);
-  gtk_json_printer_add_boolean (printer, "paragraph-start", line->is_paragraph_start);
-  gtk_json_printer_add_string (printer, "direction", direction_names[line->resolved_dir]);
+  gtk_json_printer_add_integer (printer, "start-offset", line->start_offset);
+  gtk_json_printer_add_integer (printer, "n-chars", line->n_chars);
+
+  gtk_json_printer_add_boolean (printer, "wrapped", line->wrapped);
+  gtk_json_printer_add_boolean (printer, "ellipsized", line->ellipsized);
+  gtk_json_printer_add_boolean (printer, "hyphenated", line->hyphenated);
+  gtk_json_printer_add_boolean (printer, "justified", line->justified);
+  gtk_json_printer_add_boolean (printer, "paragraph-start", line->starts_paragraph);
+  gtk_json_printer_add_boolean (printer, "paragraph-end", line->ends_paragraph);
+  gtk_json_printer_add_string (printer, "direction", direction_names[line->direction]);
 
   gtk_json_printer_start_array (printer, "runs");
   for (GSList *l = line->runs; l; l = l->next)
     {
-      PangoLayoutRun *run = l->data;
-      add_run (printer, line->layout, run);
+      PangoGlyphItem *run = l->data;
+      add_run (printer, line->data->text, run);
     }
   gtk_json_printer_end (printer);
 
   gtk_json_printer_end (printer);
+
+  gtk_json_printer_end (printer);
 }
 
 static void
-add_output (GtkJsonPrinter *printer,
-            PangoLayout    *layout)
+lines_to_json (GtkJsonPrinter *printer,
+               PangoLines     *lines)
 {
   int width, height;
+  PangoLine **l;
 
   gtk_json_printer_start_object (printer, "output");
 
-  gtk_json_printer_add_boolean (printer, "is-wrapped", pango_layout_is_wrapped (layout));
-  gtk_json_printer_add_boolean (printer, "is-ellipsized", pango_layout_is_ellipsized (layout));
-  gtk_json_printer_add_integer (printer, "unknown-glyphs", pango_layout_get_unknown_glyphs_count (layout));
-
-  pango_layout_get_size (layout, &width, &height);
+  gtk_json_printer_add_boolean (printer, "wrapped", pango_lines_is_wrapped (lines));
+  gtk_json_printer_add_boolean (printer, "ellipsized", pango_lines_is_ellipsized (lines));
+  gtk_json_printer_add_boolean (printer, "hypenated", pango_lines_is_hyphenated (lines));
+  gtk_json_printer_add_integer (printer, "unknown-glyphs", pango_lines_get_unknown_glyphs_count (lines));
+  pango_lines_get_size (lines, &width, &height);
   gtk_json_printer_add_integer (printer, "width", width);
   gtk_json_printer_add_integer (printer, "height", height);
 
-  add_log_attrs (printer, layout);
   gtk_json_printer_start_array (printer, "lines");
-  for (GSList *l = layout->lines; l; l = l->next)
+
+  l = pango_lines_get_lines (lines);
+  for (int i = 0; i < pango_lines_get_line_count (lines); i++)
     {
-      PangoLayoutLine *line = l->data;
-      add_line (printer, line);
+      int x, y;
+      pango_lines_get_line_position (lines, i, &x, &y);
+      line_to_json (printer, l[i], x, y);
     }
+
   gtk_json_printer_end (printer);
 
   gtk_json_printer_end (printer);
@@ -767,63 +799,62 @@ layout_to_json (GtkJsonPrinter            *printer,
   gtk_json_printer_start_object (printer, NULL);
 
   if (flags & PANGO_LAYOUT_SERIALIZE_CONTEXT)
-    add_context (printer, layout->context);
+    add_context (printer, pango_layout_get_context (layout));
 
   str = (const char *) g_object_get_data (G_OBJECT (layout), "comment");
   if (str)
     gtk_json_printer_add_string (printer, "comment", str);
 
-  gtk_json_printer_add_string (printer, "text", layout->text);
+  gtk_json_printer_add_string (printer, "text", pango_layout_get_text (layout));
 
-  add_attr_list (printer, layout->attrs);
+  add_attr_list (printer, pango_layout_get_attributes (layout));
 
-  if (layout->font_desc)
+  if (pango_layout_get_font_description (layout))
     {
-      char *str = pango_font_description_to_string (layout->font_desc);
+      char *str = pango_font_description_to_string (pango_layout_get_font_description (layout));
       gtk_json_printer_add_string (printer, "font", str);
       g_free (str);
     }
 
-  add_tab_array (printer, layout->tabs);
+  add_tab_array (printer, pango_layout_get_tabs (layout));
 
-  if (layout->justify)
-    gtk_json_printer_add_boolean (printer, "justify", TRUE);
-
-  if (layout->justify_last_line)
-    gtk_json_printer_add_boolean (printer, "justify-last-line", TRUE);
-
-  if (layout->single_paragraph)
-    gtk_json_printer_add_boolean (printer, "single-paragraph", TRUE);
-
-  if (!layout->auto_dir)
+  if (!pango_layout_get_auto_dir (layout))
     gtk_json_printer_add_boolean (printer, "auto-dir", FALSE);
 
-  if (layout->alignment != PANGO_ALIGN_LEFT)
-    gtk_json_printer_add_string (printer, "alignment", alignment_names[layout->alignment]);
+  if (pango_layout_get_alignment (layout) != PANGO_ALIGN_NATURAL)
+    gtk_json_printer_add_string (printer, "alignment", alignment_names[pango_layout_get_alignment (layout)]);
 
-  if (layout->wrap != PANGO_WRAP_WORD)
-    gtk_json_printer_add_string (printer, "wrap", wrap_names[layout->wrap]);
+  if (pango_layout_get_wrap (layout) != PANGO_WRAP_WORD)
+    gtk_json_printer_add_string (printer, "wrap", wrap_names[pango_layout_get_wrap (layout)]);
 
-  if (layout->ellipsize != PANGO_ELLIPSIZE_NONE)
-    gtk_json_printer_add_string (printer, "ellipsize", ellipsize_names[layout->ellipsize]);
+  if (pango_layout_get_ellipsize (layout) != PANGO_ELLIPSIZE_NONE)
+    gtk_json_printer_add_string (printer, "ellipsize", ellipsize_names[pango_layout_get_ellipsize (layout)]);
 
-  if (layout->width != -1)
-    gtk_json_printer_add_integer (printer, "width", layout->width);
+  if (pango_layout_get_width (layout) != -1)
+    gtk_json_printer_add_integer (printer, "width", pango_layout_get_width (layout));
 
-  if (layout->height != -1)
-    gtk_json_printer_add_integer (printer, "height", layout->height);
+  if (pango_layout_get_height (layout) != -1)
+    gtk_json_printer_add_integer (printer, "height", pango_layout_get_height (layout));
 
-  if (layout->indent != 0)
-    gtk_json_printer_add_integer (printer, "indent", layout->indent);
+  if (pango_layout_get_indent (layout) != 0)
+    gtk_json_printer_add_integer (printer, "indent", pango_layout_get_indent (layout));
 
-  if (layout->spacing != 0)
-    gtk_json_printer_add_integer (printer, "spacing", layout->spacing);
+  if (pango_layout_get_line_height (layout) != 0.)
+    gtk_json_printer_add_number (printer, "line-height", pango_layout_get_line_height (layout));
 
-  if (layout->line_spacing != 0.)
-    gtk_json_printer_add_number (printer, "line-spacing", layout->line_spacing);
+  if (pango_layout_get_spacing (layout) != 0)
+    gtk_json_printer_add_integer (printer, "spacing", pango_layout_get_spacing (layout));
 
   if (flags & PANGO_LAYOUT_SERIALIZE_OUTPUT)
-    add_output (printer, layout);
+    {
+      const PangoLogAttr *log_attrs;
+      int n_attrs;
+
+      log_attrs = pango_layout_get_log_attrs (layout, &n_attrs);
+      add_log_attrs (printer, log_attrs, n_attrs);
+
+      lines_to_json (printer, pango_layout_get_lines (layout));
+    }
 
   gtk_json_printer_end (printer);
 }
@@ -907,6 +938,7 @@ attr_for_type (GtkJsonParser *parser,
 
   switch (type)
     {
+    case PANGO_ATTR_SHAPE:
     default:
       g_assert_not_reached ();
 
@@ -957,31 +989,30 @@ attr_for_type (GtkJsonParser *parser,
 
     case PANGO_ATTR_FOREGROUND:
       parser_get_color (parser, &color);
-      attr = pango_attr_foreground_new (color.red, color.green, color.blue);
+      attr = pango_attr_foreground_new (&color);
       break;
 
     case PANGO_ATTR_BACKGROUND:
       parser_get_color (parser, &color);
-      attr = pango_attr_background_new (color.red, color.green, color.blue);
+      attr = pango_attr_background_new (&color);
       break;
 
     case PANGO_ATTR_UNDERLINE:
-      attr = pango_attr_underline_new ((PangoUnderline) parser_select_string (parser, underline_names));
+      attr = pango_attr_underline_new ((PangoLineStyle) parser_select_string (parser, line_style_names));
+      break;
+
+    case PANGO_ATTR_UNDERLINE_POSITION:
+      attr = pango_attr_underline_position_new ((PangoUnderlinePosition) parser_select_string (parser, underline_position_names));
       break;
 
     case PANGO_ATTR_STRIKETHROUGH:
-      attr = pango_attr_strikethrough_new (gtk_json_parser_get_boolean (parser));
+      attr = pango_attr_strikethrough_new ((PangoLineStyle) parser_select_string (parser, line_style_names));
       break;
 
     case PANGO_ATTR_RISE:
       attr = pango_attr_rise_new ((int) gtk_json_parser_get_number (parser));
       break;
 
-    case PANGO_ATTR_SHAPE:
-      /* FIXME */
-      attr = pango_attr_shape_new (&(PangoRectangle) { 0, 0, 0, 0}, &(PangoRectangle) { 0, 0, 0, 0});
-      break;
-
     case PANGO_ATTR_SCALE:
       attr = pango_attr_scale_new (gtk_json_parser_get_number (parser));
       break;
@@ -996,12 +1027,12 @@ attr_for_type (GtkJsonParser *parser,
 
     case PANGO_ATTR_UNDERLINE_COLOR:
       parser_get_color (parser, &color);
-      attr = pango_attr_underline_color_new (color.red, color.green, color.blue);
+      attr = pango_attr_underline_color_new (&color);
       break;
 
     case PANGO_ATTR_STRIKETHROUGH_COLOR:
       parser_get_color (parser, &color);
-      attr = pango_attr_strikethrough_color_new (color.red, color.green, color.blue);
+      attr = pango_attr_strikethrough_color_new (&color);
       break;
 
     case PANGO_ATTR_ABSOLUTE_SIZE:
@@ -1022,14 +1053,6 @@ attr_for_type (GtkJsonParser *parser,
       g_free (str);
       break;
 
-    case PANGO_ATTR_FOREGROUND_ALPHA:
-      attr = pango_attr_foreground_alpha_new ((int) gtk_json_parser_get_number (parser));
-      break;
-
-    case PANGO_ATTR_BACKGROUND_ALPHA:
-      attr = pango_attr_background_alpha_new ((int) gtk_json_parser_get_number (parser));
-      break;
-
     case PANGO_ATTR_ALLOW_BREAKS:
       attr = pango_attr_allow_breaks_new (gtk_json_parser_get_boolean (parser));
       break;
@@ -1043,12 +1066,12 @@ attr_for_type (GtkJsonParser *parser,
       break;
 
     case PANGO_ATTR_OVERLINE:
-      attr = pango_attr_overline_new ((PangoOverline) parser_select_string (parser, overline_names));
+      attr = pango_attr_overline_new ((PangoLineStyle) parser_select_string (parser, line_style_names));
       break;
 
     case PANGO_ATTR_OVERLINE_COLOR:
       parser_get_color (parser, &color);
-      attr = pango_attr_overline_color_new (color.red, color.green, color.blue);
+      attr = pango_attr_overline_color_new (&color);
       break;
 
     case PANGO_ATTR_LINE_HEIGHT:
@@ -1059,6 +1082,10 @@ attr_for_type (GtkJsonParser *parser,
       attr = pango_attr_line_height_new_absolute ((int) gtk_json_parser_get_number (parser));
       break;
 
+    case PANGO_ATTR_LINE_SPACING:
+      attr = pango_attr_line_spacing_new ((int) gtk_json_parser_get_number (parser));
+      break;
+
     case PANGO_ATTR_TEXT_TRANSFORM:
       attr = pango_attr_text_transform_new ((PangoTextTransform) parser_select_string (parser, text_transform_names));
       break;
@@ -1078,6 +1105,11 @@ attr_for_type (GtkJsonParser *parser,
     case PANGO_ATTR_FONT_SCALE:
       attr = pango_attr_font_scale_new ((PangoFontScale) parser_select_string (parser, font_scale_names));
       break;
+
+    case PANGO_ATTR_PARAGRAPH:
+      attr = pango_attr_paragraph_new ();
+      break;
+
     }
 
   attr->start_index = start;
@@ -1124,7 +1156,7 @@ json_to_attribute (GtkJsonParser *parser)
           break;
 
         case ATTR_TYPE:
-          type = parser_select_string (parser, attr_type_names);
+          type = get_attr_type (gtk_json_parser_get_string (parser));
           break;
 
         case ATTR_VALUE:
@@ -1138,7 +1170,12 @@ json_to_attribute (GtkJsonParser *parser)
   while (gtk_json_parser_next (parser));
 
   if (!attr && !gtk_json_parser_get_error (parser))
-    gtk_json_parser_schema_error (parser, "Attribute missing \"value\"");
+    {
+      if (type == PANGO_ATTR_INVALID)
+        gtk_json_parser_schema_error (parser, "Invalid attribute \"type\"");
+      else
+        gtk_json_parser_schema_error (parser, "Attribute missing \"value\"");
+    }
 
   gtk_json_parser_end (parser);
 
@@ -1370,9 +1407,6 @@ enum {
   LAYOUT_ATTRIBUTES,
   LAYOUT_FONT,
   LAYOUT_TABS,
-  LAYOUT_JUSTIFY,
-  LAYOUT_JUSTIFY_LAST_LINE,
-  LAYOUT_SINGLE_PARAGRAPH,
   LAYOUT_AUTO_DIR,
   LAYOUT_ALIGNMENT,
   LAYOUT_WRAP,
@@ -1380,9 +1414,8 @@ enum {
   LAYOUT_WIDTH,
   LAYOUT_HEIGHT,
   LAYOUT_INDENT,
-  LAYOUT_SPACING,
-  LAYOUT_LINE_SPACING,
-  LAYOUT_OUTPUT
+  LAYOUT_LINE_HEIGHT,
+  LAYOUT_LINES
 };
 
 static const char *layout_members[] = {
@@ -1392,9 +1425,6 @@ static const char *layout_members[] = {
   "attributes",
   "font",
   "tabs",
-  "justify",
-  "justify-last-line",
-  "single-paragraph",
   "auto-dir",
   "alignment",
   "wrap",
@@ -1402,9 +1432,8 @@ static const char *layout_members[] = {
   "width",
   "height",
   "indent",
-  "spacing",
-  "line-spacing",
-  "output",
+  "line-height",
+  "lines",
   NULL
 };
 
@@ -1454,6 +1483,14 @@ json_parser_fill_layout (GtkJsonParser               *parser,
           }
           break;
 
+        case LAYOUT_AUTO_DIR:
+          pango_layout_set_auto_dir (layout, gtk_json_parser_get_boolean (parser));
+          break;
+
+        case LAYOUT_LINE_HEIGHT:
+          pango_layout_set_line_height (layout, gtk_json_parser_get_number (parser));
+          break;
+
         case LAYOUT_TABS:
           {
             PangoTabArray *tabs = pango_tab_array_new (0, FALSE);
@@ -1463,22 +1500,6 @@ json_parser_fill_layout (GtkJsonParser               *parser,
           }
           break;
 
-        case LAYOUT_JUSTIFY:
-          pango_layout_set_justify (layout, gtk_json_parser_get_boolean (parser));
-          break;
-
-        case LAYOUT_JUSTIFY_LAST_LINE:
-          pango_layout_set_justify_last_line (layout, gtk_json_parser_get_boolean (parser));
-          break;
-
-        case LAYOUT_SINGLE_PARAGRAPH:
-          pango_layout_set_single_paragraph_mode (layout, gtk_json_parser_get_boolean (parser));
-          break;
-
-        case LAYOUT_AUTO_DIR:
-          pango_layout_set_auto_dir (layout, gtk_json_parser_get_boolean (parser));
-          break;
-
         case LAYOUT_ALIGNMENT:
           pango_layout_set_alignment (layout, (PangoAlignment) parser_select_string (parser, alignment_names));
           break;
@@ -1503,15 +1524,7 @@ json_parser_fill_layout (GtkJsonParser               *parser,
           pango_layout_set_indent (layout, (int) gtk_json_parser_get_number (parser));
           break;
 
-        case LAYOUT_SPACING:
-          pango_layout_set_spacing (layout, (int) gtk_json_parser_get_number (parser));
-          break;
-
-        case LAYOUT_LINE_SPACING:
-          pango_layout_set_line_spacing (layout, gtk_json_parser_get_number (parser));
-          break;
-
-        case LAYOUT_OUTPUT:
+        case LAYOUT_LINES:
           break;
 
         default:
@@ -1586,8 +1599,6 @@ json_parser_load_font (GtkJsonParser  *parser,
  * The format is not meant as a permanent storage format.
  *
  * Returns: a `GBytes` containing the serialized form of @layout
- *
- * Since: 1.50
  */
 GBytes *
 pango_layout_serialize (PangoLayout               *layout,
@@ -1618,9 +1629,6 @@ pango_layout_serialize (PangoLayout               *layout,
 /**
  * pango_layout_write_to_file:
  * @layout: a `PangoLayout`
- * @flags: `PangoLayoutSerializeFlags`
- * @filename: (type filename): the file to save it to
- * @error: Return location for a potential error
  *
  * A convenience method to serialize a layout to a file.
  *
@@ -1633,31 +1641,27 @@ pango_layout_serialize (PangoLayout               *layout,
  * a layout to a file for later inspection.
  *
  * Returns: %TRUE if saving was successful
- *
- * Since: 1.50
  */
 gboolean
-pango_layout_write_to_file (PangoLayout                *layout,
-                            PangoLayoutSerializeFlags   flags,
-                            const char                 *filename,
-                            GError                    **error)
+pango_layout_write_to_file (PangoLayout *layout,
+                            const char  *filename)
 {
   GBytes *bytes;
   gboolean result;
 
   g_return_val_if_fail (PANGO_IS_LAYOUT (layout), FALSE);
   g_return_val_if_fail (filename != NULL, FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-  bytes = pango_layout_serialize (layout, flags);
+  bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_CONTEXT |
+                                          PANGO_LAYOUT_SERIALIZE_OUTPUT);
+
   result = g_file_set_contents (filename,
                                 g_bytes_get_data (bytes, NULL),
                                 g_bytes_get_size (bytes),
-                                error);
+                                NULL);
   g_bytes_unref (bytes);
 
   return result;
-
 }
 
 /**
@@ -1676,8 +1680,6 @@ pango_layout_write_to_file (PangoLayout                *layout,
  * result of serializing the layout again.
  *
  * Returns: (nullable) (transfer full): a new `PangoLayout`
- *
- * Since: 1.50
  */
 PangoLayout *
 pango_layout_deserialize (PangoContext                 *context,
@@ -1713,7 +1715,8 @@ pango_layout_deserialize (PangoContext                 *context,
         code = PANGO_LAYOUT_DESERIALIZE_INVALID;
 
       g_set_error (error, PANGO_LAYOUT_DESERIALIZE_ERROR, code,
-                   "%" G_GSIZE_FORMAT ":%" G_GSIZE_FORMAT ": %s", start, end, parser_error->message);
+                   "%" G_GSIZE_FORMAT ":%" G_GSIZE_FORMAT ": %s",
+                   start, end, parser_error->message);
 
       g_clear_object (&layout);
     }
@@ -1738,8 +1741,6 @@ pango_layout_deserialize (PangoContext                 *context,
  * To recreate a font from its serialized form, use [func@Pango.Font.deserialize].
  *
  * Returns: a `GBytes` containing the serialized form of @font
- *
- * Since: 1.50
  */
 GBytes *
 pango_font_serialize (PangoFont *font)
@@ -1750,7 +1751,7 @@ pango_font_serialize (PangoFont *font)
   gsize size;
 
   g_return_val_if_fail (PANGO_IS_FONT (font), NULL);
-
+ 
   str = g_string_new ("");
 
   printer = gtk_json_printer_new (gstring_write, str, NULL);
@@ -1779,8 +1780,6 @@ pango_font_serialize (PangoFont *font)
  * result of serializing the font again.
  *
  * Returns: (nullable) (transfer full): a new `PangoFont`
- *
- * Since: 1.50
  */
 PangoFont *
 pango_font_deserialize (PangoContext  *context,
