@@ -28,8 +28,11 @@
 #include "pango-impl-utils.h"
 #include "pango-glyph.h"
 
+#include "pango-attr-private.h"
 #include "pango-item-private.h"
 #include "pango-font-private.h"
+#include "pango-userfont-private.h"
+#include "pango-userface-private.h"
 
 #include <hb-ot.h>
 
@@ -123,11 +126,10 @@ pango_hb_font_get_nominal_glyph (hb_font_t      *font,
       if ((context->show_flags & PANGO_SHOW_LINE_BREAKS) != 0 &&
           unicode == 0x2028)
         {
-          /* Always mark LS as unknown. If it ends up
-           * at the line end, PangoLayout takes care of
-           * hiding them, and if they end up in the middle
-           * of a line, we are in single paragraph mode
-           * and want to show the LS
+          /* Always mark LS as unknown. If it ends up at the line end,
+           * the line breaking code takes care of hiding them, and if
+           * they end up in the middle of a line, we are in single
+           * paragraph mode and want to show the LS.
            */
           *glyph = PANGO_GET_UNKNOWN_GLYPH (unicode);
           return TRUE;
@@ -240,17 +242,16 @@ pango_font_get_hb_font_for_context (PangoFont           *font,
 
   hb_font = pango_font_get_hb_font (font);
 
-  if (G_UNLIKELY (g_once_init_enter (&funcs)))
+  if (G_UNLIKELY (!funcs))
     {
-      hb_font_funcs_t *f = hb_font_funcs_create ();
+      funcs = hb_font_funcs_create ();
 
-      hb_font_funcs_set_nominal_glyph_func (f, pango_hb_font_get_nominal_glyph, NULL, NULL);
-      hb_font_funcs_set_glyph_h_advance_func (f, pango_hb_font_get_glyph_h_advance, NULL, NULL);
-      hb_font_funcs_set_glyph_v_advance_func (f, pango_hb_font_get_glyph_v_advance, NULL, NULL);
-      hb_font_funcs_set_glyph_extents_func (f, pango_hb_font_get_glyph_extents, NULL, NULL);
+      hb_font_funcs_set_nominal_glyph_func (funcs, pango_hb_font_get_nominal_glyph, NULL, NULL);
+      hb_font_funcs_set_glyph_h_advance_func (funcs, pango_hb_font_get_glyph_h_advance, NULL, NULL);
+      hb_font_funcs_set_glyph_v_advance_func (funcs, pango_hb_font_get_glyph_v_advance, NULL, NULL);
+      hb_font_funcs_set_glyph_extents_func (funcs, pango_hb_font_get_glyph_extents, NULL, NULL);
 
-      hb_font_funcs_make_immutable (f);
-      g_once_init_leave (&funcs, f);
+      hb_font_funcs_make_immutable (funcs);
     }
 
   context->font = font;
@@ -275,8 +276,8 @@ find_show_flags (const PangoAnalysis *analysis)
     {
       PangoAttribute *attr = l->data;
 
-      if (attr->klass->type == PANGO_ATTR_SHOW)
-        flags |= ((PangoAttrInt*)attr)->value;
+      if (attr->type == PANGO_ATTR_SHOW)
+        flags |= attr->int_value;
     }
 
   return flags;
@@ -292,30 +293,13 @@ find_text_transform (const PangoAnalysis *analysis)
     {
       PangoAttribute *attr = l->data;
 
-      if (attr->klass->type == PANGO_ATTR_TEXT_TRANSFORM)
-        transform = (PangoTextTransform) ((PangoAttrInt*)attr)->value;
+      if (attr->type == PANGO_ATTR_TEXT_TRANSFORM)
+        transform = (PangoTextTransform) attr->int_value;
     }
 
   return transform;
 }
 
-static gboolean
-font_has_color (hb_font_t *font)
-{
-  hb_face_t *face;
-
-  face = hb_font_get_face (font);
-
-#if HB_VERSION_ATLEAST (7, 0, 0)
-  if (hb_ot_color_has_paint (face))
-    return TRUE;
-#endif
-
-  return hb_ot_color_has_layers (face) ||
-         hb_ot_color_has_png (face) ||
-         hb_ot_color_has_svg (face);
-}
-
 static gboolean
 glyph_has_color (hb_font_t      *font,
                  hb_codepoint_t  glyph)
@@ -325,11 +309,6 @@ glyph_has_color (hb_font_t      *font,
 
   face = hb_font_get_face (font);
 
-#if HB_VERSION_ATLEAST (7, 0, 0)
-  if (hb_ot_color_glyph_has_paint (face, glyph))
-    return TRUE;
-#endif
-
   if (hb_ot_color_glyph_get_layers (face, glyph, 0, NULL, NULL) > 0)
     return TRUE;
 
@@ -389,7 +368,6 @@ pango_hb_shape (const char          *item_text,
   PangoGlyphInfo *infos;
   PangoTextTransform transform;
   int hyphen_index;
-  gboolean font_is_color;
 
   g_return_if_fail (analysis != NULL);
   g_return_if_fail (analysis->font != NULL);
@@ -427,7 +405,7 @@ pango_hb_shape (const char          *item_text,
 
       hyphen_index = item_offset + item_length - last_char_len;
 
-      if (log_attrs && log_attrs[num_chars].break_removes_preceding)
+      if (log_attrs[num_chars].break_removes_preceding)
         item_length -= last_char_len;
     }
 
@@ -467,7 +445,7 @@ pango_hb_shape (const char          *item_text,
               break;
 
             case PANGO_TEXT_TRANSFORM_CAPITALIZE:
-              if (log_attrs && log_attrs[i].is_word_start)
+              if (log_attrs[i].is_word_start)
                 ch = g_unichar_totitle (ch);
               break;
 
@@ -520,14 +498,13 @@ pango_hb_shape (const char          *item_text,
   pango_glyph_string_set_size (glyphs, num_glyphs);
   infos = glyphs->glyphs;
   last_cluster = -1;
-  font_is_color = font_has_color (hb_font);
 
   for (i = 0; i < num_glyphs; i++)
     {
       infos[i].glyph = hb_glyph->codepoint;
       glyphs->log_clusters[i] = hb_glyph->cluster - item_offset;
       infos[i].attr.is_cluster_start = glyphs->log_clusters[i] != last_cluster;
-      infos[i].attr.is_color = font_is_color && glyph_has_color (hb_font, hb_glyph->codepoint);
+      infos[i].attr.is_color = glyph_has_color (hb_font, hb_glyph->codepoint);
       hb_glyph++;
       last_cluster = glyphs->log_clusters[i];
     }
@@ -555,6 +532,26 @@ pango_hb_shape (const char          *item_text,
   hb_font_destroy (hb_font);
 }
 
+/* }}} */
+/* {{{ User shaping */
+
+static void
+pango_user_shape (const char          *text,
+                  unsigned int         length,
+                  const PangoAnalysis *analysis,
+                  PangoGlyphString    *glyphs,
+                  PangoShapeFlags      flags)
+{
+  PangoFont *font = analysis->font;
+  PangoUserFace *face = PANGO_USER_FACE (font->face);
+
+  face->shape_func (face, font->size,
+                    text, length,
+                    analysis,
+                    glyphs, flags,
+                    face->user_data);
+}
+
 /* }}} */
 /* {{{ Fallback shaping */
 
@@ -642,48 +639,48 @@ pango_shape_internal (const char          *item_text,
   g_return_if_fail (paragraph_text <= item_text);
   g_return_if_fail (paragraph_text + paragraph_length >= item_text + item_length);
 
-  if (analysis->font)
-    {
-      pango_hb_shape (item_text, item_length,
-                      paragraph_text, paragraph_length,
-                      analysis,
-                      log_attrs, num_chars,
-                      glyphs, flags);
-
-      if (G_UNLIKELY (glyphs->num_glyphs == 0))
-        {
-          /* If a font has been correctly chosen, but no glyphs are output,
-           * there's probably something wrong with the font.
-           *
-           * Trying to be informative, we print out the font description,
-           * and the text, but to not flood the terminal with
-           * zillions of the message, we set a flag to only err once per
-           * font.
-           */
-          GQuark warned_quark = g_quark_from_static_string ("pango-shape-fail-warned");
-
-          if (!g_object_get_qdata (G_OBJECT (analysis->font), warned_quark))
-            {
-              PangoFontDescription *desc;
-              char *font_name;
-
-              desc = pango_font_describe (analysis->font);
-              font_name = pango_font_description_to_string (desc);
-              pango_font_description_free (desc);
-
-              g_warning ("shaping failure, expect ugly output. font='%s', text='%.*s'",
-                         font_name, item_length, item_text);
-
-              g_free (font_name);
-
-              g_object_set_qdata (G_OBJECT (analysis->font), warned_quark,
-                                  GINT_TO_POINTER (1));
-            }
-        }
-    }
+  if (PANGO_IS_USER_FONT (analysis->font))
+    pango_user_shape (item_text, item_length, analysis, glyphs, flags);
+  else if (analysis->font)
+    pango_hb_shape (item_text, item_length,
+                    paragraph_text, paragraph_length,
+                    analysis,
+                    log_attrs, num_chars,
+                    glyphs, flags);
   else
     glyphs->num_glyphs = 0;
 
+  if (analysis->font && glyphs->num_glyphs == 0)
+    {
+      /* If a font has been correctly chosen, but no glyphs are output,
+       * there's probably something wrong with the font.
+       *
+       * Trying to be informative, we print out the font description,
+       * and the text, but to not flood the terminal with
+       * zillions of the message, we set a flag to only err once per
+       * font.
+       */
+      GQuark warned_quark = g_quark_from_static_string ("pango-shape-fail-warned");
+
+      if (!g_object_get_qdata (G_OBJECT (analysis->font), warned_quark))
+        {
+          PangoFontDescription *desc;
+          char *font_name;
+
+          desc = pango_font_describe (analysis->font);
+          font_name = pango_font_description_to_string (desc);
+          pango_font_description_free (desc);
+
+          g_warning ("shaping failure, expect ugly output. font='%s', text='%.*s'",
+                     font_name, item_length, item_text);
+
+          g_free (font_name);
+
+          g_object_set_qdata (G_OBJECT (analysis->font), warned_quark,
+                              GINT_TO_POINTER (1));
+        }
+    }
+
   if (G_UNLIKELY (!glyphs->num_glyphs))
     {
       fallback_shape (item_text, item_length, analysis, glyphs);
@@ -794,89 +791,6 @@ pango_shape_internal (const char          *item_text,
 
 /**
  * pango_shape:
- * @text: the text to process
- * @length: the length (in bytes) of @text
- * @analysis: `PangoAnalysis` structure from [func@Pango.itemize]
- * @glyphs: glyph string in which to store results
- *
- * Convert the characters in @text into glyphs.
- *
- * Given a segment of text and the corresponding `PangoAnalysis` structure
- * returned from [func@Pango.itemize], convert the characters into glyphs. You
- * may also pass in only a substring of the item from [func@Pango.itemize].
- *
- * It is recommended that you use [func@Pango.shape_full] instead, since
- * that API allows for shaping interaction happening across text item
- * boundaries.
- *
- * Some aspects of hyphen insertion and text transformation (in particular,
- * capitalization) require log attrs, and thus can only be handled by
- * [func@Pango.shape_item].
- *
- * Note that the extra attributes in the @analyis that is returned from
- * [func@Pango.itemize] have indices that are relative to the entire paragraph,
- * so you need to subtract the item offset from their indices before
- * calling [func@Pango.shape].
- */
-void
-pango_shape (const char          *text,
-             int                  length,
-             const PangoAnalysis *analysis,
-             PangoGlyphString    *glyphs)
-{
-  pango_shape_full (text, length, text, length, analysis, glyphs);
-}
-
-/**
- * pango_shape_full:
- * @item_text: valid UTF-8 text to shape.
- * @item_length: the length (in bytes) of @item_text. -1 means nul-terminated text.
- * @paragraph_text: (nullable): text of the paragraph (see details).
- * @paragraph_length: the length (in bytes) of @paragraph_text. -1 means nul-terminated text.
- * @analysis: `PangoAnalysis` structure from [func@Pango.itemize].
- * @glyphs: glyph string in which to store results.
- *
- * Convert the characters in @text into glyphs.
- *
- * Given a segment of text and the corresponding `PangoAnalysis` structure
- * returned from [func@Pango.itemize], convert the characters into glyphs.
- * You may also pass in only a substring of the item from [func@Pango.itemize].
- *
- * This is similar to [func@Pango.shape], except it also can optionally take
- * the full paragraph text as input, which will then be used to perform
- * certain cross-item shaping interactions. If you have access to the broader
- * text of which @item_text is part of, provide the broader text as
- * @paragraph_text. If @paragraph_text is %NULL, item text is used instead.
- *
- * Some aspects of hyphen insertion and text transformation (in particular,
- * capitalization) require log attrs, and thus can only be handled by
- * [func@Pango.shape_item].
- *
- * Note that the extra attributes in the @analyis that is returned from
- * [func@Pango.itemize] have indices that are relative to the entire paragraph,
- * so you do not pass the full paragraph text as @paragraph_text, you need
- * to subtract the item offset from their indices before calling
- * [func@Pango.shape_full].
- *
- * Since: 1.32
- */
-void
-pango_shape_full (const char          *item_text,
-                  int                  item_length,
-                  const char          *paragraph_text,
-                  int                  paragraph_length,
-                  const PangoAnalysis *analysis,
-                  PangoGlyphString    *glyphs)
-{
-  pango_shape_with_flags (item_text, item_length,
-                          paragraph_text, paragraph_length,
-                          analysis,
-                          glyphs,
-                          PANGO_SHAPE_NONE);
-}
-
-/**
- * pango_shape_with_flags:
  * @item_text: valid UTF-8 text to shape
  * @item_length: the length (in bytes) of @item_text.
  *     -1 means nul-terminated text.
@@ -893,29 +807,19 @@ pango_shape_full (const char          *item_text,
  * returned from [func@Pango.itemize], convert the characters into glyphs.
  * You may also pass in only a substring of the item from [func@Pango.itemize].
  *
- * This is similar to [func@Pango.shape_full], except it also takes flags
- * that can influence the shaping process.
- *
- * Some aspects of hyphen insertion and text transformation (in particular,
- * capitalization) require log attrs, and thus can only be handled by
- * [func@Pango.shape_item].
- *
  * Note that the extra attributes in the @analyis that is returned from
  * [func@Pango.itemize] have indices that are relative to the entire paragraph,
  * so you do not pass the full paragraph text as @paragraph_text, you need
- * to subtract the item offset from their indices before calling
- * [func@Pango.shape_with_flags].
- *
- * Since: 1.44
+ * to subtract the item offset from their indices first.
  */
 void
-pango_shape_with_flags (const char          *item_text,
-                        int                  item_length,
-                        const char          *paragraph_text,
-                        int                  paragraph_length,
-                        const PangoAnalysis *analysis,
-                        PangoGlyphString    *glyphs,
-                        PangoShapeFlags      flags)
+pango_shape (const char          *item_text,
+             int                  item_length,
+             const char          *paragraph_text,
+             int                  paragraph_length,
+             const PangoAnalysis *analysis,
+             PangoGlyphString    *glyphs,
+             PangoShapeFlags      flags)
 {
   pango_shape_internal (item_text, item_length,
                         paragraph_text, paragraph_length,
@@ -935,19 +839,14 @@ pango_shape_with_flags (const char          *item_text,
  *
  * Convert the characters in @item into glyphs.
  *
- * This is similar to [func@Pango.shape_with_flags], except it takes a
- * `PangoItem` instead of separate @item_text and @analysis arguments.
- *
- * It also takes @log_attrs, which are needed for implementing some aspects
- * of hyphen insertion and text transforms (in particular, capitalization).
+ * This is similar to [func@Pango.shape], except it takes a `PangoItem`
+ * instead of separate @item_text and @analysis arguments. It also takes
+ * @log_attrs, which may be used in implementing text transforms.
  *
  * Note that the extra attributes in the @analyis that is returned from
  * [func@Pango.itemize] have indices that are relative to the entire paragraph,
- * so you do not pass the full paragraph text as @paragraph_text, you need
- * to subtract the item offset from their indices before calling
- * [func@Pango.shape_with_flags].
- *
- * Since: 1.50
+ * so you do not pass the full paragraph text as @paragraph_text, you need to
+ * subtract the item offset from their indices before calling [func@Pango.shape].
  */
 void
 pango_shape_item (PangoItem        *item,
