@@ -38,15 +38,13 @@ static PangoContext *context;
 static gboolean opt_hex_chars;
 
 static gboolean
-test_file (const gchar *filename, GString *string)
+test_file (const char *filename, GString *string)
 {
-  gchar *contents;
+  char *contents;
   gsize  length;
   GError *error = NULL;
-  PangoLogAttr *attrs;
-  const PangoLogAttr *attrs2;
+  const PangoLogAttr *attrs;
   int len;
-  int len2;
   char *p;
   int i;
   GString *s1, *s2, *s3, *s4, *s5, *s6;
@@ -55,7 +53,6 @@ test_file (const gchar *filename, GString *string)
   char *text;
   PangoAttrList *attributes;
   PangoLayout *layout;
-  PangoLayout *layout2;
 
   g_file_get_contents (filename, &contents, &length, &error);
   g_assert_no_error (error);
@@ -93,25 +90,14 @@ test_file (const gchar *filename, GString *string)
     }
 #endif
 
-  pango_layout_get_log_attrs (layout, &attrs, &len);
-  attrs2 = pango_layout_get_log_attrs_readonly (layout, &len2);
+  attrs = pango_layout_get_log_attrs (layout, &len);
 
-  g_assert_cmpint (len, ==, len2);
-  g_assert_true (memcmp (attrs, attrs2, sizeof (PangoLogAttr) * len) == 0);
   if (!pango_validate_log_attrs (text, length, attrs, len, &error))
     {
       g_warning ("%s: Log attrs invalid: %s", filename, error->message);
 //      g_assert_not_reached ();
     }
 
-  layout2 = pango_layout_copy (layout);
-  attrs2 = pango_layout_get_log_attrs_readonly (layout2, &len2);
-
-  g_assert_cmpint (len, ==, len2);
-  g_assert_true (memcmp (attrs, attrs2, sizeof (PangoLogAttr) * len) == 0);
-
-  g_object_unref (layout2);
-
   s1 = g_string_new ("Breaks: ");
   s2 = g_string_new ("Whitespace: ");
   s3 = g_string_new ("Sentences:");
@@ -291,7 +277,6 @@ test_file (const gchar *filename, GString *string)
   g_string_free (s6, TRUE);
 
   g_object_unref (layout);
-  g_free (attrs);
   g_free (contents);
   g_free (text);
   pango_attr_list_unref (attributes);
@@ -299,10 +284,10 @@ test_file (const gchar *filename, GString *string)
   return TRUE;
 }
 
-static gchar *
-get_expected_filename (const gchar *filename)
+static char *
+get_expected_filename (const char *filename)
 {
-  gchar *f, *p, *expected;
+  char *f, *p, *expected;
 
   f = g_strdup (filename);
   p = strstr (f, ".break");
@@ -318,11 +303,11 @@ get_expected_filename (const gchar *filename)
 static void
 test_break (gconstpointer d)
 {
-  const gchar *filename = d;
-  gchar *expected_file;
+  const char *filename = d;
+  char *expected_file;
   GError *error = NULL;
   GString *dump;
-  gchar *diff;
+  char *diff;
 
   char *old_locale = g_strdup (setlocale (LC_ALL, NULL));
   setlocale (LC_ALL, "en_US.UTF-8");
@@ -378,8 +363,8 @@ main (int argc, char *argv[])
 {
   GDir *dir;
   GError *error = NULL;
-  const gchar *name;
-  gchar *path;
+  const char *name;
+  char *path;
   gboolean opt_legend = 0;
   GOptionContext *option_context;
   GOptionEntry entries[] = {
@@ -388,6 +373,8 @@ main (int argc, char *argv[])
     { NULL, 0 },
   };
 
+  setlocale (LC_ALL, "");
+
   option_context = g_option_context_new ("");
   g_option_context_add_main_entries (option_context, entries, NULL);
   g_option_context_set_ignore_unknown_options (option_context, TRUE);
@@ -398,9 +385,8 @@ main (int argc, char *argv[])
     }
   g_option_context_free (option_context);
 
-  setlocale (LC_ALL, "");
-
-  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+  install_fonts ();
+  context = pango_context_new ();
 
   if (opt_legend)
     {
