@@ -25,6 +25,7 @@ void print_attr_list (PangoAttrList  *attrs,
 
 const char *get_script_name (GUnicodeScript s);
 
-PangoFontMap *get_font_map_with_cantarell (void);
+void    install_fonts      (void);
+void    dump_fonts         (void);
 
 #endif
