@@ -28,9 +28,10 @@
 #endif
 
 #include "config.h"
-#include <pango/pangocairo.h>
+#include <pango/pango.h>
 #include "test-common.h"
 
+#include "pango/pango-attr-private.h"
 #include "pango/pango-item-private.h"
 
 
@@ -57,7 +58,7 @@ static gboolean
 affects_itemization (PangoAttribute *attr,
                      gpointer        data)
 {
-  switch ((int)attr->klass->type)
+  switch ((int)attr->type)
     {
     /* These affect font selection */
     case PANGO_ATTR_LANGUAGE:
@@ -76,7 +77,6 @@ affects_itemization (PangoAttribute *attr,
     case PANGO_ATTR_FONT_SCALE:
     /* These are part of ItemProperties, so need to break runs */
     case PANGO_ATTR_LETTER_SPACING:
-    case PANGO_ATTR_SHAPE:
     case PANGO_ATTR_RISE:
     case PANGO_ATTR_BASELINE_SHIFT:
     case PANGO_ATTR_LINE_HEIGHT:
@@ -88,6 +88,56 @@ affects_itemization (PangoAttribute *attr,
     }
 }
 
+/* Keep in sync with pango-item.c */
+static int
+compare_attr (gconstpointer p1, gconstpointer p2)
+{
+  const PangoAttribute *a1 = p1;
+  const PangoAttribute *a2 = p2;
+  if (pango_attribute_equal (a1, a2) &&
+      a1->start_index == a2->start_index &&
+      a1->end_index == a2->end_index)
+    return 0;
+
+  return 1;
+}
+
+void
+pango_item_apply_attrs (PangoItem         *item,
+                        PangoAttrIterator *iter)
+{
+  int start, end;
+  GSList *attrs = NULL;
+
+  do
+    {
+      pango_attr_iterator_range (iter, &start, &end);
+
+      if (start >= item->offset + item->length)
+        break;
+
+      if (end >= item->offset)
+        {
+          GSList *list, *l;
+
+          list = pango_attr_iterator_get_attrs (iter);
+          for (l = list; l; l = l->next)
+            {
+              if (!g_slist_find_custom (attrs, l->data, compare_attr))
+
+                attrs = g_slist_prepend (attrs, pango_attribute_copy (l->data));
+            }
+          g_slist_free_full (list, (GDestroyNotify)pango_attribute_destroy);
+        }
+
+      if (end >= item->offset + item->length)
+        break;
+    }
+  while (pango_attr_iterator_next (iter));
+
+  item->analysis.extra_attrs = g_slist_concat (item->analysis.extra_attrs, attrs);
+}
+
 static void
 apply_attributes_to_items (GList         *items,
                            PangoAttrList *attrs)
@@ -112,16 +162,13 @@ apply_attributes_to_items (GList         *items,
 static int
 get_item_char_offset (PangoItem *item)
 {
-  if (item->analysis.flags & PANGO_ANALYSIS_FLAG_HAS_CHAR_OFFSET)
-    return ((PangoItemPrivate *)item)->char_offset;
-
-  return -1;
+  return item->char_offset;
 }
 
 static void
-test_file (const gchar *filename, GString *string)
+test_file (const char *filename, GString *string)
 {
-  gchar *contents;
+  char *contents;
   gsize  length;
   GError *error = NULL;
   GString *s1, *s2, *s3, *s4, *s5, *s6, *s7;
@@ -131,6 +178,7 @@ test_file (const gchar *filename, GString *string)
   PangoAttrList *itemize_attrs;
   GList *items, *l;
   const char *sep = "";
+  PangoDirection dir;
 
   g_file_get_contents (filename, &contents, &length, &error);
   g_assert_no_error (error);
@@ -157,7 +205,8 @@ test_file (const gchar *filename, GString *string)
     length--;
 
   itemize_attrs = pango_attr_list_filter (attrs, affects_itemization, NULL);
-  items = pango_itemize (context, text, 0, length, itemize_attrs, NULL);
+  dir = pango_context_get_base_dir (context);
+  items = pango_itemize (context, dir, text, 0, length, itemize_attrs);
 
   apply_attributes_to_items (items, attrs);
   pango_attr_list_unref (itemize_attrs);
@@ -171,6 +220,8 @@ test_file (const gchar *filename, GString *string)
       GSList *a;
 
       desc = pango_font_describe (item->analysis.font);
+      /* Leave out faceid for now to avoid backend-dependent test output */
+      pango_font_description_unset_fields (desc, PANGO_FONT_MASK_FACEID);
       font = pango_font_description_to_string (desc);
 
       if (l != items)
@@ -231,10 +282,10 @@ test_file (const gchar *filename, GString *string)
   g_free (contents);
 }
 
-static gchar *
-get_expected_filename (const gchar *filename)
+static char *
+get_expected_filename (const char *filename)
 {
-  gchar *f, *p, *expected;
+  char *f, *p, *expected;
 
   f = g_strdup (filename);
   p = strstr (f, ".items");
@@ -250,14 +301,13 @@ get_expected_filename (const gchar *filename)
 static void
 test_itemize (gconstpointer d)
 {
-  const gchar *filename = d;
-  gchar *expected_file;
+  const char *filename = d;
+  char *expected_file;
   GError *error = NULL;
   GString *dump;
-  gchar *diff;
-  PangoFontFamily **families;
-  int n_families;
+  char *diff;
   gboolean found_cantarell;
+  PangoFontMap *map;
 
   char *old_locale = g_strdup (setlocale (LC_ALL, NULL));
   setlocale (LC_ALL, "en_US.UTF-8");
@@ -271,16 +321,17 @@ test_itemize (gconstpointer d)
     }
 
   found_cantarell = FALSE;
-  pango_context_list_families (context, &families, &n_families);
-  for (int i = 0; i < n_families; i++)
+  map = pango_context_get_font_map (context);
+  for (int i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (map)); i++)
     {
-      if (strcmp (pango_font_family_get_name (families[i]), "Cantarell") == 0)
+      PangoFontFamily *family = g_list_model_get_item (G_LIST_MODEL (map), i);
+      g_object_unref (family);
+      if (strcmp (pango_font_family_get_name (family), "Cantarell") == 0)
         {
           found_cantarell = TRUE;
           break;
         }
     }
-  g_free (families);
 
   if (!found_cantarell)
     {
@@ -329,10 +380,12 @@ main (int argc, char *argv[])
 {
   GDir *dir;
   GError *error = NULL;
-  const gchar *name;
-  gchar *path;
+  const char *name;
+  char *path;
 
-  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+  install_fonts ();
+
+  context = pango_context_new ();
   pango_context_set_language (context, pango_language_from_string ("en-us"));
 
   /* allow to easily generate expected output for new test cases */
