@@ -27,12 +27,10 @@
 #include <unistd.h>
 #endif
 
-#include "config.h"
 #include <pango/pangocairo.h>
-#include <pango/pangocairo-fc.h>
-#include <pango/pangofc-fontmap.h>
 #include "test-common.h"
 
+#include <hb-ot.h>
 
 static void
 test_layout (gconstpointer d)
@@ -47,12 +45,6 @@ test_layout (gconstpointer d)
   PangoContext *context;
   PangoLayout *layout;
 
-  if (!PANGO_IS_FC_FONT_MAP (pango_cairo_font_map_get_default ()))
-    {
-      g_test_skip ("Not an fc fontmap. Skipping...");
-      return;
-    }
-
   char *old_locale = g_strdup (setlocale (LC_ALL, NULL));
   setlocale (LC_ALL, "en_US.UTF-8");
   if (strstr (setlocale (LC_ALL, NULL), "en_US") == NULL)
@@ -68,11 +60,12 @@ test_layout (gconstpointer d)
   g_assert_no_error (error);
   orig = g_bytes_new_take (contents, length);
 
-  context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+  context = pango_context_new ();
   layout = pango_layout_deserialize (context, orig, PANGO_LAYOUT_DESERIALIZE_CONTEXT, &error);
   g_assert_no_error (error);
 
-  bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_CONTEXT | PANGO_LAYOUT_SERIALIZE_OUTPUT);
+  bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_CONTEXT |
+                                                 PANGO_LAYOUT_SERIALIZE_OUTPUT);
 
   g_object_unref (layout);
   g_object_unref (context);
@@ -104,67 +97,17 @@ test_layout (gconstpointer d)
   g_free (diff);
 }
 
-static void
-install_fonts (const char *dir)
-{
-  FcConfig *config;
-  PangoFontMap *map;
-  char *path;
-  gsize len;
-  char *conf;
-
-  map = g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
-
-  config = FcConfigCreate ();
-
-  path = g_build_filename (dir, "fonts.conf", NULL);
-  g_file_get_contents (path, &conf, &len, NULL);
-
-  if (!FcConfigParseAndLoadFromMemory (config, (const FcChar8 *) conf, TRUE))
-    g_error ("Failed to parse fontconfig configuration");
-
-  g_free (conf);
-  g_free (path);
-
-  FcConfigAppFontAddDir (config, (const FcChar8 *) dir);
-  pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (map), config);
-  FcConfigDestroy (config);
-
-  pango_cairo_font_map_set_default (PANGO_CAIRO_FONT_MAP (map));
-
-  g_object_unref (map);
-}
-
 int
 main (int argc, char *argv[])
 {
   GDir *dir;
   GError *error = NULL;
-  char *opt_fonts = NULL;
-  const gchar *name;
+  const char *name;
   char *path;
-  GOptionContext *option_context;
-  GOptionEntry entries[] = {
-    { "fonts", 0, 0, G_OPTION_ARG_FILENAME, &opt_fonts, "Fonts to use", "DIR" },
-    { NULL, 0 },
-  };
 
   setlocale (LC_ALL, "");
-  option_context = g_option_context_new ("");
-  g_option_context_add_main_entries (option_context, entries, NULL);
-  g_option_context_set_ignore_unknown_options (option_context, TRUE);
-  if (!g_option_context_parse (option_context, &argc, &argv, &error))
-    {
-      g_error ("failed to parse options: %s", error->message);
-      return 1;
-    }
-  g_option_context_free (option_context);
 
-  if (opt_fonts)
-    {
-      install_fonts (opt_fonts);
-      g_free (opt_fonts);
-    }
+  install_fonts ();
 
   /* allow to easily generate expected output for new test cases */
   if (argc > 1 && argv[1][0] != '-')
@@ -180,11 +123,12 @@ main (int argc, char *argv[])
       g_file_get_contents (argv[1], &contents, &length, &error);
       g_assert_no_error (error);
       orig = g_bytes_new_take (contents, length);
-      context = pango_font_map_create_context (pango_cairo_font_map_get_default ());
+      context = pango_context_new ();
       layout = pango_layout_deserialize (context, orig, PANGO_LAYOUT_DESERIALIZE_CONTEXT, &error);
       g_assert_no_error (error);
 
-      bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_CONTEXT | PANGO_LAYOUT_SERIALIZE_OUTPUT);
+      bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_CONTEXT |
+                                              PANGO_LAYOUT_SERIALIZE_OUTPUT);
 
       g_object_unref (layout);
       g_object_unref (context);
@@ -199,13 +143,6 @@ main (int argc, char *argv[])
 
   g_test_init (&argc, &argv, NULL);
 
-  if (!opt_fonts)
-    {
-      path = g_test_build_filename (G_TEST_DIST, "fonts", NULL);
-      install_fonts (path);
-      g_free (path);
-    }
-
   path = g_test_build_filename (G_TEST_DIST, "layouts", NULL);
   dir = g_dir_open (path, 0, &error);
   g_free (path);
