@@ -3,6 +3,8 @@
 #include <pango/pango.h>
 #include <pango/pangocairo.h>
 
+#include "test-common.h"
+
 #define WIDTH 100
 #define HEIGHT 100
 const char *text = "Hamburgerfonts\nวิวิวิวิวิวิ\nبهداد";
@@ -21,9 +23,15 @@ create_surface (void)
 static PangoLayout *
 create_layout (cairo_t *cr)
 {
-  PangoLayout *layout = pango_cairo_create_layout (cr);
+  PangoContext *context;
+  PangoLayout *layout;
+
+  context = pango_cairo_create_context (cr);
+  layout = pango_layout_new (context);
+  g_object_unref (context);
   pango_layout_set_text (layout, text, -1);
   pango_layout_set_width (layout, WIDTH * PANGO_SCALE);
+
   return layout;
 }
 
@@ -37,9 +45,9 @@ draw (cairo_t *cr, PangoLayout *layout, unsigned int i)
 
   cairo_identity_matrix (cr);
   cairo_scale (cr, (100 + i) / 100.,  (100 + i) / 100.);
-  pango_cairo_update_layout (cr, layout);
+  pango_cairo_update_context (cr, pango_layout_get_context (layout));
 
-  pango_cairo_show_layout (cr, layout);
+  pango_cairo_show_lines (cr, pango_layout_get_lines (layout));
 }
 
 static gpointer
@@ -49,6 +57,8 @@ thread_func (gpointer data)
   PangoLayout *layout;
   int i;
 
+  install_fonts ();
+
   cairo_t *cr = cairo_create (surface);
 
   layout = create_layout (cr);
@@ -110,7 +120,7 @@ pangocairo_threads (void)
 
     /* cairo_surface_write_to_png (ref_surface, "test-pangocairo-threads-reference.png"); */
 
-    g_assert_cmpint (WIDTH, ==, cairo_format_stride_for_width (CAIRO_FORMAT_A8, WIDTH));
+    g_assert (WIDTH == cairo_format_stride_for_width (CAIRO_FORMAT_A8, WIDTH));
 
     for (i = 0; i < num_threads; i++)
       {
@@ -132,7 +142,7 @@ pangocairo_threads (void)
 
   g_ptr_array_unref (surfaces);
 
-  pango_cairo_font_map_set_default (NULL);
+  pango_font_map_set_default (NULL);
 
 }
 
@@ -141,6 +151,8 @@ main (int argc, char **argv)
 {
   g_test_init (&argc, &argv, NULL);
 
+  install_fonts ();
+
   if (argc > 1)
     num_threads = atoi (argv[1]);
   if (argc > 2)
