@@ -131,9 +131,9 @@ apply_attributes_to_items (GList         *items,
 }
 
 static void
-test_file (const gchar *filename, GString *string)
+test_file (const char *filename, GString *string)
 {
-  gchar *contents;
+  char *contents;
   gsize  length;
   GError *error = NULL;
   char *test;
@@ -145,6 +145,7 @@ test_file (const gchar *filename, GString *string)
   GString *s1, *s2, *s3, *s4, *s5, *s6, *s7, *s8, *s9;
   char *p1;
   const char *sep = "";
+  PangoDirection dir;
 
   if (!g_file_get_contents (filename, &contents, &length, &error))
     {
@@ -183,7 +184,8 @@ test_file (const gchar *filename, GString *string)
   itemize_attrs = pango_attr_list_filter (attrs, affects_itemization, NULL);
   shape_attrs = pango_attr_list_filter (attrs, affects_break_or_shape, NULL);
 
-  items = pango_itemize (context, text, 0, length, itemize_attrs, NULL);
+  dir = pango_context_get_base_dir (context);
+  items = pango_itemize (context, dir, text, 0, length, itemize_attrs);
   apply_attributes_to_items (items, shape_attrs);
 
   pango_attr_list_unref (itemize_attrs);
@@ -337,10 +339,10 @@ test_file (const gchar *filename, GString *string)
   pango_attr_list_unref (attrs);
 }
 
-static gchar *
-get_expected_filename (const gchar *filename)
+static char *
+get_expected_filename (const char *filename)
 {
-  gchar *f, *p, *expected;
+  char *f, *p, *expected;
 
   f = g_strdup (filename);
   p = strstr (f, ".shape");
@@ -356,11 +358,11 @@ get_expected_filename (const gchar *filename)
 static void
 test_shape (gconstpointer d)
 {
-  const gchar *filename = d;
-  gchar *expected_file;
+  const char *filename = d;
+  char *expected_file;
   GError *error = NULL;
   GString *dump;
-  gchar *diff;
+  char *diff;
 
   expected_file = get_expected_filename (filename);
 
@@ -388,8 +390,8 @@ main (int argc, char *argv[])
 {
   GDir *dir;
   GError *error = NULL;
-  const gchar *name;
-  gchar *path;
+  const char *name;
+  char *path;
   GOptionContext *option_context;
   GOptionEntry entries[] = {
     { "hex-chars", '0', 0, G_OPTION_ARG_NONE, &opt_hex_chars, "Print all chars in hex", NULL },
