@@ -20,6 +20,7 @@
  */
 
 #include <pango/pango.h>
+#include <pango/pango-attr-private.h>
 
 static void
 test_copy (PangoAttribute *attr)
@@ -36,12 +37,9 @@ static void
 test_attributes_basic (void)
 {
   PangoFontDescription *desc;
-  PangoRectangle rect = { 0, 0, 10, 10 };
 
   test_copy (pango_attr_language_new (pango_language_from_string ("ja-JP")));
   test_copy (pango_attr_family_new ("Times"));
-  test_copy (pango_attr_foreground_new (100, 200, 300));
-  test_copy (pango_attr_background_new (100, 200, 300));
   test_copy (pango_attr_size_new (1024));
   test_copy (pango_attr_size_new_absolute (1024));
   test_copy (pango_attr_style_new (PANGO_STYLE_ITALIC));
@@ -51,23 +49,20 @@ test_attributes_basic (void)
   desc = pango_font_description_from_string ("Computer Modern 12");
   test_copy (pango_attr_font_desc_new (desc));
   pango_font_description_free (desc);
-  test_copy (pango_attr_underline_new (PANGO_UNDERLINE_LOW));
-  test_copy (pango_attr_underline_new (PANGO_UNDERLINE_ERROR_LINE));
-  test_copy (pango_attr_underline_color_new (100, 200, 300));
-  test_copy (pango_attr_overline_new (PANGO_OVERLINE_SINGLE));
-  test_copy (pango_attr_overline_color_new (100, 200, 300));
+  test_copy (pango_attr_underline_new (PANGO_LINE_STYLE_SOLID));
+  test_copy (pango_attr_underline_new (PANGO_LINE_STYLE_DOTTED));
+  test_copy (pango_attr_underline_color_new (&(PangoColor){100, 200, 300}));
+  test_copy (pango_attr_overline_new (PANGO_LINE_STYLE_SOLID));
+  test_copy (pango_attr_overline_color_new (&(PangoColor){100, 200, 300}));
   test_copy (pango_attr_strikethrough_new (TRUE));
-  test_copy (pango_attr_strikethrough_color_new (100, 200, 300));
+  test_copy (pango_attr_strikethrough_color_new (&(PangoColor){100, 200, 300}));
   test_copy (pango_attr_rise_new (256));
   test_copy (pango_attr_scale_new (2.56));
   test_copy (pango_attr_fallback_new (FALSE));
   test_copy (pango_attr_letter_spacing_new (1024));
-  test_copy (pango_attr_shape_new (&rect, &rect));
   test_copy (pango_attr_gravity_new (PANGO_GRAVITY_SOUTH));
   test_copy (pango_attr_gravity_hint_new (PANGO_GRAVITY_HINT_STRONG));
   test_copy (pango_attr_font_features_new ("csc=1"));
-  test_copy (pango_attr_foreground_alpha_new (8000));
-  test_copy (pango_attr_background_alpha_new (8000));
   test_copy (pango_attr_allow_breaks_new (FALSE));
   test_copy (pango_attr_show_new (PANGO_SHOW_SPACES));
   test_copy (pango_attr_insert_hyphens_new (FALSE));
@@ -98,120 +93,72 @@ test_attributes_equal (void)
   pango_attribute_destroy (attr3);
 }
 
+static gpointer
+copy_my_attribute_data (gconstpointer data)
+{
+  return (gpointer)data;
+}
+
+static void
+destroy_my_attribute_data (gpointer data)
+{
+}
+
+static gboolean
+my_attribute_data_equal (gconstpointer data1,
+                         gconstpointer data2)
+{
+  return data1 == data2;
+}
+
+static char *
+my_attribute_data_serialize (gconstpointer data)
+{
+  return g_strdup_printf ("%#x", GPOINTER_TO_UINT (data));
+}
+
 static void
 test_attributes_register (void)
 {
   PangoAttrType type;
-  GEnumClass *class;
+  PangoAttribute *attr;
+  PangoAttribute *attr2;
+  PangoAttrList *list;
+  gboolean ret;
+  char *str;
+
+  type = pango_attr_type_register ("my-attribute",
+                                   PANGO_ATTR_VALUE_POINTER,
+                                   PANGO_ATTR_AFFECTS_RENDERING,
+                                   PANGO_ATTR_MERGE_OVERRIDES,
+                                   copy_my_attribute_data,
+                                   destroy_my_attribute_data,
+                                   my_attribute_data_equal,
+                                   my_attribute_data_serialize);
 
-  type = pango_attr_type_register ("my-attribute");
   g_assert_cmpstr (pango_attr_type_get_name (type), ==, "my-attribute");
 
-  class = g_type_class_ref (PANGO_TYPE_ATTR_TYPE);
-  for (int i = 0; i < class->n_values; i++)
-    {
-      g_assert_cmpint (type, !=, class->values[i].value);
-      g_assert_null (pango_attr_type_get_name (class->values[i].value));
-    }
+  attr = pango_attribute_new (type, (gpointer)0x42);
 
-  g_type_class_unref (class);
-}
+  g_assert_true (pango_attribute_get_pointer (attr) == (gpointer)0x42);
 
-static void
-test_binding (PangoAttribute *attr)
-{
-  enum {
-    INVALID, INT, LANGUAGE, STRING, SIZE, FONT_DESC, COLOR, SHAPE, FLOAT, FONT_FEATURES,
-  } attr_base[] = {
-    INVALID, LANGUAGE, STRING, INT, INT, INT, INT, SIZE, FONT_DESC, COLOR,
-    COLOR, INT, INT, INT, SHAPE, FLOAT, INT, INT, COLOR, COLOR, SIZE,
-    INT, INT, FONT_FEATURES, INT, INT, INT, INT, INT, INT, COLOR, FLOAT,
-    INT, INT, INT, INT, INT, INT
-  };
+  attr2 = pango_attribute_new (type, (gpointer)0x43);
 
-  switch (attr_base[attr->klass->type])
-    {
-    case INT:
-      g_assert_nonnull (pango_attribute_as_int (attr));
-      break;
-    case LANGUAGE:
-      g_assert_nonnull (pango_attribute_as_language (attr));
-      break;
-    case STRING:
-      g_assert_nonnull (pango_attribute_as_string (attr));
-      break;
-    case SIZE:
-      g_assert_nonnull (pango_attribute_as_size (attr));
-      break;
-    case FONT_DESC:
-      g_assert_nonnull (pango_attribute_as_font_desc (attr));
-      break;
-    case COLOR:
-      g_assert_nonnull (pango_attribute_as_color (attr));
-      break;
-    case SHAPE:
-      g_assert_nonnull (pango_attribute_as_shape (attr));
-      break;
-    case FLOAT:
-      g_assert_nonnull (pango_attribute_as_float (attr));
-      break;
-    case FONT_FEATURES:
-      g_assert_nonnull (pango_attribute_as_font_features (attr));
-      break;
-    case INVALID:
-    default:
-      g_assert_not_reached ();
-    }
+  ret = pango_attribute_equal (attr, attr2);
+  g_assert_false (ret);
+
+  list = pango_attr_list_new ();
+  pango_attr_list_insert (list, attr2);
+
+  str = pango_attr_list_to_string (list);
+  g_assert_cmpstr (str, ==, "0 4294967295 my-attribute 0x43");
+  g_free (str);
+
+  pango_attr_list_unref (list);
 
   pango_attribute_destroy (attr);
 }
 
-static void
-test_binding_helpers (void)
-{
-  PangoFontDescription *desc;
-  PangoRectangle rect = { 0, 0, 10, 10 };
-
-  test_binding (pango_attr_language_new (pango_language_from_string ("ja-JP")));
-  test_binding (pango_attr_family_new ("Times"));
-  test_binding (pango_attr_foreground_new (100, 200, 300));
-  test_binding (pango_attr_background_new (100, 200, 300));
-  test_binding (pango_attr_size_new (1024));
-  test_binding (pango_attr_size_new_absolute (1024));
-  test_binding (pango_attr_style_new (PANGO_STYLE_ITALIC));
-  test_binding (pango_attr_weight_new (PANGO_WEIGHT_ULTRALIGHT));
-  test_binding (pango_attr_variant_new (PANGO_VARIANT_SMALL_CAPS));
-  test_binding (pango_attr_stretch_new (PANGO_STRETCH_SEMI_EXPANDED));
-  desc = pango_font_description_from_string ("Computer Modern 12");
-  test_binding (pango_attr_font_desc_new (desc));
-  pango_font_description_free (desc);
-  test_binding (pango_attr_underline_new (PANGO_UNDERLINE_LOW));
-  test_binding (pango_attr_underline_new (PANGO_UNDERLINE_ERROR_LINE));
-  test_binding (pango_attr_underline_color_new (100, 200, 300));
-  test_binding (pango_attr_overline_new (PANGO_OVERLINE_SINGLE));
-  test_binding (pango_attr_overline_color_new (100, 200, 300));
-  test_binding (pango_attr_strikethrough_new (TRUE));
-  test_binding (pango_attr_strikethrough_color_new (100, 200, 300));
-  test_binding (pango_attr_rise_new (256));
-  test_binding (pango_attr_scale_new (2.56));
-  test_binding (pango_attr_fallback_new (FALSE));
-  test_binding (pango_attr_letter_spacing_new (1024));
-  test_binding (pango_attr_shape_new (&rect, &rect));
-  test_binding (pango_attr_gravity_new (PANGO_GRAVITY_SOUTH));
-  test_binding (pango_attr_gravity_hint_new (PANGO_GRAVITY_HINT_STRONG));
-  test_binding (pango_attr_font_features_new ("csc=1"));
-  test_binding (pango_attr_foreground_alpha_new (8000));
-  test_binding (pango_attr_background_alpha_new (8000));
-  test_binding (pango_attr_allow_breaks_new (FALSE));
-  test_binding (pango_attr_show_new (PANGO_SHOW_SPACES));
-  test_binding (pango_attr_insert_hyphens_new (FALSE));
-  test_binding (pango_attr_text_transform_new (PANGO_TEXT_TRANSFORM_UPPERCASE));
-  test_binding (pango_attr_line_height_new (1.5));
-  test_binding (pango_attr_line_height_new_absolute (3000));
-  test_binding (pango_attr_word_new ());
-  test_binding (pango_attr_sentence_new ());
-}
-
 static void
 assert_attributes (GSList     *attrs,
                    const char *expected)
@@ -237,12 +184,32 @@ assert_attributes (GSList     *attrs,
   pango_attr_list_unref (list2);
 }
 
+static void
+assert_attr_list_order (PangoAttrList *list)
+{
+  GSList *attrs, *l;
+  guint start = 0;
+
+  attrs = pango_attr_list_get_attributes (list);
+
+  for (l = attrs; l; l = l->next)
+    {
+      PangoAttribute *attr = l->data;
+      g_assert (start <= attr->start_index);
+      start = attr->start_index;
+    }
+
+  g_slist_free_full (attrs, (GDestroyNotify) pango_attribute_destroy);
+}
+
 static void
 assert_attr_list (PangoAttrList *list,
                   const char    *expected)
 {
   PangoAttrList *list2;
 
+  assert_attr_list_order (list);
+
   list2 = pango_attr_list_from_string (expected);
   if (!pango_attr_list_equal (list, list2))
     {
@@ -456,7 +423,7 @@ test_list_change5 (void)
   attr = attribute_from_string ("5 15 style italic");
   g_assert (attr->start_index == 5);
   g_assert (attr->end_index == 15);
-  g_assert (((PangoAttrInt *)attr)->value == PANGO_STYLE_ITALIC);
+  g_assert (attr->int_value == PANGO_STYLE_ITALIC);
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "0 3 weight ultrabold\n"
@@ -667,7 +634,7 @@ test_list_splice (void)
   list = pango_attr_list_copy (base);
 
   other = pango_attr_list_from_string ("0 3 size 20\n"
-                                       "2 4 stretch 2\n");
+                                       "2 4 stretch condensed\n");
 
   pango_attr_list_splice (list, other, 11, 5);
 
@@ -745,7 +712,7 @@ never_true (PangoAttribute *attribute, gpointer user_data)
 static gboolean
 just_weight (PangoAttribute *attribute, gpointer user_data)
 {
-  if (attribute->klass->type == PANGO_ATTR_WEIGHT)
+  if (attribute->type == PANGO_ATTR_WEIGHT)
     return TRUE;
   else
     return FALSE;
@@ -758,7 +725,7 @@ test_list_filter (void)
   PangoAttrList *out;
 
   list = pango_attr_list_from_string ("0 -1 size 10\n"
-                                      "10 20 stretch 2\n"
+                                      "10 20 stretch condensed\n"
                                       "20 -1 weight 700\n");
 
   out = pango_attr_list_filter (list, never_true, NULL);
@@ -782,7 +749,7 @@ test_iter (void)
   PangoAttribute *attr;
   PangoAttrIterator *iter;
   PangoAttrIterator *copy;
-  gint start, end;
+  int start, end;
 
   /* Empty list */
   list = pango_attr_list_new ();
@@ -882,7 +849,7 @@ test_iter_get_font (void)
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10 * PANGO_SCALE);
   pango_attr_list_insert (list, attr);
-  attr = attribute_from_string ("0 -1 family Times");
+  attr = attribute_from_string ("0 -1 family \"Times\"");
   pango_attr_list_insert (list, attr);
   attr = attribute_from_string ("10 30 stretch condensed");
   pango_attr_list_insert (list, attr);
@@ -892,7 +859,7 @@ test_iter_get_font (void)
   pango_attr_list_insert (list, attr);
   attr = attribute_from_string ("20 -1 rise 100");
   pango_attr_list_insert (list, attr);
-  attr = attribute_from_string ("20 -1 fallback 0");
+  attr = attribute_from_string ("20 -1 fallback false");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
@@ -943,7 +910,7 @@ test_iter_get_attrs (void)
   list = pango_attr_list_new ();
   attr = pango_attr_size_new (10 * PANGO_SCALE);
   pango_attr_list_insert (list, attr);
-  attr = attribute_from_string ("0 -1 family Times");
+  attr = attribute_from_string ("0 -1 family \"Times\"");
   pango_attr_list_insert (list, attr);
   attr = attribute_from_string ("10 30 stretch condensed");
   pango_attr_list_insert (list, attr);
@@ -951,31 +918,31 @@ test_iter_get_attrs (void)
   pango_attr_list_insert (list, attr);
   attr = attribute_from_string ("20 -1 rise 100");
   pango_attr_list_insert (list, attr);
-  attr = attribute_from_string ("20 -1 fallback 0");
+  attr = attribute_from_string ("20 -1 fallback false");
   pango_attr_list_insert (list, attr);
 
   iter = pango_attr_list_get_iterator (list);
   assert_attr_iterator (iter, "0 -1 size 10240\n"
-                              "0 -1 family Times\n");
+                              "0 -1 family \"Times\"\n");
 
   pango_attr_iterator_next (iter);
   assert_attr_iterator (iter, "0 -1 size 10240\n"
-                              "0 -1 family Times\n"
-                              "10 30 stretch 2\n"
+                              "0 -1 family \"Times\"\n"
+                              "10 30 stretch condensed\n"
                               "10 20 language ja-jp\n");
 
   pango_attr_iterator_next (iter);
   assert_attr_iterator (iter, "0 -1 size 10240\n"
-                              "0 -1 family Times\n"
-                              "10 30 stretch 2\n"
+                              "0 -1 family \"Times\"\n"
+                              "10 30 stretch condensed\n"
                               "20 -1 rise 100\n"
-                              "20 -1 fallback 0\n");
+                              "20 -1 fallback false\n");
 
   pango_attr_iterator_next (iter);
   assert_attr_iterator (iter, "0 -1 size 10240\n"
-                              "0 -1 family Times\n"
+                              "0 -1 family \"Times\"\n"
                               "20 -1 rise 100\n"
-                              "20 -1 fallback 0\n");
+                              "20 -1 fallback false\n");
 
   pango_attr_iterator_next (iter);
   g_assert_null (pango_attr_iterator_get_attrs (iter));
@@ -990,15 +957,15 @@ test_list_update (void)
   PangoAttrList *list;
 
   list = pango_attr_list_from_string ("0 200 rise 100\n"
-                                      "5 15 family Times\n"
+                                      "5 15 family \"Times\"\n"
                                       "10 11 size 10240\n"
-                                      "11 100 fallback 0\n"
-                                      "30 60 stretch 2\n");
+                                      "11 100 fallback false\n"
+                                      "30 60 stretch condensed\n");
 
   pango_attr_list_update (list, 8, 10, 20);
 
   assert_attr_list (list, "0 210 rise 100\n"
-                          "5 8 family Times\n"
+                          "5 8 family \"Times\"\n"
                           "28 110 fallback false\n"
                           "40 70 stretch condensed\n");
 
@@ -1025,11 +992,11 @@ test_list_update3 (void)
 {
   PangoAttrList *list;
 
-  list = pango_attr_list_from_string ("5 4294967285 family Times\n");
+  list = pango_attr_list_from_string ("5 4294967285 family \"Times\"\n");
 
   pango_attr_list_update (list, 8, 10, 30);
 
-  assert_attr_list (list, "5 -1 family Times\n");
+  assert_attr_list (list, "5 -1 family \"Times\"\n");
 
   pango_attr_list_unref (list);
 }
@@ -1110,28 +1077,28 @@ test_insert (void)
   PangoAttribute *attr;
 
   list = pango_attr_list_from_string ("0 200 rise 100\n"
-                                      "5 15 family Times\n"
+                                      "5 15 family \"Times\"\n"
                                       "10 11 size 10240\n"
-                                      "11 100 fallback 0\n"
-                                      "30 60 stretch 2\n");
+                                      "11 100 fallback false\n"
+                                      "30 60 stretch condensed\n");
 
-  attr = attribute_from_string ("10 25 family Times");
+  attr = attribute_from_string ("10 25 family \"Times\"");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "0 200 rise 100\n"
-                          "5 25 family Times\n"
+                          "5 25 family \"Times\"\n"
                           "10 11 size 10240\n"
                           "11 100 fallback false\n"
                           "30 60 stretch condensed\n");
 
-  attr = attribute_from_string ("11 25 family Futura");
+  attr = attribute_from_string ("11 25 family \"Futura\"");
   pango_attr_list_insert (list, attr);
 
   assert_attr_list (list, "0 200 rise 100\n"
-                          "5 25 family Times\n"
+                          "5 25 family \"Times\"\n"
                           "10 11 size 10240\n"
                           "11 100 fallback false\n"
-                          "11 25 family Futura\n"
+                          "11 25 family \"Futura\"\n"
                           "30 60 stretch condensed\n");
 
   pango_attr_list_unref (list);
@@ -1144,21 +1111,21 @@ test_insert2 (void)
   PangoAttribute *attr;
 
   list = pango_attr_list_from_string ("0 200 rise 100\n"
-                                      "5 15 family Times\n"
+                                      "5 15 family \"Times\"\n"
                                       "10 11 size 10240\n"
-                                      "11 100 fallback 0\n"
-                                      "20 30 family Times\n"
-                                      "30 40 family Futura\n"
-                                      "30 60 stretch 2\n");
+                                      "11 100 fallback false\n"
+                                      "20 30 family \"Times\"\n"
+                                      "30 40 family \"Futura\"\n"
+                                      "30 60 stretch condensed\n");
 
-  attr = attribute_from_string ("10 35 family Times");
+  attr = attribute_from_string ("10 35 family \"Times\"");
   pango_attr_list_change (list, attr);
 
   assert_attr_list (list, "0 200 rise 100\n"
-                          "5 35 family Times\n"
+                          "5 35 family \"Times\"\n"
                           "10 11 size 10240\n"
                           "11 100 fallback false\n"
-                          "35 40 family Futura\n"
+                          "35 40 family \"Futura\"\n"
                           "30 60 stretch condensed\n");
 
   pango_attr_list_unref (list);
@@ -1180,10 +1147,10 @@ test_merge (void)
   PangoAttrList *list2;
 
   list = pango_attr_list_from_string ("0 200 rise 100\n"
-                                      "5 15 family Times\n"
+                                      "5 15 family \"Times\"\n"
                                       "10 11 size 10240\n"
-                                      "11 100 fallback 0\n"
-                                      "30 60 stretch 2\n");
+                                      "11 100 fallback false\n"
+                                      "30 60 stretch condensed\n");
 
   list2 = pango_attr_list_from_string ("11 13 size 10240\n"
                                        "13 15 size 11264\n"
@@ -1192,7 +1159,7 @@ test_merge (void)
   pango_attr_list_filter (list2, attr_list_merge_filter, list);
 
   assert_attr_list (list, "0 200 rise 100\n"
-                          "5 15 family Times\n"
+                          "5 15 family \"Times\"\n"
                           "10 13 size 10240\n"
                           "11 100 fallback false\n"
                           "13 15 size 11264\n"
@@ -1218,7 +1185,7 @@ test_merge2 (void)
   attr = attribute_from_string ("2 3 foreground #ffff00000000");
   pango_attr_list_change (list, attr);
 
-  assert_attr_list (list, "0 10 underline single\n"
+  assert_attr_list (list, "0 10 underline solid\n"
                           "0 2 foreground #00000000ffff\n"
                           "2 3 foreground #ffff00000000\n"
                           "3 10 foreground #00000000ffff");
@@ -1226,7 +1193,7 @@ test_merge2 (void)
   attr = attribute_from_string ("3 4 foreground #0000ffff0000");
   pango_attr_list_change (list, attr);
 
-  assert_attr_list (list, "0 10 underline single\n"
+  assert_attr_list (list, "0 10 underline solid\n"
                           "0 2 foreground #00000000ffff\n"
                           "2 3 foreground #ffff00000000\n"
                           "3 4 foreground #0000ffff0000\n"
@@ -1235,7 +1202,7 @@ test_merge2 (void)
   attr = attribute_from_string ("4 5 foreground #00000000ffff");
   pango_attr_list_change (list, attr);
 
-  assert_attr_list (list, "0 10 underline single\n"
+  assert_attr_list (list, "0 10 underline solid\n"
                           "0 2 foreground #00000000ffff\n"
                           "2 3 foreground #ffff00000000\n"
                           "3 4 foreground #0000ffff0000\n"
@@ -1258,25 +1225,25 @@ print_tags_for_attributes (PangoAttrIterator *iter,
   if (attr)
     g_string_append_printf (s, "%d  %d rise %d\n",
                             attr->start_index, attr->end_index,
-                            ((PangoAttrInt*)attr)->value);
+                            attr->int_value);
 
   attr = pango_attr_iterator_get (iter, PANGO_ATTR_SIZE);
   if (attr)
     g_string_append_printf (s, "%d  %d size %d\n",
                             attr->start_index, attr->end_index,
-                            ((PangoAttrInt*)attr)->value);
+                            attr->int_value);
 
   attr = pango_attr_iterator_get (iter, PANGO_ATTR_SCALE);
   if (attr)
     g_string_append_printf (s, "%d  %d scale %f\n",
                             attr->start_index, attr->end_index,
-                            ((PangoAttrFloat*)attr)->value);
+                            attr->double_value);
 
   attr = pango_attr_iterator_get (iter, PANGO_ATTR_ALLOW_BREAKS);
   if (attr)
     g_string_append_printf (s, "%d  %d allow_breaks %d\n",
                             attr->start_index, attr->end_index,
-                            ((PangoAttrInt*)attr)->value);
+                            attr->int_value);
 }
 
 static void
@@ -1338,6 +1305,68 @@ test_iter_epsilon_zero (void)
   g_string_free (s, TRUE);
 }
 
+static void
+test_gnumeric_splice (void)
+{
+  PangoAttrList *list, *list2;
+
+  list = pango_attr_list_from_string ("0 -1 font-desc \"Sans 10\"\n");
+  list2 = pango_attr_list_from_string ("1 2 weight bold\n");
+
+  pango_attr_list_splice (list, list2, 0, 0);
+
+  assert_attr_list (list, "0 4294967295 font-desc \"Sans 10\"\n"
+                          "1 2 weight bold\n");
+
+  pango_attr_list_unref (list);
+  pango_attr_list_unref (list2);
+}
+
+static void
+test_change_order (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = pango_attr_list_from_string ("1 -1 font-features \"tnum=1\"\n"
+                                      "0 20 font-desc \"sans-serif\"\n"
+                                      "0 9 size 102400\n");
+
+  attr = pango_attr_font_features_new ("tnum=2");
+  attr->end_index = 9;
+
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "0 9 font-features \"tnum=2\"\n"
+                          "0 20 font-desc \"sans-serif\"\n"
+                          "0 9 size 102400\n"
+                          "9 4294967295 font-features \"tnum=1\"\n");
+
+  pango_attr_list_unref (list);
+}
+
+static void
+test_pitivi_crash (void)
+{
+  PangoAttrList *list;
+  PangoAttribute *attr;
+
+  list = pango_attr_list_from_string ("0 8 font-features \"tnum=1\"\n"
+                                      "0 20 font-desc \"sans-serif\"\n"
+                                      "0 9 size 102400\n");
+
+  attr = pango_attr_font_features_new ("tnum=2");
+  attr->end_index = 9;
+
+  pango_attr_list_change (list, attr);
+
+  assert_attr_list (list, "0 9 font-features \"tnum=2\"\n"
+                          "0 20 font-desc \"sans-serif\"\n"
+                          "0 9 size 102400\n");
+
+  pango_attr_list_unref (list);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1346,7 +1375,6 @@ main (int argc, char *argv[])
   g_test_add_func ("/attributes/basic", test_attributes_basic);
   g_test_add_func ("/attributes/equal", test_attributes_equal);
   g_test_add_func ("/attributes/register", test_attributes_register);
-  g_test_add_func ("/attributes/binding-helpers", test_binding_helpers);
   g_test_add_func ("/attributes/list/basic", test_list);
   g_test_add_func ("/attributes/list/change", test_list_change);
   g_test_add_func ("/attributes/list/change2", test_list_change2);
@@ -1377,6 +1405,9 @@ main (int argc, char *argv[])
   g_test_add_func ("/attributes/iter/get_font", test_iter_get_font);
   g_test_add_func ("/attributes/iter/get_attrs", test_iter_get_attrs);
   g_test_add_func ("/attributes/iter/epsilon_zero", test_iter_epsilon_zero);
+  g_test_add_func ("/attributes/gnumeric-splice", test_gnumeric_splice);
+  g_test_add_func ("/attributes/list/change_order", test_change_order);
+  g_test_add_func ("/attributes/pitivi-crash", test_pitivi_crash);
 
   return g_test_run ();
 }
