@@ -11,7 +11,7 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
@@ -30,7 +30,7 @@
 
 #include <glib.h>
 
-#include <pango/pangocairo.h>
+#include <pango/pango.h>
 
 static void verbose (const char *format, ...) G_GNUC_PRINTF (1, 2);
 static void
@@ -80,8 +80,8 @@ static void
 iter_char_test (PangoLayout *layout)
 {
   PangoRectangle   extents, run_extents;
-  PangoLayoutIter *iter;
-  PangoLayoutRun  *run;
+  PangoLineIter *iter;
+  PangoRun  *run;
   int              num_chars;
   int              i, index, offset;
   int              leading_x, trailing_x, x0, x1;
@@ -91,139 +91,147 @@ iter_char_test (PangoLayout *layout)
   text = pango_layout_get_text (layout);
   num_chars = g_utf8_strlen (text, -1);
 
-  iter = pango_layout_get_iter (layout);
+  iter = pango_lines_get_iter (pango_layout_get_lines (layout));
   iter_next_ok = TRUE;
 
   for (i = 0 ; i < num_chars; ++i)
     {
-      gchar *char_str;
-      g_assert_true (iter_next_ok);
+      char *char_str;
+      g_assert (iter_next_ok);
 
-      index = pango_layout_iter_get_index (iter);
+      index = pango_line_iter_get_index (iter);
       ptr = text + index;
       char_str = g_strndup (ptr, g_utf8_next_char (ptr) - ptr);
       verbose ("i=%d (visual), index = %d '%s':\n",
-	       i, index, char_str);
+               i, index, char_str);
       g_free (char_str);
 
-      pango_layout_iter_get_char_extents (iter, &extents);
+      pango_line_iter_get_char_extents (iter, &extents);
       verbose ("  char extents: x=%d,y=%d w=%d,h=%d\n",
-	       extents.x, extents.y,
-	       extents.width, extents.height);
+               extents.x, extents.y,
+               extents.width, extents.height);
 
-      run = pango_layout_iter_get_run (iter);
+      run = pango_line_iter_get_run (iter);
 
       if (run)
-	{
+        {
           PangoFontDescription *desc;
           char *str;
+          PangoItem *item;
+          const PangoAnalysis *analysis;
+          PangoGlyphString *glyphs;
+          int length;
 
-	  /* Get needed data for the GlyphString */
-	  pango_layout_iter_get_run_extents(iter, NULL, &run_extents);
-	  offset = run->item->offset;
-	  rtl = run->item->analysis.level%2;
-          desc = pango_font_describe (run->item->analysis.font);
+          item = pango_run_get_item (run);
+          analysis = pango_item_get_analysis (item);
+          glyphs = pango_run_get_glyphs (run);
+
+          /* Get needed data for the GlyphString */
+          pango_line_iter_get_run_extents (iter, NULL, &run_extents);
+          offset = pango_item_get_byte_offset (item);
+          length = pango_item_get_byte_length (item);
+
+          rtl = pango_analysis_get_bidi_level (analysis) % 2;
+          desc = pango_font_describe (pango_analysis_get_font (analysis));
           str = pango_font_description_to_string (desc);
-	  verbose ("  (current run: font=%s,offset=%d,x=%d,len=%d,rtl=%d)\n",
-		   str, offset, run_extents.x, run->item->length, rtl);
+          verbose ("  (current run: font=%s,offset=%d,x=%d,len=%d,rtl=%d)\n",
+                   str, offset, run_extents.x, length, rtl);
           g_free (str);
           pango_font_description_free (desc);
 
-	  /* Calculate expected x result using index_to_x */
-	  pango_glyph_string_index_to_x (run->glyphs,
-					 (char *)(text + offset), run->item->length,
-					 &run->item->analysis,
-					 index - offset, FALSE, &leading_x);
-	  pango_glyph_string_index_to_x (run->glyphs,
-					 (char *)(text + offset), run->item->length,
-					 &run->item->analysis,
-					 index - offset, TRUE, &trailing_x);
+          /* Calculate expected x result using index_to_x */
+          pango_glyph_string_index_to_x (glyphs,
+                                         (char *)(text + offset), length,
+                                         pango_item_get_analysis (item),
+                                         index - offset, FALSE, &leading_x);
+          pango_glyph_string_index_to_x (glyphs,
+                                         (char *)(text + offset), length,
+                                         pango_item_get_analysis (item),
+                                         index - offset, TRUE, &trailing_x);
 
-	  x0 = run_extents.x + MIN (leading_x, trailing_x);
-	  x1 = run_extents.x + MAX (leading_x, trailing_x);
+          x0 = run_extents.x + MIN (leading_x, trailing_x);
+          x1 = run_extents.x + MAX (leading_x, trailing_x);
 
-	  verbose ("  (index_to_x ind=%d: expected x=%d, width=%d)\n",
-		   index - offset, x0, x1 - x0);
+          verbose ("  (index_to_x ind=%d: expected x=%d, width=%d)\n",
+                   index - offset, x0, x1 - x0);
 
-	  g_assert_cmpint (extents.x, ==, x0);
-	  g_assert_cmpint (extents.width, ==, x1 - x0);
-	}
+          g_assert (extents.x == x0);
+          g_assert (extents.width == x1 - x0);
+        }
       else
-	{
-	  /* We're on a line terminator */
-	}
+        {
+          /* We're on a line terminator */
+        }
 
-      iter_next_ok = pango_layout_iter_next_char (iter);
+      iter_next_ok = pango_line_iter_next_char (iter);
       verbose ("more to go? %d\n", iter_next_ok);
     }
 
   /* There should be one character position iterator for each character in the
    * input string */
-  g_assert_false (iter_next_ok);
+  g_assert (!iter_next_ok);
 
-  pango_layout_iter_free (iter);
+  pango_line_iter_free (iter);
 }
 
 static void
 iter_cluster_test (PangoLayout *layout)
 {
   PangoRectangle   extents;
-  PangoLayoutIter *iter;
+  PangoLineIter *iter;
   int              index;
   gboolean         iter_next_ok;
-  PangoLayoutLine *last_line = NULL;
+  PangoLine       *last_line = NULL;
   int              expected_next_x = 0;
 
-  iter = pango_layout_get_iter (layout);
+  iter = pango_lines_get_iter (pango_layout_get_lines (layout));
   iter_next_ok = TRUE;
 
   while (iter_next_ok)
     {
-      PangoLayoutLine *line = pango_layout_iter_get_line (iter);
+      PangoLine *line = pango_line_iter_get_line (iter);
 
       /* Every cluster is part of a run */
-      g_assert_true (pango_layout_iter_get_run (iter));
+      g_assert (pango_line_iter_get_run (iter));
 
-      index = pango_layout_iter_get_index (iter);
+      index = pango_line_iter_get_index (iter);
 
-      pango_layout_iter_get_cluster_extents (iter, NULL, &extents);
+      pango_line_iter_get_cluster_extents (iter, NULL, &extents);
 
-      iter_next_ok = pango_layout_iter_next_cluster (iter);
+      iter_next_ok = pango_line_iter_next_cluster (iter);
 
       verbose ("index = %d:\n", index);
       verbose ("  cluster extents: x=%d,y=%d w=%d,h=%d\n",
-	       extents.x, extents.y,
-	       extents.width, extents.height);
+               extents.x, extents.y,
+               extents.width, extents.height);
       verbose ("more to go? %d\n", iter_next_ok);
 
       /* All the clusters on a line should be next to each other and occupy
        * the entire line. They advance linearly from left to right */
-      g_assert_cmpint (extents.width, >=, 0);
+      g_assert (extents.width >= 0);
 
       if (last_line == line)
-	g_assert_cmpint (extents.x, ==, expected_next_x);
+        g_assert (extents.x == expected_next_x);
 
       expected_next_x = extents.x + extents.width;
 
       last_line = line;
     }
 
-  g_assert_false (iter_next_ok);
+  g_assert (!iter_next_ok);
 
-  pango_layout_iter_free (iter);
+  pango_line_iter_free (iter);
 }
 
 static void
-test_layout_iter (void)
+test_line_iter (void)
 {
   const char  **ptext;
-  PangoFontMap *fontmap;
   PangoContext *context;
   PangoFontDescription *font_desc;
   PangoLayout  *layout;
 
-  fontmap = pango_cairo_font_map_new ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
   font_desc = pango_font_description_from_string ("cantarell 11");
   pango_context_set_font_description (context, font_desc);
 
@@ -235,7 +243,7 @@ test_layout_iter (void)
       verbose ("--------- checking next text ----------\n");
       verbose (" <%s>\n", *ptext);
       verbose ( "len=%ld, bytes=%ld\n",
-		(long)g_utf8_strlen (*ptext, -1), (long)strlen (*ptext));
+                (long)g_utf8_strlen (*ptext, -1), (long)strlen (*ptext));
 
       pango_layout_set_text (layout, *ptext, -1);
       iter_char_test (layout);
@@ -245,71 +253,6 @@ test_layout_iter (void)
   g_object_unref (layout);
   g_object_unref (context);
   pango_font_description_free (font_desc);
-  g_object_unref (fontmap);
-}
-
-static void
-test_glyphitem_iter (void)
-{
-  PangoFontMap *fontmap;
-  PangoContext *context;
-  PangoFontDescription *font_desc;
-  PangoLayout  *layout;
-  PangoLayoutLine *line;
-  const char *text;
-  GSList *l;
-
-  fontmap = pango_cairo_font_map_new ();
-  context = pango_font_map_create_context (fontmap);
-  font_desc = pango_font_description_from_string ("cantarell 11");
-  pango_context_set_font_description (context, font_desc);
-
-  layout = pango_layout_new (context);
-  /* This shouldn't form any ligatures. */
-  pango_layout_set_text (layout, "test تست", -1);
-  text = pango_layout_get_text (layout);
-
-  line = pango_layout_get_line (layout, 0);
-  for (l = line->runs; l; l = l->next)
-  {
-    PangoGlyphItem *run = l->data;
-    int direction;
-
-    for (direction = 0; direction < 2; direction++)
-    {
-      PangoGlyphItemIter iter;
-      gboolean have_cluster;
-      PangoGlyphItemIter *iter2;
-
-      for (have_cluster = direction ?
-	     pango_glyph_item_iter_init_start (&iter, run, text) :
-	     pango_glyph_item_iter_init_end (&iter, run, text);
-	   have_cluster;
-	   have_cluster = direction ?
-	     pango_glyph_item_iter_next_cluster (&iter) :
-	     pango_glyph_item_iter_prev_cluster (&iter))
-      {
-        verbose ("start index %d end index %d\n", iter.start_index, iter.end_index);
-        g_assert_true (iter.start_index < iter.end_index);
-        g_assert_true (iter.start_index + 2 >= iter.end_index);
-        g_assert_true (iter.start_char + 1 == iter.end_char);
-
-        iter2 = pango_glyph_item_iter_copy (&iter);
-        g_assert_true (iter2->start_glyph == iter.start_glyph);
-        g_assert_true (iter2->start_index == iter.start_index);
-        g_assert_true (iter2->start_char == iter.start_char);
-        g_assert_true (iter2->end_glyph == iter.end_glyph);
-        g_assert_true (iter2->end_index == iter.end_index);
-        g_assert_true (iter2->end_char == iter.end_char);
-        pango_glyph_item_iter_free (iter2);
-      }
-    }
-  }
-
-  g_object_unref (layout);
-  g_object_unref (context);
-  pango_font_description_free (font_desc);
-  g_object_unref (fontmap);
 }
 
 int
@@ -317,8 +260,7 @@ main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
-  g_test_add_func ("/layout/iter", test_layout_iter);
-  g_test_add_func ("/layout/glyphitem-iter", test_glyphitem_iter);
+  g_test_add_func ("/layout/iter", test_line_iter);
 
   return g_test_run ();
 }
