@@ -23,7 +23,7 @@
 
 #include <locale.h>
 #include <gio/gio.h>
-#include <pango/pangocairo.h>
+#include <pango/pango.h>
 
 #define N_SENTENCES 20
 
@@ -41,8 +41,6 @@ random_word (PangoDirection dir)
         return ltr_words[g_test_rand_int_range (0, n_ltr_words)];
       case PANGO_DIRECTION_RTL:
         return rtl_words[g_test_rand_int_range (0, n_rtl_words)];
-      case PANGO_DIRECTION_TTB_LTR:
-      case PANGO_DIRECTION_TTB_RTL:
       case PANGO_DIRECTION_WEAK_LTR:
       case PANGO_DIRECTION_WEAK_RTL:
       case PANGO_DIRECTION_NEUTRAL:
@@ -93,12 +91,12 @@ compare_size (gconstpointer a,
 
 static void
 layout_check_size (PangoLayout *layout,
-                   int          width,
-                   Size        *out_size)
+                   int                width,
+                   Size              *out_size)
 {
   out_size->set_width = width;
   pango_layout_set_width (layout, width);
-  pango_layout_get_size (layout, &out_size->width, &out_size->height);
+  pango_lines_get_size (pango_layout_get_lines (layout), &out_size->width, &out_size->height);
 }
 
 static void
@@ -106,7 +104,6 @@ test_wrap_char (gconstpointer data)
 {
   PangoDirection dir = GPOINTER_TO_UINT (data);
   PangoFontDescription *desc;
-  PangoFontMap *fontmap;
   PangoContext *context;
   PangoLayout *layout;
   char *sentence;
@@ -114,8 +111,13 @@ test_wrap_char (gconstpointer data)
   Size sizes[100];
   gsize i, j;
 
-  fontmap = pango_cairo_font_map_new ();
-  context = pango_font_map_create_context (fontmap);
+  if (dir != PANGO_DIRECTION_LTR)
+    {
+      g_test_skip ("Skipping known broken tetts");
+      return;
+    }
+
+  context = pango_context_new ();
   desc = pango_font_description_from_string ("Sans 10");
   layout = pango_layout_new (context);
   pango_layout_set_font_description (layout, desc);
@@ -171,16 +173,13 @@ test_wrap_char (gconstpointer data)
 
   g_object_unref (layout);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
 
-#if 0
 static void
 test_wrap_char_min_width (gconstpointer data)
 {
   PangoDirection dir = GPOINTER_TO_UINT (data);
   PangoFontDescription *desc;
-  PangoFontMap *fontmap;
   PangoContext *context;
   PangoLayout *test_layout, *ref_layout;
   char *sentence, *s;
@@ -188,8 +187,7 @@ test_wrap_char_min_width (gconstpointer data)
   gsize j;
   int test_width, ref_width;
 
-  fontmap = pango_cairo_font_map_new ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
   desc = pango_font_description_from_string ("Sans 10");
   ref_layout = pango_layout_new (context);
   pango_layout_set_font_description (ref_layout, desc);
@@ -215,8 +213,8 @@ test_wrap_char_min_width (gconstpointer data)
       g_string_free (ref_string, TRUE);
       g_free (sentence);
 
-      pango_layout_get_size (test_layout, &test_width, NULL);
-      pango_layout_get_size (ref_layout, &ref_width, NULL);
+      pango_lines_get_size (pango_layout_get_lines (test_layout), &test_width, NULL);
+      pango_lines_get_size (pango_layout_get_lines (ref_layout), &ref_width, NULL);
 
       g_assert_cmpint (test_width, <=, ref_width);
     }
@@ -224,9 +222,7 @@ test_wrap_char_min_width (gconstpointer data)
   g_object_unref (test_layout);
   g_object_unref (ref_layout);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
-#endif
 
 static char **
 load_hunspell_words (const char  *language)
@@ -313,14 +309,12 @@ main (int argc, char *argv[])
   rtl_words = init_rtl_words ();
   n_rtl_words = g_strv_length (rtl_words);
 
-#if 0
   g_test_add_data_func ("/layout/ltr/wrap-char", GUINT_TO_POINTER (PANGO_DIRECTION_LTR), test_wrap_char);
   g_test_add_data_func ("/layout/rtl/wrap-char", GUINT_TO_POINTER (PANGO_DIRECTION_RTL), test_wrap_char);
   g_test_add_data_func ("/layout/any/wrap-char", GUINT_TO_POINTER (PANGO_DIRECTION_NEUTRAL), test_wrap_char);
   g_test_add_data_func ("/layout/ltr/wrap-char-min-width", GUINT_TO_POINTER (PANGO_DIRECTION_LTR), test_wrap_char_min_width);
   g_test_add_data_func ("/layout/rtl/wrap-char-min-width", GUINT_TO_POINTER (PANGO_DIRECTION_RTL), test_wrap_char_min_width);
   g_test_add_data_func ("/layout/any/wrap-char-min-width", GUINT_TO_POINTER (PANGO_DIRECTION_NEUTRAL), test_wrap_char_min_width);
-#endif
 
   result = g_test_run ();
 
