@@ -22,45 +22,34 @@
 
 #include <glib.h>
 #include <pango/pangocairo.h>
-#include <pango/pangocairo-fc.h>
 #include <gio/gio.h>
 
-static PangoFontMap *generate_font_map (void);
+#include "test-common.h"
 
 static void
 test_serialize_attr_list (void)
 {
   const char *valid[] = {
     "5 16 style italic",
-    "    5     16     style      italic     ",
-    "0 -1 language    de    ",
-    "0 1 family    Times",
-    "0 1 family    \"Times\"    ",
-    "0 1 family    \"Times\\n\\\"New\\\" Roman\"    ",
     "0 10 foreground red, 5 15 weight bold, 0 200 font-desc \"Sans Small-Caps 10\"",
     "0 10 foreground red\n5 15 weight bold\n0 200 font-desc \"Sans Small-Caps 10\"",
     "  0   10   fallback  false,\n 5 15 weight semilight\n\n \n \n",
     "0 100 font-desc \"Cantarell, Sans, Italic Ultra-Light 64\", 10 11 weight 100",
     "0 -1 size 10",
     "0 1 weight 700, 2 4 weight book",
-    "0 200 rise 100\n5 15 family Times\n10 11 size 10240\n11 100 fallback 0\n30 60 stretch 2\n",
-    "",
+    "0 200 rise 100\n5 15 family \"Times\"\n10 11 size 10240\n11 100 fallback false\n30 60 stretch condensed\n",
+    ""
   };
   const char *roundtripped[] = {
     "5 16 style italic",
-    "5 16 style italic",
-    "0 4294967295 language de",
-    "0 1 family \"Times\"",
-    "0 1 family \"Times\"",
-    "0 1 family \"Times\\n\\\"New\\\" Roman\"",
-    "0 10 foreground #ffff00000000\n5 15 weight bold\n0 200 font-desc \"Sans Small-Caps 10\"",
-    "0 10 foreground #ffff00000000\n5 15 weight bold\n0 200 font-desc \"Sans Small-Caps 10\"",
+    "0 10 foreground #ffff00000000ffff\n5 15 weight bold\n0 200 font-desc \"Sans Small-Caps 10\"",
+    "0 10 foreground #ffff00000000ffff\n5 15 weight bold\n0 200 font-desc \"Sans Small-Caps 10\"",
     "0 10 fallback false\n5 15 weight semilight",
     "0 100 font-desc \"Cantarell,Sans Ultra-Light Italic 64\"\n10 11 weight thin",
     "0 4294967295 size 10",
     "0 1 weight bold\n2 4 weight book",
     "0 200 rise 100\n5 15 family \"Times\"\n10 11 size 10240\n11 100 fallback false\n30 60 stretch condensed",
-    "",
+    ""
   };
   const char *invalid[] = {
     "not an attr list",
@@ -144,7 +133,6 @@ test_serialize_tab_array (void)
 static void
 test_serialize_font (void)
 {
-  PangoFontMap *fontmap;
   PangoContext *context;
   PangoFontDescription *desc;
   PangoFont *font;
@@ -154,27 +142,31 @@ test_serialize_font (void)
   GError *error = NULL;
   const char *expected =
     "{\n"
-    "  \"description\" : \"Cantarell 20 @wght=600\",\n"
+    "  \"description\" : \"Cantarell Italic 20 @wght=600\",\n"
     "  \"checksum\" : \"5bcb6ee14ee9d210b2e91d643de1fe456e9d1aea770983fdb05951545efebbe2\",\n"
     "  \"variations\" : {\n"
     "    \"wght\" : 5583\n"
     "  },\n"
     "  \"matrix\" : [\n"
     "    1,\n"
-    "    -0,\n"
-    "    -0,\n"
+    "    0.20000000000000001,\n"
+    "    0,\n"
     "    1,\n"
     "    0,\n"
     "    0\n"
     "  ]\n"
     "}";
 
-  fontmap = generate_font_map ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
   desc = pango_font_description_from_string ("Cantarell Italic 20 @wght=600");
   font = pango_context_load_font (context, desc);
 
   bytes = pango_font_serialize (font);
+  if (strcmp (g_bytes_get_data (bytes, NULL), expected) != 0)
+    {
+      g_print ("expected\n%s\n", expected);
+      g_print ("got\n%s\n", (char* )g_bytes_get_data (bytes, NULL));
+    }
   g_assert_cmpstr (g_bytes_get_data (bytes, NULL), ==, expected);
 
   font2 = pango_font_deserialize (context, bytes, &error);
@@ -191,7 +183,6 @@ test_serialize_font (void)
   g_bytes_unref (bytes);
   g_bytes_unref (bytes2);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
 
 static void
@@ -202,7 +193,6 @@ test_serialize_layout_minimal (void)
     "  \"text\" : \"Almost nothing\"\n"
     "}\n";
 
-  PangoFontMap *fontmap;
   PangoContext *context;
   GBytes *bytes;
   PangoLayout *layout;
@@ -210,8 +200,7 @@ test_serialize_layout_minimal (void)
   GBytes *out_bytes;
   const char *str;
 
-  fontmap = generate_font_map ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
 
   bytes = g_bytes_new_static (test, strlen (test) + 1);
 
@@ -222,7 +211,7 @@ test_serialize_layout_minimal (void)
   g_assert_null (pango_layout_get_attributes (layout));
   g_assert_null (pango_layout_get_tabs (layout));
   g_assert_null (pango_layout_get_font_description (layout));
-  g_assert_cmpint (pango_layout_get_alignment (layout), ==, PANGO_ALIGN_LEFT);
+  g_assert_cmpint (pango_layout_get_alignment (layout), ==, PANGO_ALIGN_NATURAL);
   g_assert_cmpint (pango_layout_get_width (layout), ==, -1);
 
   out_bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_DEFAULT);
@@ -235,7 +224,6 @@ test_serialize_layout_minimal (void)
   g_object_unref (layout);
   g_bytes_unref (bytes);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
 
 static void
@@ -248,18 +236,18 @@ test_serialize_layout_valid (void)
     "    {\n"
     "      \"end\" : 4,\n"
     "      \"type\" : \"foreground\",\n"
-    "      \"value\" : \"#ffff00000000\"\n"
+    "      \"value\" : \"#ffff00000000ffff\"\n"
     "    },\n"
     "    {\n"
     "      \"start\" : 5,\n"
     "      \"end\" : 8,\n"
     "      \"type\" : \"foreground\",\n"
-    "      \"value\" : \"#000080800000\"\n"
+    "      \"value\" : \"#000080800000ffff\"\n"
     "    },\n"
     "    {\n"
     "      \"start\" : 14,\n"
     "      \"type\" : \"foreground\",\n"
-    "      \"value\" : \"#ffff0000ffff\"\n"
+    "      \"value\" : \"#ffff0000ffffffff\"\n"
     "    }\n"
     "  ],\n"
     "  \"font\" : \"Sans Bold 32\",\n"
@@ -285,20 +273,17 @@ test_serialize_layout_valid (void)
     "  },\n"
     "  \"alignment\" : \"center\",\n"
     "  \"width\" : 350000,\n"
-    "  \"line-spacing\" : 1.5\n"
+    "  \"line-height\" : 1.5\n"
     "}\n";
 
-  PangoFontMap *fontmap;
   PangoContext *context;
   GBytes *bytes;
   PangoLayout *layout;
-  PangoTabArray *tabs;
   GError *error = NULL;
   GBytes *out_bytes;
   char *s;
 
-  fontmap = generate_font_map ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
 
   bytes = g_bytes_new_static (test, strlen (test) + 1);
 
@@ -307,15 +292,13 @@ test_serialize_layout_valid (void)
   g_assert_true (PANGO_IS_LAYOUT (layout));
   g_assert_cmpstr (pango_layout_get_text (layout), ==, "Some fun with layouts!");
   g_assert_nonnull (pango_layout_get_attributes (layout));
-  tabs = pango_layout_get_tabs (layout);
-  g_assert_nonnull (tabs);
-  pango_tab_array_free (tabs);
+  g_assert_nonnull (pango_layout_get_tabs (layout));
   s = pango_font_description_to_string (pango_layout_get_font_description (layout));
   g_assert_cmpstr (s, ==, "Sans Bold 32");
   g_free (s);
   g_assert_cmpint (pango_layout_get_alignment (layout), ==, PANGO_ALIGN_CENTER);
   g_assert_cmpint (pango_layout_get_width (layout), ==, 350000);
-  g_assert_cmpfloat_with_epsilon (pango_layout_get_line_spacing (layout), 1.5, 0.0001);
+  g_assert_cmpfloat_with_epsilon (pango_layout_get_line_height (layout), 1.5, 0.0001);
 
   out_bytes = pango_layout_serialize (layout, PANGO_LAYOUT_SERIALIZE_DEFAULT);
 
@@ -333,7 +316,6 @@ test_serialize_layout_valid (void)
 
   g_object_unref (layout);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
 
 static void
@@ -349,14 +331,12 @@ test_serialize_layout_context (void)
     "  \"text\" : \"Some fun with layouts!\"\n"
     "}\n";
 
-  PangoFontMap *fontmap;
   PangoContext *context;
   GBytes *bytes;
   PangoLayout *layout;
   GError *error = NULL;
 
-  fontmap = generate_font_map ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
 
   bytes = g_bytes_new_static (test, strlen (test) + 1);
 
@@ -372,7 +352,6 @@ test_serialize_layout_context (void)
   g_object_unref (layout);
   g_bytes_unref (bytes);
   g_object_unref (context);
-  g_object_unref (fontmap);
 }
 
 static void
@@ -390,7 +369,7 @@ test_serialize_layout_invalid (void)
       "    }\n"
       "  ]\n"
       "}\n",
-      PANGO_LAYOUT_DESERIALIZE_INVALID_VALUE
+      PANGO_LAYOUT_DESERIALIZE_MISSING_VALUE
     },
     {
       "{\n"
@@ -406,7 +385,7 @@ test_serialize_layout_invalid (void)
       "{\n"
       "  \"attributes\" : [\n"
       "    {\n"
-      "      \"type\" : \"alignment\",\n"
+      "      \"type\" : \"background\",\n"
       "      \"value\" : \"nonsense\"\n"
       "    }\n"
       "  ]\n"
@@ -429,11 +408,9 @@ test_serialize_layout_invalid (void)
     }
   };
 
-  PangoFontMap *fontmap;
   PangoContext *context;
 
-  fontmap = generate_font_map ();
-  context = pango_font_map_create_context (fontmap);
+  context = pango_context_new ();
 
   for (int i = 0; i < G_N_ELEMENTS (test); i++)
     {
@@ -453,41 +430,6 @@ test_serialize_layout_invalid (void)
     }
 
   g_object_unref (context);
-  g_object_unref (fontmap);
-}
-
-static PangoFontMap *
-generate_font_map (void)
-{
-  char *dir;
-  FcConfig *config;
-  PangoFontMap *map;
-  char *path;
-  gsize len;
-  char *conf;
-
-  dir = g_test_build_filename (G_TEST_DIST, "fonts", NULL);
-
-  map = g_object_new (PANGO_TYPE_CAIRO_FC_FONT_MAP, NULL);
-
-  config = FcConfigCreate ();
-
-  path = g_test_build_filename (G_TEST_DIST, "fonts/fonts.conf", NULL);
-  g_file_get_contents (path, &conf, &len, NULL);
-
-  if (!FcConfigParseAndLoadFromMemory (config, (const FcChar8 *) conf, TRUE))
-    g_error ("Failed to parse fontconfig configuration");
-
-  g_free (conf);
-  g_free (path);
-
-  FcConfigAppFontAddDir (config, (const FcChar8 *) dir);
-  pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (map), config);
-  FcConfigDestroy (config);
-
-  g_free (dir);
-
-  return map;
 }
 
 int
@@ -495,6 +437,8 @@ main (int argc, char *argv[])
 {
   g_test_init (&argc, &argv, NULL);
 
+  install_fonts ();
+
   g_test_add_func ("/serialize/attr-list", test_serialize_attr_list);
   g_test_add_func ("/serialize/tab-array", test_serialize_tab_array);
   g_test_add_func ("/serialize/font", test_serialize_font);
