@@ -23,13 +23,17 @@
 
 #include "viewer-render.h"
 #include "viewer-cairo.h"
+#include "userfont.h"
 
 #include <pango/pangocairo.h>
+#include <pango/pango-run.h>
 
 #include <hb-ot.h>
-#include <hb-glib.h>
+
 
 static int opt_annotate = 0;
+static gboolean opt_userfont = 0;
+static char **opt_font_file = NULL;
 
 typedef struct
 {
@@ -51,8 +55,45 @@ pangocairo_view_create (const PangoViewer *klass G_GNUC_UNUSED)
 
   instance->backend = cairo_viewer_iface_create (&instance->iface);
 
-  instance->fontmap = pango_cairo_font_map_new ();
-  pango_cairo_font_map_set_resolution (PANGO_CAIRO_FONT_MAP (instance->fontmap), opt_dpi);
+  if (opt_font_file != NULL)
+    {
+      PangoFontFace *first = NULL;
+
+      instance->fontmap = pango_font_map_new ();
+
+      for (int i = 0; opt_font_file[i]; i++)
+        {
+          PangoFontFace *face;
+
+          face = PANGO_FONT_FACE (pango_hb_face_new_from_file (opt_font_file[i],
+                                                               0, -1,
+                                                               NULL, NULL));
+
+          pango_font_map_add_face (instance->fontmap, face);
+
+          if (!first)
+            first = face;
+        }
+
+      /* We always need monospace, or Pango will complain */
+      if (!pango_font_map_get_family (instance->fontmap, "monospace"))
+        {
+          PangoGenericFamily *family;
+
+          family = pango_generic_family_new ("monospace");
+          pango_generic_family_add_family (family, pango_font_face_get_family (first));
+          pango_font_map_add_family (instance->fontmap, PANGO_FONT_FAMILY (family));
+        }
+    }
+  else
+    {
+      instance->fontmap = pango_font_map_new_default ();
+    }
+
+  pango_font_map_set_resolution (PANGO_FONT_MAP (instance->fontmap), opt_dpi);
+
+  if (opt_userfont)
+    add_userfont (instance->fontmap);
 
   instance->font_options = cairo_font_options_create ();
   if (opt_hinting != HINT_DEFAULT)
@@ -102,7 +143,7 @@ pangocairo_view_get_context (gpointer instance)
   CairoViewer *c = (CairoViewer *) instance;
   PangoContext *context;
 
-  context = pango_font_map_create_context (c->fontmap);
+  context = pango_context_new_with_font_map (c->fontmap);
   pango_cairo_context_set_font_options (context, c->font_options);
   pango_context_set_round_glyph_positions (context, !c->subpixel_positions);
 
@@ -207,9 +248,9 @@ render_callback (PangoLayout *layout,
       cairo_pattern_t *pattern;
       PangoRectangle ink, logical;
       double lw = cairo_get_line_width (cr);
-      PangoLayoutIter* iter;
+      PangoLineIter *iter;
 
-      pango_layout_get_extents (layout, &ink, &logical);
+      pango_lines_get_extents (pango_layout_get_lines (layout), &ink, &logical);
 
       if (annotate & ANNOTATE_GRAVITY_ROOF)
         {
@@ -287,15 +328,15 @@ render_callback (PangoLayout *layout,
           iter = pango_layout_get_iter (layout);
           do
             {
-              PangoLayoutLine *line = pango_layout_iter_get_line (iter);
+              PangoLine *line = pango_line_iter_get_line (iter);
               double width = (double)logical.width / PANGO_SCALE;
 
-              y = pango_layout_iter_get_baseline (iter);
+              y = pango_line_iter_get_line_baseline (iter);
               cairo_save (cr);
               cairo_translate (cr,
                              (double)logical.x / PANGO_SCALE + width * 0.5,
                              (double)y / PANGO_SCALE);
-              if (line->resolved_dir)
+              if (pango_line_get_resolved_direction (line))
                 cairo_scale (cr, -1, 1);
               cairo_move_to     (cr, -width * .5, -lw*0.2);
               cairo_rel_line_to (cr, +width * .9, -lw*0.3);
@@ -308,8 +349,8 @@ render_callback (PangoLayout *layout,
               cairo_fill (cr);
               cairo_restore (cr);
             }
-          while (pango_layout_iter_next_line (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_line (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -324,7 +365,9 @@ render_callback (PangoLayout *layout,
           iter = pango_layout_get_iter (layout);
           do
             {
-              PangoLayoutRun *run;
+              PangoRun *run;
+              PangoItem *item;
+              const PangoAnalysis *analysis;
               PangoRectangle rect;
               hb_font_t *hb_font;
               hb_ot_layout_baseline_tag_t baselines[] = {
@@ -341,38 +384,43 @@ render_callback (PangoLayout *layout,
               hb_tag_t script, lang;
               hb_position_t coord;
 
-              run = pango_layout_iter_get_run (iter);
+              run = pango_line_iter_get_run (iter);
               if (!run)
                 {
                   baseline_tag = 0;
                   continue;
                 }
 
+              item = pango_run_get_item (run);
+              analysis = pango_item_get_analysis (item);
+
               if (baseline_tag == 0)
                 {
-                  hb_script_t script = hb_glib_script_to_script (run->item->analysis.script);
+                  hb_script_t script = (hb_script_t) g_unicode_script_to_iso15924 (pango_analysis_get_script (analysis));
 
-                  if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
+                  if (pango_analysis_get_flags (analysis) & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
                     baseline_tag = HB_OT_LAYOUT_BASELINE_TAG_IDEO_EMBOX_CENTRAL;
                   else
                     baseline_tag = hb_ot_layout_get_horizontal_baseline_tag_for_script (script);
                 }
 
-              y = pango_layout_iter_get_run_baseline (iter);
-              pango_layout_iter_get_run_extents (iter, NULL, &rect);
+              y = pango_line_iter_get_run_baseline (iter);
+              pango_line_iter_get_run_extents (iter, NULL, &rect);
 
-              hb_font = pango_font_get_hb_font (run->item->analysis.font);
-              if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
+              hb_font = pango_font_get_hb_font (pango_analysis_get_font (analysis));
+              if (pango_analysis_get_flags (analysis) & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
                 dir = HB_DIRECTION_TTB;
               else
                 dir = HB_DIRECTION_LTR;
-              script = hb_glib_script_to_script (run->item->analysis.script);
+              script = (hb_script_t) g_unicode_script_to_iso15924 (pango_analysis_get_script (analysis));
               lang = HB_TAG_NONE;
 
               for (int i = 0; i < G_N_ELEMENTS (baselines); i++)
                 {
                   char buf[5] = { 0, };
 
+                  hb_tag_to_string (baselines[i], buf);
+
                   cairo_save (cr);
 
                   if (baselines[i] == baseline_tag)
@@ -386,7 +434,10 @@ render_callback (PangoLayout *layout,
                                                  script,
                                                  lang,
                                                  &coord))
-                    cairo_set_dash (cr, NULL, 0, 0);
+                    {
+                      g_print ("baseline %s, value %d\n", buf, coord);
+                      cairo_set_dash (cr, NULL, 0, 0);
+                    }
                   else
                     {
                       double dashes[] = { 4 * lw, 4 * lw };
@@ -398,6 +449,7 @@ render_callback (PangoLayout *layout,
                                                                lang,
                                                                &coord);
 
+                      g_print ("baseline %s, fallback value %d\n", buf, coord);
                       cairo_set_dash (cr, dashes, 2, 0);
                       cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
                     }
@@ -410,16 +462,13 @@ render_callback (PangoLayout *layout,
                   cairo_move_to (cr,
                                  (double)rect.x / PANGO_SCALE - 5,
                                  (double)(y - coord) / PANGO_SCALE - 5);
-                  hb_tag_to_string (baselines[i], buf);
-                  if (baselines[i] == baseline_tag)
-                    g_print ("dominant %s\n", buf);
                   cairo_show_text (cr, buf);
 
                   cairo_restore (cr);
                 }
             }
-          while (pango_layout_iter_next_run (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_run (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 #endif
@@ -461,7 +510,7 @@ render_callback (PangoLayout *layout,
             {
               PangoRectangle rect;
 
-              pango_layout_iter_get_line_extents (iter, NULL, &rect);
+              pango_line_iter_get_line_extents (iter, NULL, &rect);
               cairo_rectangle (cr,
                                (double)rect.x / PANGO_SCALE - lw / 2,
                                (double)rect.y / PANGO_SCALE - lw / 2,
@@ -469,8 +518,8 @@ render_callback (PangoLayout *layout,
                                (double)rect.height / PANGO_SCALE + lw);
               cairo_stroke (cr);
             }
-          while (pango_layout_iter_next_line (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_line (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -483,14 +532,14 @@ render_callback (PangoLayout *layout,
           iter = pango_layout_get_iter (layout);
           do
             {
-              PangoLayoutRun *run;
+              PangoRun *run;
               PangoRectangle rect;
 
-              run = pango_layout_iter_get_run (iter);
+              run = pango_line_iter_get_run (iter);
               if (!run)
                 continue;
 
-              pango_layout_iter_get_run_extents (iter, NULL, &rect);
+              pango_line_iter_get_run_extents (iter, NULL, &rect);
               cairo_rectangle (cr,
                                (double)rect.x / PANGO_SCALE - lw / 2,
                                (double)rect.y / PANGO_SCALE - lw / 2,
@@ -498,8 +547,8 @@ render_callback (PangoLayout *layout,
                                (double)rect.height / PANGO_SCALE + lw);
               cairo_stroke (cr);
             }
-          while (pango_layout_iter_next_run (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_run (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -514,7 +563,7 @@ render_callback (PangoLayout *layout,
             {
               PangoRectangle rect;
 
-              pango_layout_iter_get_cluster_extents (iter, NULL, &rect);
+              pango_line_iter_get_cluster_extents (iter, NULL, &rect);
               cairo_rectangle (cr,
                                (double)rect.x / PANGO_SCALE - lw / 2,
                                (double)rect.y / PANGO_SCALE - lw / 2,
@@ -522,8 +571,8 @@ render_callback (PangoLayout *layout,
                                (double)rect.height / PANGO_SCALE + lw);
               cairo_stroke (cr);
             }
-          while (pango_layout_iter_next_cluster (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_cluster (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -538,7 +587,7 @@ render_callback (PangoLayout *layout,
             {
               PangoRectangle rect;
 
-              pango_layout_iter_get_char_extents (iter, &rect);
+              pango_line_iter_get_char_extents (iter, &rect);
               cairo_rectangle (cr,
                                (double)rect.x / PANGO_SCALE - lw / 2,
                                (double)rect.y / PANGO_SCALE - lw / 2,
@@ -546,8 +595,8 @@ render_callback (PangoLayout *layout,
                                (double)rect.height / PANGO_SCALE + lw);
               cairo_stroke (cr);
             }
-          while (pango_layout_iter_next_cluster (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_cluster (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -560,29 +609,36 @@ render_callback (PangoLayout *layout,
           iter = pango_layout_get_iter (layout);
           do
             {
-              PangoLayoutRun *run;
+              PangoRun *run;
+              PangoItem *item;
+              const PangoAnalysis *analysis;
+              PangoGlyphString *glyphs;
               PangoRectangle rect;
               int x_pos, y_pos;
 
-              run = pango_layout_iter_get_run (iter);
+              run = pango_line_iter_get_run (iter);
               if (!run)
                 continue;
 
-              pango_layout_iter_get_run_extents (iter, NULL, &rect);
+              item = pango_run_get_item (run);
+              analysis = pango_item_get_analysis (item);
+              glyphs = pango_run_get_glyphs (run);
+
+              pango_line_iter_get_run_extents (iter, NULL, &rect);
 
               x_pos = rect.x;
-              y_pos = pango_layout_iter_get_run_baseline (iter);
+              y_pos = pango_line_iter_get_run_baseline (iter);
 
-              for (int i = 0; i < run->glyphs->num_glyphs; i++)
+              for (int i = 0; i < glyphs->num_glyphs; i++)
                 {
                   PangoRectangle extents;
 
-                  pango_font_get_glyph_extents (run->item->analysis.font,
-                                                run->glyphs->glyphs[i].glyph,
+                  pango_font_get_glyph_extents (pango_analysis_get_font (analysis),
+                                                glyphs->glyphs[i].glyph,
                                                 &extents, NULL);
 
-                  rect.x = x_pos + run->glyphs->glyphs[i].geometry.x_offset + extents.x;
-                  rect.y = y_pos + run->glyphs->glyphs[i].geometry.y_offset + extents.y;
+                  rect.x = x_pos + glyphs->glyphs[i].geometry.x_offset + extents.x;
+                  rect.y = y_pos + glyphs->glyphs[i].geometry.y_offset + extents.y;
                   rect.width = extents.width;
                   rect.height = extents.height;
 
@@ -594,16 +650,16 @@ render_callback (PangoLayout *layout,
                   cairo_stroke (cr);
 
                   cairo_arc (cr,
-                             (double) (x_pos + run->glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE,
-                             (double) (y_pos + run->glyphs->glyphs[i].geometry.y_offset) / PANGO_SCALE,
+                             (double) (x_pos + glyphs->glyphs[i].geometry.x_offset) / PANGO_SCALE,
+                             (double) (y_pos + glyphs->glyphs[i].geometry.y_offset) / PANGO_SCALE,
                              3.0, 0, 2*G_PI);
                   cairo_fill (cr);
 
-                  x_pos += run->glyphs->glyphs[i].geometry.width;
+                  x_pos += glyphs->glyphs[i].geometry.width;
                 }
             }
-          while (pango_layout_iter_next_run (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_run (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -618,40 +674,45 @@ render_callback (PangoLayout *layout,
           cairo_save (cr);
           cairo_set_source_rgba (cr, 1.0, 0.0, 1.0, 0.5);
 
-          attrs = pango_layout_get_log_attrs_readonly (layout, &n_attrs);
+          attrs = pango_layout_get_log_attrs (layout, &n_attrs);
 
           iter = pango_layout_get_iter (layout);
           do
             {
               PangoRectangle rect;
-              PangoLayoutRun *run;
+              PangoRun *run;
+              PangoItem *item;
+              PangoGlyphString *glyphs;
               const char *text, *start, *p;
               int x, y;
               gboolean trailing;
 
-              pango_layout_iter_get_run_extents (iter, NULL, &rect);
-              run = pango_layout_iter_get_run_readonly (iter);
+              pango_line_iter_get_run_extents (iter, NULL, &rect);
+              run = pango_line_iter_get_run (iter);
 
               if (!run)
                 continue;
 
+              item = pango_run_get_item (run);
+              glyphs = pango_run_get_glyphs (run);
+
               text = pango_layout_get_text (layout);
-              start = text + run->item->offset;
+              start = text + pango_item_get_char_offset (item);
 
               offset = g_utf8_strlen (text, start - text);
 
-              y = pango_layout_iter_get_run_baseline (iter);
+              y = pango_line_iter_get_run_baseline (iter);
 
               trailing = FALSE;
               p = start;
-              for (int i = 0; i <= run->item->num_chars; i++)
+              for (int i = 0; i <= pango_item_get_char_length (item); i++)
                 {
                   if (attrs[offset + i].is_cursor_position)
                     {
-                      pango_glyph_string_index_to_x_full (run->glyphs,
-                                                          text + run->item->offset,
-                                                          run->item->length,
-                                                          &run->item->analysis,
+                      pango_glyph_string_index_to_x_full (glyphs,
+                                                          text + pango_item_get_byte_offset (item),
+                                                          pango_item_get_byte_length (item),
+                                                          pango_item_get_analysis (item),
                                                           (PangoLogAttr *)attrs + offset,
                                                           p - start,
                                                           trailing,
@@ -670,7 +731,7 @@ render_callback (PangoLayout *layout,
                       g_free (s);
                    }
 
-                  if (i < run->item->num_chars)
+                  if (i < pango_item_get_char_length (item))
                     {
                       num++;
                       p = g_utf8_next_char (p);
@@ -680,8 +741,8 @@ render_callback (PangoLayout *layout,
 
                 }
             }
-          while (pango_layout_iter_next_run (iter));
-          pango_layout_iter_free (iter);
+          while (pango_line_iter_next_run (iter));
+          pango_line_iter_free (iter);
           cairo_restore (cr);
         }
 
@@ -698,7 +759,7 @@ render_callback (PangoLayout *layout,
           cairo_save (cr);
           cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 0.5);
 
-          attrs = pango_layout_get_log_attrs_readonly (layout, &n_attrs);
+          attrs = pango_layout_get_log_attrs (layout, &n_attrs);
 
           for (i = 0, p = text; i <= length; i++, p = g_utf8_next_char (p))
             {
@@ -707,7 +768,7 @@ render_callback (PangoLayout *layout,
               if (!attrs[i].is_cursor_position)
                 continue;
 
-              pango_layout_get_caret_pos (layout, p - text, &rect, NULL);
+              pango_lines_get_caret_pos (pango_layout_get_lines (layout), NULL, p - text, &rect, NULL);
 
               cairo_move_to (cr,
                              (double)rect.x / PANGO_SCALE + (double)rect.width / PANGO_SCALE - lw / 2,
@@ -910,6 +971,8 @@ pangocairo_view_get_option_group (const PangoViewer *klass G_GNUC_UNUSED)
   GOptionEntry entries[] =
   {
     {"annotate", 0, 0, G_OPTION_ARG_CALLBACK, parse_annotate_arg, annotate_arg_help, "FLAGS"},
+    { "font-file", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &opt_font_file, "Create a fontmap with this font", "FILE" },
+    { "userfont", 0, 0, G_OPTION_ARG_NONE, &opt_userfont, "Add userfont" },
     {NULL}
   };
   GOptionGroup *group;
