@@ -54,8 +54,6 @@ int opt_height = -1;
 int opt_indent = 0;
 int opt_spacing = 0;
 double opt_line_spacing = -1.0;
-gboolean opt_justify = 0;
-gboolean opt_justify_last_line = 0;
 int opt_runs = 1;
 PangoAlignment opt_align = PANGO_ALIGN_LEFT;
 PangoEllipsizeMode opt_ellipsize = PANGO_ELLIPSIZE_NONE;
@@ -131,9 +129,7 @@ make_layout(PangoContext *context,
 
   pango_layout_set_auto_dir (layout, opt_auto_dir);
   pango_layout_set_ellipsize (layout, opt_ellipsize);
-  pango_layout_set_justify (layout, opt_justify);
-  pango_layout_set_justify_last_line (layout, opt_justify_last_line);
-  pango_layout_set_single_paragraph_mode (layout, opt_single_par);
+  pango_layout_set_single_paragraph (layout, opt_single_par);
   pango_layout_set_wrap (layout, opt_wrap);
 
   font_description = pango_font_description_from_string (opt_font);
@@ -172,10 +168,11 @@ make_layout(PangoContext *context,
         pango_layout_set_spacing (layout, opt_spacing);
       else
         pango_layout_set_spacing (layout, (opt_spacing * opt_dpi * PANGO_SCALE + 36) / 72);
-      pango_layout_set_line_spacing (layout, 0.0);
+      pango_layout_set_line_height (layout, 0.0);
     }
+
   if (opt_line_spacing >= 0.0)
-    pango_layout_set_line_spacing (layout, (float)opt_line_spacing);
+    pango_layout_set_line_height (layout, (float)opt_line_spacing);
 
   align = opt_align;
   if (align != PANGO_ALIGN_CENTER &&
@@ -195,24 +192,19 @@ make_layout(PangoContext *context,
 out:
   if (opt_serialized_output)
     {
-      GError *error = NULL;
-
-      if (!pango_layout_write_to_file (layout,
-                                       PANGO_LAYOUT_SERIALIZE_CONTEXT|PANGO_LAYOUT_SERIALIZE_OUTPUT,
-                                       opt_serialized_output,
-                                       &error))
-        fail ("%s\n", error->message);
+      if (!pango_layout_write_to_file (layout, opt_serialized_output))
+        fail ("Failed to serialize\n");
     }
 
   return layout;
 }
 
-gchar *
+char *
 get_options_string (void)
 {
   PangoFontDescription *font_description = pango_font_description_from_string (opt_font);
-  gchar *font_name;
-  gchar *result;
+  char *font_name;
+  char *result;
 
   if (opt_waterfall)
     pango_font_description_unset_fields (font_description, PANGO_FONT_MASK_SIZE);
@@ -278,7 +270,8 @@ output_body (PangoLayout    *layout,
 	  pango_font_description_free (desc);
 	}
 
-      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
+      pango_lines_get_extents (pango_layout_get_lines (layout), NULL, &logical_rect);
+      pango_extents_to_pixels (&logical_rect, NULL);
 
       if (render_cb)
 	(*render_cb) (layout, x, y+*height, cb_context, cb_data);
@@ -347,7 +340,7 @@ do_output (PangoContext     *context,
       char *options_string = get_options_string ();
       pango_context_set_base_gravity (context, PANGO_GRAVITY_SOUTH);
       layout = make_layout (context, options_string, 10);
-      pango_layout_get_extents (layout, NULL, &rect);
+      pango_lines_get_extents (pango_layout_get_lines (layout), NULL, &rect);
 
       width = MAX (width, PANGO_PIXELS (rect.width));
       height += PANGO_PIXELS (rect.height);
@@ -420,35 +413,78 @@ do_output (PangoContext     *context,
 }
 
 static gboolean
-parse_enum (GType       type,
-	    int        *value,
-	    const char *name,
-	    const char *arg,
-	    gpointer    data G_GNUC_UNUSED,
-	    GError **error)
+parse_int (const char *word,
+           int        *out)
 {
-  char *possible_values = NULL;
-  gboolean ret;
+  char *end;
+  long val;
+  int i;
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  ret = pango_parse_enum (type,
-			  arg,
-			  value,
-			  FALSE,
-			  &possible_values);
-G_GNUC_END_IGNORE_DEPRECATIONS
+  if (word == NULL)
+    return FALSE;
 
-  if (!ret && error)
+  val = strtol (word, &end, 10);
+  i = val;
+
+  if (end != word && *end == '\0' && val >= 0 && val == i)
     {
-      g_set_error(error,
-		  G_OPTION_ERROR,
-		  G_OPTION_ERROR_BAD_VALUE,
-		  "Argument for %s must be one of %s",
-		  name,
-		  possible_values);
+      if (out)
+        *out = i;
+
+      return TRUE;
     }
 
-  g_free (possible_values);
+  return FALSE;
+}
+
+static gboolean
+parse_enum (GType       type,
+            int        *value,
+            const char *name,
+            const char *arg,
+            gpointer    data G_GNUC_UNUSED,
+            GError **error)
+{
+  GEnumClass *class = NULL;
+  gboolean ret = TRUE;
+  GEnumValue *v = NULL;
+
+  class = g_type_class_ref (type);
+
+  if (G_LIKELY (arg))
+    v = g_enum_get_value_by_nick (class, arg);
+
+  if (v)
+    {
+      if (G_LIKELY (value))
+        *value = v->value;
+    }
+  else if (!parse_int (arg, value))
+    {
+      ret = FALSE;
+      int i;
+      GString *s = g_string_new (NULL);
+
+      for (i = 0, v = g_enum_get_value (class, i);
+           v;
+           i++  , v = g_enum_get_value (class, i))
+        {
+          if (i)
+            g_string_append_c (s, '/');
+          g_string_append (s, v->value_nick);
+        }
+
+      g_set_error(error,
+                  G_OPTION_ERROR,
+                  G_OPTION_ERROR_BAD_VALUE,
+                  "Argument for %s must be one of %s",
+                  name,
+                  s->str);
+
+      g_string_free (s, TRUE);
+    }
+
+  g_type_class_unref (class);
 
   return ret;
 }
@@ -709,6 +745,7 @@ parse_margin (const char *name G_GNUC_UNUSED,
 {
   switch (sscanf (arg, "%d%*[ ,]%d%*[ ,]%d%*[ ,]%d", &opt_margin_t, &opt_margin_r, &opt_margin_b, &opt_margin_l))
   {
+    default:
     case 0:
     {
       g_set_error(error,
@@ -718,14 +755,16 @@ parse_margin (const char *name G_GNUC_UNUSED,
       return FALSE;
     }
     case 1: opt_margin_r = opt_margin_t;
+      G_GNUC_FALLTHROUGH;
     case 2: opt_margin_b = opt_margin_t;
+      G_GNUC_FALLTHROUGH;
     case 3: opt_margin_l = opt_margin_r;
   }
   return TRUE;
 }
 
 
-static gchar *
+static char *
 backends_to_string (void)
 {
   GString *backends = g_string_new (NULL);
@@ -737,7 +776,7 @@ backends_to_string (void)
 	g_string_append (backends, (*viewer)->id);
 	g_string_append_c (backends, '/');
       }
-  g_string_truncate (backends, MAX (0, (gint)backends->len - 1));
+  g_string_truncate (backends, MAX (0, (int)backends->len - 1));
 
   return g_string_free(backends,FALSE);
 }
@@ -756,7 +795,7 @@ backends_get_count (void)
 }
 
 
-static gchar *
+static char *
 backend_description (void)
 {
  GString *description  = g_string_new("Pango backend to use for rendering ");
@@ -790,7 +829,7 @@ parse_backend (const char *name G_GNUC_UNUSED,
     opt_viewer = *viewer;
   else
     {
-      gchar *backends = backends_to_string ();
+      char *backends = backends_to_string ();
 
       g_set_error(error,
 		  G_OPTION_ERROR,
@@ -822,9 +861,9 @@ show_version(const char *name G_GNUC_UNUSED,
 void
 parse_options (int argc, char *argv[])
 {
-  gchar *backend_options = backends_to_string ();
+  char *backend_options = backends_to_string ();
   GOptionFlags backend_flag = backends_get_count () > 1 ? 0 : G_OPTION_FLAG_HIDDEN;
-  gchar *backend_desc = backend_description ();
+  char *backend_desc = backend_description ();
   GOptionEntry entries[] =
   {
     {"no-auto-dir",	0, G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE,	&opt_auto_dir,
@@ -865,14 +904,10 @@ parse_options (int argc, char *argv[])
      "Subpixel order",                                      "rgb/bgr/vrgb/vbgr"},
     {"indent",		0, 0, G_OPTION_ARG_INT,				&opt_indent,
      "Width in points to indent paragraphs",			    "points"},
-    {"spacing",		0, 0, G_OPTION_ARG_INT,				&opt_spacing,
-     "Spacing in points between lines",			            "points"},
+    {"spacing",                0, 0, G_OPTION_ARG_INT,                         &opt_spacing,
+     "Spacing in points between lines",                                    "points"},
     {"line-spacing",	0, 0, G_OPTION_ARG_DOUBLE,		        &opt_line_spacing,
      "Spread factor for line height",			            "factor"},
-    {"justify",         0, 0, G_OPTION_ARG_NONE,                        &opt_justify,
-     "Stretch paragraph lines to be justified",                         NULL},
-    {"justify-last-line", 0, 0, G_OPTION_ARG_NONE,                      &opt_justify_last_line,
-     "Justify the last line of the paragraph",                          NULL},
     {"language",	0, 0, G_OPTION_ARG_STRING,			&opt_language,
      "Language to use for font selection",			    "en_US/etc"},
     {"margin",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_margin,
@@ -893,8 +928,8 @@ parse_options (int argc, char *argv[])
      "Angle at which to rotate results",			   "degrees"},
     {"runs",		'n', 0, G_OPTION_ARG_INT,			&opt_runs,
      "Run Pango layout engine this many times",			   "integer"},
-    {"single-par",	0, 0, G_OPTION_ARG_NONE,			&opt_single_par,
-     "Enable single-paragraph mode",					NULL},
+    {"single-par",     0, 0, G_OPTION_ARG_NONE,                        &opt_single_par,
+     "Enable single-paragraph mode",                                   NULL},
     {"text",		't', 0, G_OPTION_ARG_STRING,			&opt_text,
      "Text to display (instead of a file)",			    "string"},
     {"version",		0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &show_version,
